@@ -27,10 +27,17 @@
  ******************************************************************************
  *
  * $Log: cgiutil.c,v $
- * Revision 1.22.2.2  2006/02/03 18:57:51  sdlime
- * Argh! I mis-applied the patch for 1628 and didn't delete one line.
+ * Revision 1.26  2006/08/29 01:56:53  sdlime
+ * Fixed buffer overflow with POSTs and huge numbers of name/value pairs. Reduced MAX_PARAMS (now MS_MAX_CGI_PARAMS) from 10,000 to 100.
  *
- * Revision 1.22.2.1  2006/02/02 00:32:25  sdlime
+ * Revision 1.25  2006/05/08 20:28:43  frank
+ * force stdin into binary mode when reading from stdin on win32 (bug 1768)
+ *
+ * Revision 1.24  2006/02/18 21:35:35  hobu
+ * be explicit about the assignment when getting the
+ * CONTENT_TYPE on line 137
+ *
+ * Revision 1.23  2006/02/02 00:29:36  sdlime
  * Fixed bug with default content-type and POST requests. (bug 1628)
  *
  * Revision 1.22  2005/07/22 17:26:11  frank
@@ -54,7 +61,7 @@
 #include "cgiutil.h"
 #include "map.h"
 
-MS_CVSID("$Id: cgiutil.c,v 1.22.2.2 2006/02/03 18:57:51 sdlime Exp $")
+MS_CVSID("$Id: cgiutil.c,v 1.26 2006/08/29 01:56:53 sdlime Exp $")
 
 #define LF 10
 #define CR 13
@@ -64,6 +71,8 @@ static char *readPostBody( cgiRequestObj *request )
     char *data; 
     int data_max, data_len, chunk_size;
 
+    msIO_needBinaryStdin();
+    
 /* -------------------------------------------------------------------- */
 /*      If the length is provided, read in one gulp.                    */
 /* -------------------------------------------------------------------- */
@@ -136,38 +145,46 @@ int loadParams(cgiRequestObj *request){
 
     request->type = MS_POST_REQUEST;
 
-    if(s = getenv("CONTENT_TYPE"))
+    s = getenv("CONTENT_TYPE"); 
+    if (s != NULL)
       request->contenttype = strdup(s);
-    /* we've to set default content-type which is
-     * application/octet-stream according to
-     * W3 RFC 2626 section 7.2.1 */
+     /* we've to set default content-type which is
+      * application/octet-stream according to
+      * W3 RFC 2626 section 7.2.1 */
     else request->contenttype = strdup("application/octet-stream");
 
     post_data = readPostBody( request );
     if(strcmp(request->contenttype, "application/x-www-form-urlencoded")) 
-        request->postrequest = post_data;
-    else 
-    {
-        int data_len = strlen(post_data);
-        while( data_len > 0 && isspace(post_data[data_len-1]) )
-            post_data[--data_len] = '\0';
+      request->postrequest = post_data;
+    else {
+      int data_len = strlen(post_data);
+      while( data_len > 0 && isspace(post_data[data_len-1]) )
+        post_data[--data_len] = '\0';
             
-        while( post_data[0] )
-        {
-            request->ParamValues[m] = makeword(post_data,'&');
-            plustospace(request->ParamValues[m]);
-            unescape_url(request->ParamValues[m]);
-            request->ParamNames[m] = makeword(request->ParamValues[m],'=');
-            m++;
+      while( post_data[0] ) {
+        if(m >= MS_MAX_CGI_PARAMS) {
+          msIO_printf("Too many name/value pairs, aborting.\n");
+	  exit(0);
         }
-        free( post_data );
+
+        request->ParamValues[m] = makeword(post_data,'&');
+        plustospace(request->ParamValues[m]);
+        unescape_url(request->ParamValues[m]);
+        request->ParamNames[m] = makeword(request->ParamValues[m],'=');
+        m++;
+      }
+      free( post_data );
     }
   
-    /*check the QUERY_STRING even in the post request since it can contain 
-      information. Eg a wfs request with  */
+    /* check the QUERY_STRING even in the post request since it can contain 
+       information. Eg a wfs request with  */
     s = getenv("QUERY_STRING");
-    if (s){
+    if(s) {
       for(x=0;s[0] != '\0';x++) {       
+        if(m >= MS_MAX_CGI_PARAMS) {
+          msIO_printf("Too many name/value pairs, aborting.\n");
+          exit(0);
+        }
         request->ParamValues[m] = makeword(s,'&');
         plustospace(request->ParamValues[m]);
         unescape_url(request->ParamValues[m]);
@@ -193,6 +210,10 @@ int loadParams(cgiRequestObj *request){
       }
 
       for(x=0;s[0] != '\0';x++) {       
+        if(m >= MS_MAX_CGI_PARAMS) {
+          msIO_printf("Too many name/value pairs, aborting.\n");
+          exit(0);
+        }
         request->ParamValues[m] = makeword(s,'&');
         plustospace(request->ParamValues[m]);
         unescape_url(request->ParamValues[m]);
@@ -210,6 +231,10 @@ int loadParams(cgiRequestObj *request){
   s = getenv("HTTP_COOKIE");
   if(s != NULL) {    
     for(x=0;s[0] != '\0';x++) {
+      if(m >= MS_MAX_CGI_PARAMS) {
+	msIO_printf("Too many name/value pairs, aborting.\n");
+	exit(0);
+      }
       request->ParamValues[m] = makeword(s,';');
       plustospace(request->ParamValues[m]);
       unescape_url(request->ParamValues[m]);
