@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cgiutil.h,v 1.17.4.1 2006/03/19 17:58:13 tamas Exp $
+ * $Id: cgiutil.h,v 1.19 2006/08/29 01:56:53 sdlime Exp $
  *
  * Project:  MapServer
  * Purpose:  cgiRequestObj and CGI parsing utility related declarations.
@@ -28,8 +28,11 @@
  ******************************************************************************
  *
  * $Log: cgiutil.h,v $
- * Revision 1.17.4.1  2006/03/19 17:58:13  tamas
- * Hiding items not to be exposed to the mapscript interface (bug 1660)
+ * Revision 1.19  2006/08/29 01:56:53  sdlime
+ * Fixed buffer overflow with POSTs and huge numbers of name/value pairs. Reduced MAX_PARAMS (now MS_MAX_CGI_PARAMS) from 10,000 to 100.
+ *
+ * Revision 1.18  2006/02/22 05:04:34  sdlime
+ * Applied patch for bug 1660 to hide certain structures from Swig-based MapScript.
  *
  * Revision 1.17  2005/06/14 16:03:33  dan
  * Updated copyright date to 2005
@@ -54,7 +57,7 @@
 /*
 ** Misc. defines
 */
-#define MAX_PARAMS 10000
+#define MS_MAX_CGI_PARAMS 100
 
 enum MS_REQUEST_TYPE {MS_GET_REQUEST, MS_POST_REQUEST};
 
