@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: map.h,v 1.437.2.8 2006/03/30 23:40:50 hobu Exp $
+ * $Id: map.h,v 1.469 2006/10/04 14:54:49 dan Exp $
  *
  * Project:  MapServer
  * Purpose:  Primary MapServer include file.
@@ -28,290 +28,116 @@
  ******************************************************************************
  *
  * $Log: map.h,v $
- * Revision 1.437.2.8  2006/03/30 23:40:50  hobu
- * bump version to 4.8.3
+ * Revision 1.469  2006/10/04 14:54:49  dan
+ * Update for 4.10.0
  *
- * Revision 1.437.2.7  2006/03/22 22:08:29  hobu
- * Bump version to 4.8.2
+ * Revision 1.468  2006/09/27 21:57:02  dan
+ * Update for 4.10.0-rc1
  *
- * Revision 1.437.2.6  2006/03/19 17:58:13  tamas
- * Hiding items not to be exposed to the mapscript interface (bug 1660)
+ * Revision 1.467  2006/09/25 20:51:56  tamas
+ * Hiding labelitemindex, labelsizeitemindex, labelangleitemindex from the SWIG interface (bug 1906)
  *
- * Revision 1.437.2.5  2006/02/03 19:12:19  dan
- * Update for 4.8.1
+ * Revision 1.466  2006/09/23 15:23:34  frank
+ * Avoid c++ style comments.
  *
- * Revision 1.437.2.4  2006/02/02 16:29:37  dan
- * Update for 4.8.0 release
+ * Revision 1.465  2006/09/06 21:50:36  dan
+ * Update for 4.10.0-beta3
  *
- * Revision 1.437.2.3  2006/01/30 15:06:32  dan
+ * Revision 1.464  2006/09/01 02:30:15  sdlime
+ * Dan beat me to the bug 1428 fix. I took a bit futher by removing msLayerGetFilterString() from layerobject.c and refer to that in the mapscript getFilter/getFilterString methods.
+ *
+ * Revision 1.463  2006/08/31 20:48:47  dan
+ * Fixed MapScript getExpressionString() that was failing on expressions
+ * longer that 256 chars (SWIG) and 512 chars (PHP). Also moved all that
+ * code to a msGetExpressionString() in mapfile.c (bug 1428)
+ *
+ * Revision 1.462  2006/08/29 02:57:18  dan
+ * Prepare for 4.10.0-beta2
+ *
+ * Revision 1.461  2006/08/27 16:28:11  hobu
+ * remove mpatrol support (bug 1883).
+ *
+ * Revision 1.460  2006/08/26 17:25:05  frank
+ * added --disable-fast-nint, made ignore-missing off by default (bug 1716)
+ *
+ * Revision 1.459  2006/08/18 02:09:22  dan
+ * Updated for 4.10.0-beta1
+ *
+ * Revision 1.458  2006/08/17 20:57:46  tamas
+ * Made some object members immutable (bug 1803)
+ *
+ * Revision 1.457  2006/08/17 04:37:17  sdlime
+ * In keeping with naming conventions (like it or not) label->angle_follow becomes label->autofollow...
+ *
+ * Revision 1.456  2006/08/16 14:05:07  sdlime
+ * Removed any ambiguity with msCommifyString(). At the moment it only handles North American representaions of numbers (e.g. 2,345.678).
+ *
+ * Revision 1.455  2006/06/27 13:35:45  sdlime
+ * Initial version of GEOS C-API code including SWIG bindings for most everything useful.
+ *
+ * Revision 1.454  2006/06/08 07:51:11  umberto
+ * webObj constructor and destructor (bug 1798)
+ *
+ * Revision 1.453  2006/06/04 16:46:10  hobu
+ * trim log so it only goes back to the beginning of 4.8
+ *
+ * Revision 1.452  2006/06/04 16:39:32  hobu
+ * remove old SDE Raster cruft (was commented out anyway)
+ *
+ * Revision 1.451  2006/06/01 20:08:04  dan
+ * Added missing prototype for msGenerateEncryptionKey() (bug 1792)
+ *
+ * Revision 1.450  2006/06/01 19:56:31  dan
+ * Added ability to encrypt tokens (passwords, etc.) in database connection
+ * strings (MS-RFC-18, bug 1792)
+ *
+ * Revision 1.449  2006/05/26 21:46:27  tamas
+ * Moving layerObj.sameconnection and msCheckConnection() internal to the MYGIS data provider.
+ *
+ * Revision 1.448  2006/05/19 20:53:27  dan
+ * Use lp->layerinfo for OGR connections (instead of ogrlayerinfo) (bug 331)
+ *
+ * Revision 1.447  2006/05/11 14:10:35  sdlime
+ * Added GEOS Union, difference and intersection methods.
+ *
+ * Revision 1.446  2006/04/27 04:05:17  sdlime
+ * Initial support for relative coordinates. (bug 1547)
+ *
+ * Revision 1.445  2006/03/22 23:31:20  sdlime
+ * Applied latest patch for curved labels. (bug 1620)
+ *
+ * Revision 1.444  2006/02/22 05:04:34  sdlime
+ * Applied patch for bug 1660 to hide certain structures from Swig-based MapScript.
+ *
+ * Revision 1.443  2006/02/18 20:59:12  sdlime
+ * Initial code for curved labels. (bug 1620)
+ *
+ * Revision 1.442  2006/01/31 17:09:28  sdlime
+ * Added function to 'commify' a number stored as a string. (supports bug 1636)
+ *
+ * Revision 1.441  2006/01/30 15:06:57  dan
  * Treat classindex as an int instead of a char in resultCacheMemberObj to
  * prevent problems with more than 128 classes (bug 1633)
  *
- * Revision 1.437.2.2  2006/01/25 22:55:17  dan
- * Update for 4.8.0-rc3
+ * Revision 1.440  2006/01/20 06:06:53  sdlime
+ * Added contains operator to mapgeos.cpp and the Swig-based MapScript interface.
  *
- * Revision 1.437.2.1  2006/01/16 20:41:22  sdlime
+ * Revision 1.439  2006/01/16 20:21:18  sdlime
  * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
  *
+ * Revision 1.438  2006/01/10 03:10:32  dan
+ * Update version, this is now 4.9 (dev)
+ *
  * Revision 1.437  2006/01/10 03:01:01  dan
  * Update for 4.8.0-rc2
  *
- * Revision 1.436  2005/12/23 06:20:50  sdlime
- * Updated files for 4.8.0-rc1.
- *
- * Revision 1.435  2005/12/16 13:28:33  dan
- * Update version to 4.8.0-beta3
- *
- * Revision 1.434  2005/12/14 19:06:31  sdlime
- * Added browseformat and legendformat to webObj. (bug 1518)
- *
- * Revision 1.433  2005/12/08 19:05:32  hobu
- * switch off SDE raster support
- *
- * Revision 1.432  2005/11/24 04:38:57  dan
- * Update version for 4.8.0-beta2
- *
- * Revision 1.431  2005/11/22 22:14:43  sean
- * expose maximum length of layer, class, and style arrays in mapscript (bug 1522)
- *
- * Revision 1.430  2005/11/04 20:58:35  dan
- * Updated MS_VERSION to 4.8.0-beta1
- *
- * Revision 1.429  2005/11/01 05:35:50  frank
- * added preliminary implementation of OGR based WKT translation, still untested
- *
- * Revision 1.428  2005/10/30 05:05:07  sdlime
- * Initial support for WKT via GEOS. The reader is only integrated via the map file reader, with MapScript, CGI and URL support following ASAP. (bug 1466)
- *
- * Revision 1.427  2005/10/29 02:03:43  jani
- * MS RFC 8: Pluggable External Feature Layer Providers (bug 1477).
- *
- * Revision 1.426  2005/10/28 02:08:14  frank
- * a few mapogr.cpp funcs used from WFS code, exposed in map.h
- *
- * Revision 1.425  2005/10/28 01:09:41  jani
- * MS RFC 3: Layer vtable architecture (bug 1477)
- *
- * Revision 1.424  2005/10/20 19:37:03  frank
- * added msAddPointToLine
- *
- * Revision 1.423  2005/10/18 03:10:45  frank
- * added msShapeDeleteLine
- *
- * Revision 1.422  2005/10/14 05:04:12  sdlime
- * Added msRotateSymbol(), changed freeSymbol to a public function called msFreeSymbol() in mapsymbol.c. Added to map.h as well.
- *
- * Revision 1.421  2005/09/23 21:18:58  assefa
- * Remove call to ming.h inside map.h (Bug 1479)
- *
- * Revision 1.420  2005/08/25 14:20:16  sdlime
- * Applied patch for bug 1440.
- *
- * Revision 1.419  2005/07/16 19:07:38  jerryp
- * Bug 1420: PostGIS connector no longer needs two layer close functions.
- *
- * Revision 1.418  2005/07/13 19:35:08  julien
- * Bug 1381: Support for case-insensitive Expression
- *
- * Revision 1.417  2005/06/29 14:11:50  dan
- * Set MS_VERSION to 4.7
- *
- * Revision 1.416  2005/06/14 18:09:52  dan
- * Ready for 4.6.0 release
- *
- * Revision 1.415  2005/06/14 16:03:33  dan
- * Updated copyright date to 2005
- *
- * Revision 1.414  2005/06/09 18:27:51  dan
- * Ready for 4.6.0-rc1
- *
- * Revision 1.413  2005/05/27 21:19:12  dan
- * Ready for 4.6.0-beta3
- *
- * Revision 1.412  2005/05/27 15:00:12  dan
- * New regex wrappers to solve issues with previous version (bug 1354)
- *
- * Revision 1.411  2005/05/25 21:02:05  dan
- * Added a regex wrapper, allowing MapServer to build with PHP compiled
- * with its builtin regex (bug 1354)
- *
- * Revision 1.410  2005/05/19 05:57:08  sdlime
- * Added prototype for msGEOSFreeGeometry.
- *
- * Revision 1.409  2005/05/19 04:09:34  sdlime
- * Removed the LINE_VERT_THRESHOLD test (bug 564) from PDF/SWF/SVG/imagemap drivers.
- *
- * Revision 1.408  2005/05/13 13:14:47  frank
- * remove MAX/MIN macros, using MS_MAX/MS_MIN
- *
- * Revision 1.407  2005/05/11 14:15:16  dan
- * Updated version to 4.6.0-beta2
- *
- * Revision 1.406  2005/05/10 15:00:19  sean
- * fonts hash now properly exposed in mapscript (bug 1345)
- *
- * Revision 1.405  2005/04/26 23:01:36  dan
- * Updated for 4.6.0-beta1 release
- *
- * Revision 1.404  2005/04/25 06:41:55  sdlime
- * Applied Bill's newest gradient patch, more concise in the mapfile and potential to use via MapScript.
- *
- * Revision 1.403  2005/04/15 17:10:36  sdlime
- * Applied Bill Benko's patch for bug 1305, gradient support.
- *
- * Revision 1.402  2005/04/07 17:23:16  assefa
- * Remove #ifdef USE_SVG. It was added during development.
- *
- * Revision 1.401  2005/03/25 05:43:20  frank
- * added msAddLineDirectly()
- *
- * Revision 1.400  2005/03/24 22:25:32  frank
- * Implement assembly for MS_NINT on win32 and linux/i86.
- *
- * Revision 1.399  2005/03/03 15:31:40  assefa
- * Add compressed parameter in the SVG object.
- *
- * Revision 1.398  2005/02/23 04:40:17  sdlime
- * Added wrapper for creating convex hulls to GEOS support. Added to MapScript as well.
- *
- * Revision 1.397  2005/02/22 07:40:27  sdlime
- * A bunch of updates to GEOS integration. Can move many primatives between MapServer and GEOS, still need to do collections (e.g. multi-point/line/polygon). Added buffer method to mapscript (mapscript/shape.i).
- *
- * Revision 1.396  2005/02/18 03:06:44  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.395  2005/02/09 21:51:17  frank
- * added msForceTmpFileBase and -tmpbase mapserv switch
- *
- * Revision 1.394  2005/02/09 20:43:49  assefa
- * Add SVG utility function msSaveImagetoFpSVG.
- *
- * Revision 1.393  2005/02/06 19:31:32  sean
- * msSetup() function calls gdFontCacheSetup().  msSetup should be called immediately on mapscript module import to avoid race condition with font cache (bug 1203).
- *
- * Revision 1.392  2005/02/03 00:06:57  assefa
- * Add SVG function prototypes and related calls.
- *
- * Revision 1.391  2005/01/28 06:16:53  sdlime
- * Applied patch to make function prototypes ANSI C compliant. Thanks to Petter Reinholdtsen. This fixes but 1181.
- *
- * Revision 1.390  2005/01/26 15:30:52  hobu
- * Add drawSDE prototype for experimental
- * SDE raster support
- *
- * Revision 1.389  2005/01/26 06:14:53  sdlime
- * Added style maxwidth/minwidth read/write/copy.
- *
- * Revision 1.388  2005/01/12 21:11:21  frank
- * removed LABELS_ROTATE_WITH_MAP, rotate labels if angle!=0 or labelangleitem
- *
- * Revision 1.387  2005/01/11 00:24:07  frank
- * added labelObj arg to msAddLabel()
- *
- * Revision 1.386  2005/01/03 03:01:56  sdlime
- * Changed my mind, thickness becomes width in a style object.
- *
- * Revision 1.385  2004/12/22 03:57:42  sdlime
- * Added thickness related parameters to the styleObj in map.h.
- *
- * Revision 1.384  2004/12/14 21:30:43  sdlime
- * Moved functions to build lists of inner and outer rings to mapprimitive.c from mapgml.c. They are needed to covert between MapServer polygons and GEOS gemometries (bug 771).
- *
- * Revision 1.383  2004/11/30 20:02:03  dan
- * Set version to 4.5 (this is the new dev version)
- *
- * Revision 1.382  2004/11/29 21:33:53  dan
- * Update for v4.4.0
- *
- * Revision 1.381  2004/11/22 15:51:52  dan
- * Updated MS_VERSION for 4.4.0-beta3
- *
- * Revision 1.380  2004/11/22 03:43:54  sdlime
- * Added tests to mimimize the threat of recursion problems when evaluating LAYER REQUIRES or LABELREQUIRES expressions. Note that via MapScript it is possible to circumvent that test by defining layers with problems after running prepareImage. Other things crop up in that case too (symbol scaling dies) so it should be considered bad programming practice.
- *
- * Revision 1.379  2004/11/19 03:59:19  sean
- * Fix to msSymbolSetImageGD so that pixmap transparency is preserved.  Renamed
- * the msGDSetImage and GetImage to msSymbolSetImageGD and msSymbolGetImageGD each
- * with a symbolObj* as the first argument (bug 1074).
- *
- * Revision 1.378  2004/11/18 16:12:02  sean
- * exposed MS_GD_ALPHA to swig
- *
- * Revision 1.377  2004/11/17 23:53:07  assefa
- * Advertize only gd and gdal formats for wms capabilities (Bug 455).
- *
- * Revision 1.376  2004/11/16 21:57:49  dan
- * Final pass at updating WMS/WFS client/server interfaces to lookup "ows_*"
- * metadata in addition to default "wms_*"/"wfs_*" metadata (bug 568)
- *
- * Revision 1.375  2004/11/16 14:34:26  frank
- * Avoid warning caused by last comment.
- *
- * Revision 1.374  2004/11/16 04:31:17  dan
- * Re-enabled building PHP MapScript using PHP's bundled regex/ *.o. This is
- * needed to build in an environment with PHP configured as an Apache DSO
- * (bugs 990, 520)
- *
- * Revision 1.373  2004/11/15 20:35:02  dan
- * Added msLayerIsOpen() to all vector layer types (bug 1051)
- *
- * Revision 1.372  2004/11/12 20:23:16  frank
- * include z and m in formatted pointObj, fmt is const!
- *
- * Revision 1.371  2004/11/10 19:22:23  sean
- * Comment out msImageLoadGDStream and pipe all gd image input through existing
- * msImageLoadGDCtx function.  Moved Frank's recent transparency and interlacing
- * work to msImageLoadGDCtx and moved the image format detection up into
- * msImageLoadGD.  All this to avoid passing a FILE* to GD (bug 1047).
- *
- * Revision 1.370  2004/11/10 17:51:50  sean
- * copy fileIOCtx code from gd_io_file.c to end of mapgd.c, renaming the struct
- * allocation function to msNewGDFileCtx (bug 1047).
- *
- * Revision 1.369  2004/11/10 16:40:33  sean
- * eliminate msSaveImageStreamGD and instead pipe GD output through new
- * msSaveImageGDCtx function.  This avoids passing a FILE* to GD (bug 1047).
- *
- * Revision 1.368  2004/11/05 16:29:40  assefa
- * add function msGetOutputFormatMimeListGD.
- *
- * Revision 1.367  2004/11/04 22:01:11  frank
- * Removed the msDrawWMSLayerPDF declaration.  It depends on stuff we don't have
- * in map.h.
- *
- * Revision 1.366  2004/11/04 21:45:28  frank
- * Added msDrawWMSLayerPDF() prototype.
- *
- * Revision 1.365  2004/11/03 21:04:44  dan
- * Update for 4.4.0-beta2
- *
- * Revision 1.364  2004/10/28 18:16:16  dan
- * Fixed WMS GetLegendGraphic which was returning an exception (GD error)
- * when requested layer was out of scale (bug 1006)
- *
- * Revision 1.363  2004/10/28 04:59:15  sdlime
- * Increased MS_MAXSYMBOLSIZE constant from 100 to 500. This value is used as a default within styleObj structures. This allows for larger markers to be draw, particularly useful with reference maps where you want a image sized crosshair.
- *
- * Revision 1.362  2004/10/26 15:54:35  sean
- * replace empty arg lists with ( void ) to eliminate swig warnings, and move
- * MS_DLL_EXPORT symbol ahead of function types.
- *
- * Revision 1.361  2004/10/22 04:03:31  dan
- * Ready for 4.4.0-beta1
- *
- * Revision 1.360  2004/10/21 19:19:44  assefa
- * Add utility function trimLeft.
- *
- * Revision 1.359  2004/10/21 04:30:54  frank
- * Added standardized headers.  Added MS_CVSID().
- *
  */
 
 
 #ifndef MAP_H
 #define MAP_H
 
-/* $Id: map.h,v 1.437.2.8 2006/03/30 23:40:50 hobu Exp $
+/* $Id: map.h,v 1.469 2006/10/04 14:54:49 dan Exp $
 **
 ** Main includes. If a particular header was needed by several .c files then
 ** I just put it here. What the hell, it works and it's all right here. -SDL-
@@ -348,11 +174,6 @@ typedef long ms_int32;
 typedef int ms_int32;
 #endif
     
-
-#ifdef USE_MPATROL
-#include "mpatrol.h"
-#endif
-
 #include "maperror.h"
 #include "mapprimitive.h"
 #include "mapshape.h"
@@ -370,7 +191,6 @@ typedef int ms_int32;
 #include <pdflib.h>
 #endif
 
-
 #include <sys/types.h> /* regular expression support */
 
 /* The regex lib from the system and the regex lib from PHP needs to
@@ -394,7 +214,7 @@ extern "C" {
 
 /* General defines, wrapable */
 
-#define MS_VERSION "4.8.3"
+#define MS_VERSION "4.10.0"
 
 #define MS_TRUE 1 /* logical control variables */
 #define MS_FALSE 0
@@ -485,12 +305,15 @@ extern "C" {
 #define MS_ABS(a) (((a)<0) ? -(a) : (a))
 #define MS_SGN(a) (((a)<0) ? -1 : 1)
 
+#define MS_NINT_GENERIC(x) ((x) >= 0.0 ? ((long) ((x)+.5)) : ((long) ((x)-.5)))
+
 /* see http://mega-nerd.com/FPcast/ for some discussion of fast
    conversion to nearest int.  We avoid lrint() for now because it
    would be hard to include math.h "properly". */
 
-#if defined(WE_HAVE_THE_C99_LRINT)
+#if defined(WE_HAVE_THE_C99_LRINT) && !defined(USE_GENERIC_MS_NINT)
 #   define MS_NINT(x) lrint(x)
+/*#   define MS_NINT(x) lround(x) */
 #elif defined(_MSC_VER) && defined(_WIN32) && !defined(USE_GENERIC_MS_NINT)
     static __inline long int MS_NINT (double flt) 
     {	int intgr;
@@ -512,7 +335,7 @@ extern "C" {
         return __lrintres;
     }
 #else
-#  define MS_NINT(x)      ((x) >= 0.0 ? ((long) ((x)+.5)) : ((long) ((x)-.5)))
+#  define MS_NINT(x)      MS_NINT_GENERIC(x)
 #endif
 
 
@@ -569,31 +392,32 @@ extern "C" {
 #define MS_3PI2  4.71238898038468985769  /* (3 * MS_PI2) */
 #define MS_2PI   6.28318530717958647693  /* (2 * MS_PI) */
 
+#define MS_ENCRYPTION_KEY_SIZE  16   /* Key size: 128 bits = 16 bytes */
+
 #endif
 
 /* General enumerated types - needed by scripts */
-    enum MS_FILE_TYPE {MS_FILE_MAP, MS_FILE_SYMBOL};
-    enum MS_UNITS {MS_INCHES, MS_FEET, MS_MILES, MS_METERS, MS_KILOMETERS, MS_DD, MS_PIXELS};
-    enum MS_SHAPE_TYPE {MS_SHAPE_POINT, MS_SHAPE_LINE, MS_SHAPE_POLYGON, MS_SHAPE_NULL};
-    enum MS_LAYER_TYPE {MS_LAYER_POINT, MS_LAYER_LINE, MS_LAYER_POLYGON, MS_LAYER_RASTER, MS_LAYER_ANNOTATION, MS_LAYER_QUERY, MS_LAYER_CIRCLE, MS_LAYER_TILEINDEX};
-    enum MS_FONT_TYPE {MS_TRUETYPE, MS_BITMAP};
-    enum MS_LABEL_POSITIONS {MS_UL, MS_LR, MS_UR, MS_LL, MS_CR, MS_CL, MS_UC, MS_LC, MS_CC, MS_AUTO, MS_XY}; /* arrangement matters for auto placement, don't change it */
-    enum MS_BITMAP_FONT_SIZES {MS_TINY , MS_SMALL, MS_MEDIUM, MS_LARGE, MS_GIANT};
-    enum MS_QUERYMAP_STYLES {MS_NORMAL, MS_HILITE, MS_SELECTED};
-    enum MS_CONNECTION_TYPE {MS_INLINE, MS_SHAPEFILE, MS_TILED_SHAPEFILE, MS_SDE, MS_OGR, MS_UNUSED_1, MS_POSTGIS, MS_WMS, MS_ORACLESPATIAL, MS_WFS, MS_GRATICULE, MS_MYGIS, MS_RASTER, MS_PLUGIN };
-    enum MS_JOIN_CONNECTION_TYPE {MS_DB_XBASE, MS_DB_CSV, MS_DB_MYSQL, MS_DB_ORACLE, MS_DB_POSTGRES};
-    enum MS_JOIN_TYPE {MS_JOIN_ONE_TO_ONE, MS_JOIN_ONE_TO_MANY};
+enum MS_FILE_TYPE {MS_FILE_MAP, MS_FILE_SYMBOL};
+enum MS_UNITS {MS_INCHES, MS_FEET, MS_MILES, MS_METERS, MS_KILOMETERS, MS_DD, MS_PIXELS, MS_PERCENTAGES};
+enum MS_SHAPE_TYPE {MS_SHAPE_POINT, MS_SHAPE_LINE, MS_SHAPE_POLYGON, MS_SHAPE_NULL};
+enum MS_LAYER_TYPE {MS_LAYER_POINT, MS_LAYER_LINE, MS_LAYER_POLYGON, MS_LAYER_RASTER, MS_LAYER_ANNOTATION, MS_LAYER_QUERY, MS_LAYER_CIRCLE, MS_LAYER_TILEINDEX};
+enum MS_FONT_TYPE {MS_TRUETYPE, MS_BITMAP};
 
-    enum MS_CAPS_JOINS_AND_CORNERS {MS_CJC_NONE, MS_CJC_BEVEL, MS_CJC_BUTT, MS_CJC_MITER, MS_CJC_ROUND, MS_CJC_SQUARE, MS_CJC_TRIANGLE}; 
+#define MS_POSITIONS_LENGTH 12
+enum MS_POSITIONS_ENUM {MS_UL=101, MS_LR, MS_UR, MS_LL, MS_CR, MS_CL, MS_UC, MS_LC, MS_CC, MS_AUTO, MS_XY, MS_FOLLOW}; /* arrangement matters for auto placement, don't change it.  Added MS_FOLLOW for bug #1620 implementation */
 
-    enum MS_RETURN_VALUE {MS_SUCCESS, MS_FAILURE, MS_DONE};
-
-    enum MS_IMAGEMODE { MS_IMAGEMODE_PC256, MS_IMAGEMODE_RGB, MS_IMAGEMODE_RGBA, MS_IMAGEMODE_INT16, MS_IMAGEMODE_FLOAT32, MS_IMAGEMODE_BYTE, MS_IMAGEMODE_NULL };
+enum MS_BITMAP_FONT_SIZES {MS_TINY , MS_SMALL, MS_MEDIUM, MS_LARGE, MS_GIANT};
+enum MS_QUERYMAP_STYLES {MS_NORMAL, MS_HILITE, MS_SELECTED};
+enum MS_CONNECTION_TYPE {MS_INLINE, MS_SHAPEFILE, MS_TILED_SHAPEFILE, MS_SDE, MS_OGR, MS_UNUSED_1, MS_POSTGIS, MS_WMS, MS_ORACLESPATIAL, MS_WFS, MS_GRATICULE, MS_MYGIS, MS_RASTER, MS_PLUGIN };
+enum MS_JOIN_CONNECTION_TYPE {MS_DB_XBASE, MS_DB_CSV, MS_DB_MYSQL, MS_DB_ORACLE, MS_DB_POSTGRES};
+enum MS_JOIN_TYPE {MS_JOIN_ONE_TO_ONE, MS_JOIN_ONE_TO_MANY};
 
+enum MS_CAPS_JOINS_AND_CORNERS {MS_CJC_NONE, MS_CJC_BEVEL, MS_CJC_BUTT, MS_CJC_MITER, MS_CJC_ROUND, MS_CJC_SQUARE, MS_CJC_TRIANGLE}; 
+enum MS_RETURN_VALUE {MS_SUCCESS, MS_FAILURE, MS_DONE};
+enum MS_IMAGEMODE { MS_IMAGEMODE_PC256, MS_IMAGEMODE_RGB, MS_IMAGEMODE_RGBA, MS_IMAGEMODE_INT16, MS_IMAGEMODE_FLOAT32, MS_IMAGEMODE_BYTE, MS_IMAGEMODE_NULL };
 
 #define MS_FILE_DEFAULT MS_FILE_MAP   
 
-
 #ifndef SWIG
 /* Filter object */    
 typedef enum 
@@ -623,9 +447,16 @@ typedef struct _FilterNode
 
       
 }FilterEncodingNode;
-#endif /*SWIG*/
 
 
+/* Label path object - used to hold path and bounds of curved labels - Bug #1620 implementation. */
+typedef struct {
+  multipointObj path;
+  shapeObj bounds;
+  double *angles;
+} labelPathObj;
+#endif /*SWIG*/
+
 /* FONTSET OBJECT - used to hold aliases for TRUETYPE fonts */
     typedef struct {
 #ifdef SWIG
@@ -633,10 +464,11 @@ typedef struct _FilterNode
 #endif
         char *filename; 
         int numfonts;
+        hashTableObj fonts;
 #ifdef SWIG
         %mutable;
 #endif
-        hashTableObj fonts;
+
 #ifndef SWIG
         struct map_obj *map;
 #endif
@@ -758,6 +590,7 @@ typedef struct _FilterNode
 
         double angle;
         int autoangle; /* true or false */
+        int autofollow;  /* true or false, bug #1620 implementation */
 
         int buffer; /* space to reserve around a label */
 
@@ -774,6 +607,7 @@ typedef struct _FilterNode
   int force; /* labels *must* be drawn */
 
   char *encoding;
+
 } labelObj;
 
 /* WEB OBJECT - holds parameters for a mapserver/mapscript interface */
@@ -781,7 +615,13 @@ typedef struct {
   char *log;
   char *imagepath, *imageurl;
 
+#ifdef SWIG
+%immutable;
+#endif /* SWIG */
   struct map_obj *map;
+#ifdef SWIG
+%mutable;
+#endif /* SWIG */
 
 #ifndef __cplusplus
   char *template;
@@ -799,7 +639,13 @@ typedef struct {
   char *legendformat;
   char *browseformat;
 
+#ifdef SWIG
+%immutable;
+#endif /* SWIG */
   hashTableObj metadata;
+#ifdef SWIG
+%mutable;
+#endif /* SWIG */
 
 } webObj;
 
@@ -861,7 +707,13 @@ typedef struct class_obj{
 
   int numstyles;
 
+#ifdef SWIG
+%immutable;
+#endif /* SWIG */
   labelObj label;
+#ifdef SWIG
+%mutable;
+#endif /* SWIG */
 
   char *name; /* should be unique within a layer */
   char *title; /* used for legend labeling */
@@ -878,10 +730,22 @@ typedef struct class_obj{
 
   int type;
 
+#ifdef SWIG
+%immutable;
+#endif /* SWIG */
   hashTableObj metadata;
+#ifdef SWIG
+%mutable;
+#endif /* SWIG */
 
   double minscale, maxscale;
+#ifdef SWIG
+%immutable;
+#endif /* SWIG */
   struct layer_obj *layer;
+#ifdef SWIG
+%mutable;
+#endif /* SWIG */
   int debug;
 
   char *keyimage;
@@ -910,6 +774,9 @@ typedef struct {
   shapeObj *poly; /* label bounding box */
 
   int status; /* has this label been drawn or not */
+
+  labelPathObj *labelpath;  /* Path & bounds of curved labels.  Bug #1620 implementation */
+  
 } labelCacheMemberObj;
 
 typedef struct {
@@ -987,7 +854,13 @@ typedef struct {
   int markersize;
   int minboxsize;
   int maxboxsize;
+#ifdef SWIG
+%immutable;
+#endif /* SWIG */
   struct map_obj *map;
+#ifdef SWIG
+%mutable;
+#endif /* SWIG */
 } referenceMapObj;
 
 /* SCALEBAR OBJECT */
@@ -1013,7 +886,13 @@ typedef struct {
 /* LEGEND OBJECT */
 typedef struct {
   colorObj imagecolor;
+#ifdef SWIG
+        %immutable;
+#endif
   labelObj label;
+#ifdef SWIG
+        %mutable;
+#endif
   int keysizex, keysizey;
   int keyspacingx, keyspacingy;
   colorObj outlinecolor; /* Color of outline of box, -1 for no outline */
@@ -1030,7 +909,13 @@ typedef struct {
 #else /* __cplusplus */
    char *_template;
 #endif /* __cplusplus */
+#ifdef SWIG
+%immutable;
+#endif /* SWIG */
   struct map_obj *map;
+#ifdef SWIG
+%mutable;
+#endif /* SWIG */
 } legendObj;
 
 #ifndef SWIG
@@ -1063,6 +948,7 @@ typedef struct
 
 struct layerVTable;
 typedef struct layerVTable layerVTableObj;
+
 #endif /*SWIG*/
 
 /* LAYER OBJECT - basic unit of a map */
@@ -1125,7 +1011,9 @@ typedef struct layer_obj {
   int labelcache, postlabelcache; /* on or off */
 
   char *labelitem, *labelsizeitem, *labelangleitem;
+#ifndef SWIG
   int labelitemindex, labelsizeitemindex, labelangleitemindex;
+#endif /* not SWIG */
 
   char *tileitem;
   char *tileindex; /* layer index file for tiling support */
@@ -1151,10 +1039,8 @@ typedef struct layer_obj {
 #ifndef SWIG
   layerVTableObj *vtable;
 
-  struct layer_obj *sameconnection;
   /* SDL has converted OracleSpatial, SDE, Graticules, MyGIS */
   void *layerinfo; /* all connection types should use this generic pointer to a vendor specific structure */
-  void *ogrlayerinfo; /* For OGR layers, will contain a msOGRLayerInfo struct */
   void *wfslayerinfo; /* For WFS layers, will contain a msWFSLayerInfo struct */
 #endif /* not SWIG */
 
@@ -1183,8 +1069,14 @@ typedef struct layer_obj {
   char *requires; /* context expressions, simple enough to not use expressionObj */
   char *labelrequires;
 
+#ifdef SWIG
+%immutable;
+#endif /* SWIG */
   hashTableObj metadata;
-  
+#ifdef SWIG
+%mutable;
+#endif /* SWIG */
+
   int transparency; /* transparency value 0-100 */
   
   int dump;
@@ -1224,14 +1116,14 @@ typedef struct map_obj{ /* structure for a map */
 %immutable;
 #endif /* SWIG */
   int numlayers; /* number of layers in mapfile */
-#ifdef SWIG
-%mutable;
-#endif /* SWIG */
 
   symbolSetObj symbolset;
   fontSetObj fontset;
 
   labelCacheObj labelcache; /* we need this here so multiple feature processors can access it */
+#ifdef SWIG
+%mutable;
+#endif /* SWIG */
 
   int transparent; /* TODO - Deprecated */
   int interlace; /* TODO - Deprecated */
@@ -1240,6 +1132,7 @@ typedef struct map_obj{ /* structure for a map */
   rectObj extent; /* map extent array */
   double cellsize; /* in map units */
 
+
 #ifndef SWIG
   geotransformObj gt; /* rotation / geotransform */
 #endif /*SWIG*/
@@ -1274,6 +1167,9 @@ typedef struct map_obj{ /* structure for a map */
   projectionObj latlon; /* geographic projection definition */
 #endif /* not SWIG */
 
+#ifdef SWIG
+%immutable;
+#endif /* SWIG */  
   referenceMapObj reference;
   scalebarObj scalebar;
   legendObj legend;
@@ -1281,6 +1177,9 @@ typedef struct map_obj{ /* structure for a map */
   queryMapObj querymap;
 
   webObj web;
+#ifdef SWIG
+%mutable;
+#endif /* SWIG */
 
   int *layerorder;
 
@@ -1288,7 +1187,19 @@ typedef struct map_obj{ /* structure for a map */
 
   char *datapattern, *templatepattern;   
 
+#ifdef SWIG
+%immutable;
+#endif /* SWIG */
   hashTableObj configoptions;
+#ifdef SWIG
+%mutable;
+#endif /* SWIG */
+
+#ifndef SWIG
+  /* Private encryption key information - see mapcrypto.c */
+  int encryption_key_loaded;        /* MS_TRUE once key has been loaded */
+  unsigned char encryption_key[MS_ENCRYPTION_KEY_SIZE]; /* 128bits encryption key */
+#endif
 } mapObj;
 
 
@@ -1319,11 +1230,11 @@ typedef struct {
 #endif
   int width, height;
   char *imagepath, *imageurl;
+
+  outputFormatObj *format;
 #ifdef SWIG
 %mutable;
 #endif
-
-  outputFormatObj *format;
   int renderer;
   int size;
 
@@ -1383,6 +1294,7 @@ struct layerVTable {
 };
 #endif /*SWIG*/
 
+
 /* Function prototypes, wrapable */
 MS_DLL_EXPORT int msSaveImage(mapObj *map, imageObj *img, char *filename);
 MS_DLL_EXPORT void msFreeImage(imageObj *img);
@@ -1392,7 +1304,6 @@ MS_DLL_EXPORT void msCleanup(void);
 /* Function prototypes, not wrapable */
 
 #ifndef SWIG
-/*int msDrawSDELayer(mapObj *map, layerObj *layer, gdImagePtr img);  in mapsde.c */
 
 /*
 ** helper functions not part of the general API but needed in
@@ -1420,6 +1331,8 @@ MS_DLL_EXPORT int initStyle(styleObj *style);
 MS_DLL_EXPORT void initReferenceMap(referenceMapObj *ref);
 MS_DLL_EXPORT void initScalebar(scalebarObj *scalebar);
 MS_DLL_EXPORT void initGrid( graticuleObj *pGraticule );
+MS_DLL_EXPORT void initWeb(webObj *web);
+MS_DLL_EXPORT void freeWeb(webObj *web);
 
 MS_DLL_EXPORT featureListNodeObjPtr insertFeatureList(featureListNodeObjPtr *list, shapeObj *shape);
 MS_DLL_EXPORT void freeFeatureList(featureListNodeObjPtr list);
@@ -1430,7 +1343,9 @@ MS_DLL_EXPORT int msLoadProjectionString(projectionObj *p, char *value);
 MS_DLL_EXPORT int loadExpressionString(expressionObj *exp, char *value);
 /* Use this next, thread safe wrapper, function everywhere else */
 MS_DLL_EXPORT int msLoadExpressionString(expressionObj *exp, char *value);
+MS_DLL_EXPORT char *msGetExpressionString(expressionObj *exp);
 MS_DLL_EXPORT void freeExpression(expressionObj *exp);
+MS_DLL_EXPORT char *msGetExpressionString(expressionObj *exp);
 
 MS_DLL_EXPORT int getClassIndex(layerObj *layer, char *str);
 
@@ -1566,6 +1481,7 @@ MS_DLL_EXPORT int msIsXMLTagValid(const char *string);
 MS_DLL_EXPORT char *strcatalloc(char *pszDest, char *pszSrc);
 MS_DLL_EXPORT char *msJoinStrings(char **array, int arrayLength, const char *delimeter);
 MS_DLL_EXPORT char *msHashString(const char *pszStr);
+MS_DLL_EXPORT char *msCommifyString(char *str);
 
 #ifdef NEED_STRDUP
 MS_DLL_EXPORT char *strdup(char *s);
@@ -1619,13 +1535,15 @@ MS_DLL_EXPORT int msInitFontSet(fontSetObj *fontset);
 MS_DLL_EXPORT int msFreeFontSet(fontSetObj *fontset);
 
 MS_DLL_EXPORT int msGetLabelSize(char *string, labelObj *label, rectObj *rect, fontSetObj *fontSet, double scalefactor, int adjustBaseline);
-MS_DLL_EXPORT int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int tileindex, pointObj *point, char *string, double featuresize, labelObj *);
+MS_DLL_EXPORT int msGetLabelSizeEx(char *string, labelObj *label, rectObj *rect, fontSetObj *fontSet, double scalefactor, int adjustBaseline, double **offsets);
+MS_DLL_EXPORT int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int tileindex, pointObj *point, labelPathObj *labelpath, char *string, double featuresize, labelObj *label);
 
 MS_DLL_EXPORT gdFontPtr msGetBitmapFont(int size);
 MS_DLL_EXPORT int msImageTruetypePolyline(symbolSetObj *symbolset, gdImagePtr img, shapeObj *p, styleObj *style, double scalefactor);
 MS_DLL_EXPORT int msImageTruetypeArrow(symbolSetObj *symbolset, gdImagePtr img, shapeObj *p, styleObj *style, double scalefactor);
 
 MS_DLL_EXPORT void msFreeShape(shapeObj *shape); /* in mapprimative.c */
+MS_DLL_EXPORT void msFreeLabelPathObj(labelPathObj *path);
 MS_DLL_EXPORT shapeObj *msShapeFromWKT(const char *string);
 MS_DLL_EXPORT char *msShapeToWKT(shapeObj *shape);
 MS_DLL_EXPORT void msInitShape(shapeObj *shape);
@@ -1638,10 +1556,13 @@ MS_DLL_EXPORT void msRectToPolygon(rectObj rect, shapeObj *poly);
 MS_DLL_EXPORT void msClipPolylineRect(shapeObj *shape, rectObj rect);
 MS_DLL_EXPORT void msClipPolygonRect(shapeObj *shape, rectObj rect);
 MS_DLL_EXPORT void msTransformShape(shapeObj *shape, rectObj extent, double cellsize, imageObj *image);
+MS_DLL_EXPORT void msOffsetPointRelativeTo(pointObj *point, layerObj *layer);
+MS_DLL_EXPORT void msOffsetShapeRelativeTo(shapeObj *shape, layerObj *layer);
 MS_DLL_EXPORT void msTransformShapeToPixel(shapeObj *shape, rectObj extent, double cellsize);
 MS_DLL_EXPORT void msTransformPixelToShape(shapeObj *shape, rectObj extent, double cellsize);
 MS_DLL_EXPORT void msImageCartographicPolyline(gdImagePtr im, shapeObj *p, styleObj *style, symbolObj *symbol, int c, double size, double scalefactor);
 MS_DLL_EXPORT int msPolylineLabelPoint(shapeObj *p, pointObj *lp, int min_length, double *angle, double *length);
+MS_DLL_EXPORT labelPathObj* msPolylineLabelPath(shapeObj *p, int min_length, fontSetObj *fontset, char *string, labelObj *label, double scalefactor, int *status);
 MS_DLL_EXPORT int msPolygonLabelPoint(shapeObj *p, pointObj *lp, int min_dimension);
 MS_DLL_EXPORT int msAddLine(shapeObj *p, lineObj *new_line);
 MS_DLL_EXPORT int msAddLineDirectly(shapeObj *p, lineObj *new_line);
@@ -1828,6 +1749,7 @@ MS_DLL_EXPORT void msDrawLineSymbolGD(symbolSetObj *symbolset, gdImagePtr img, s
 MS_DLL_EXPORT void msDrawShadeSymbolGD(symbolSetObj *symbolset, gdImagePtr img, shapeObj *p, styleObj *style, double scalefactor);
 
 MS_DLL_EXPORT int msDrawTextGD(gdImagePtr img, pointObj labelPnt, char *string, labelObj *label, fontSetObj *fontset, double scalefactor);
+MS_DLL_EXPORT int msDrawTextLineGD(gdImagePtr img, char *string, labelObj *label, labelPathObj *labelpath, fontSetObj *fontset, double scalefactor);
 MS_DLL_EXPORT int msDrawLabelCacheGD(gdImagePtr img, mapObj *map);
 
 MS_DLL_EXPORT void msImageCopyMerge (gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int srcX, int srcY, int w, int h, int pct);
@@ -2086,13 +2008,47 @@ MS_DLL_EXPORT void *msGetSymbol(const char *pszLibrary,
 #include "mapows.h"
 
 /* ==================================================================== */
-/*      prototypes for functions in mapgeos.cpp                         */
+/*      prototypes for functions in mapgeos.c                         */
 /* ==================================================================== */
+MS_DLL_EXPORT void msGEOSSetup();
+MS_DLL_EXPORT void msGEOSCleanup();
 MS_DLL_EXPORT void msGEOSFreeGeometry(shapeObj *shape);
+
 MS_DLL_EXPORT shapeObj *msGEOSShapeFromWKT(const char *string);
 MS_DLL_EXPORT char *msGEOSShapeToWKT(shapeObj *shape);
+
 MS_DLL_EXPORT shapeObj *msGEOSBuffer(shapeObj *shape, double width);
 MS_DLL_EXPORT shapeObj *msGEOSConvexHull(shapeObj *shape);
+MS_DLL_EXPORT shapeObj *msGEOSBoundary(shapeObj *shape);
+MS_DLL_EXPORT pointObj *msGEOSGetCentroid(shapeObj *shape);
+MS_DLL_EXPORT shapeObj *msGEOSUnion(shapeObj *shape1, shapeObj *shape2);
+MS_DLL_EXPORT shapeObj *msGEOSIntersection(shapeObj *shape1, shapeObj *shape2);
+MS_DLL_EXPORT shapeObj *msGEOSDifference(shapeObj *shape1, shapeObj *shape2);
+MS_DLL_EXPORT shapeObj *msGEOSSymDifference(shapeObj *shape1, shapeObj *shape2);
+
+MS_DLL_EXPORT int msGEOSContains(shapeObj *shape1, shapeObj *shape2);
+MS_DLL_EXPORT int msGEOSOverlaps(shapeObj *shape1, shapeObj *shape2);
+MS_DLL_EXPORT int msGEOSWithin(shapeObj *shape1, shapeObj *shape2);
+MS_DLL_EXPORT int msGEOSCrosses(shapeObj *shape1, shapeObj *shape2);
+MS_DLL_EXPORT int msGEOSIntersects(shapeObj *shape1, shapeObj *shape2);
+MS_DLL_EXPORT int msGEOSTouches(shapeObj *shape1, shapeObj *shape2);
+MS_DLL_EXPORT int msGEOSEquals(shapeObj *shape1, shapeObj *shape2);
+MS_DLL_EXPORT int msGEOSDisjoint(shapeObj *shape1, shapeObj *shape2);
+
+MS_DLL_EXPORT double msGEOSArea(shapeObj *shape);
+MS_DLL_EXPORT double msGEOSLength(shapeObj *shape);
+MS_DLL_EXPORT double msGEOSDistance(shapeObj *shape1, shapeObj *shape2);
+
+/* ==================================================================== */
+/*      prototypes for functions in mapcrypto.c                         */
+/* ==================================================================== */
+MS_DLL_EXPORT int msGenerateEncryptionKey(unsigned char *k);
+MS_DLL_EXPORT int msReadEncryptionKeyFromFile(const char *keyfile, unsigned char *k);
+MS_DLL_EXPORT void msEncryptStringWithKey(const unsigned char *key, const char *in, char *out);
+MS_DLL_EXPORT void msDecryptStringWithKey(const unsigned char *key, const char *in, char *out);
+MS_DLL_EXPORT char *msDecryptStringTokens(mapObj *map, const char *in);
+MS_DLL_EXPORT void msHexEncode(const unsigned char *in, char *out, int numbytes);
+MS_DLL_EXPORT int msHexDecode(const char *in, unsigned char *out, int numchars);
 
 
 #endif
