@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapcontext.c,v 1.70 2005/02/18 03:06:45 dan Exp $
+ * $Id: mapcontext.c,v 1.78 2006/09/05 13:49:42 julien Exp $
  *
  * Name:     mapcontext.c
  * Project:  MapServer
@@ -30,6 +30,30 @@
  **********************************************************************
  *
  * $Log: mapcontext.c,v $
+ * Revision 1.78  2006/09/05 13:49:42  julien
+ * WMC true/false bool and use format when formatlist not avail. bug 1723,1692
+ *
+ * Revision 1.77  2006/06/15 14:58:24  julien
+ * Add SLD xsd and 1.1.0 reference in the root element
+ *
+ * Revision 1.76  2006/06/14 18:13:39  julien
+ * Support WMC Min/Max scale in write mode. bug 1581
+ *
+ * Revision 1.75  2006/03/10 15:30:15  julien
+ * Set the wms_time metadata when we have the time dimension in context1.1
+ *
+ * Revision 1.74  2006/03/09 21:08:54  julien
+ * Remove XML header tag check
+ *
+ * Revision 1.73  2006/02/14 03:53:04  julien
+ * Change layer server type for 1.1.0 version
+ *
+ * Revision 1.72  2006/02/14 03:38:47  julien
+ * Update to MapContext 1.1.0, add dimensions support in context bug 1581
+ *
+ * Revision 1.71  2006/02/09 16:57:14  julien
+ * Support SLD_BODY in Web Map Context
+ *
  * Revision 1.70  2005/02/18 03:06:45  dan
  * Turned all C++ (//) comments into C comments (bug 1238)
  *
@@ -257,7 +281,7 @@
 
 #include "map.h"
 
-MS_CVSID("$Id: mapcontext.c,v 1.70 2005/02/18 03:06:45 dan Exp $")
+MS_CVSID("$Id: mapcontext.c,v 1.78 2006/09/05 13:49:42 julien Exp $")
 
 #if defined(USE_WMS_LYR) && defined(USE_OGR)
 
@@ -636,7 +660,7 @@ int msLoadMapContextLayerFormat(CPLXMLNode *psFormat, layerObj *layer)
       pszValue1 = (char*)CPLGetXMLValue(psFormat, 
                                         "current", NULL);
       if(pszValue1 != NULL && 
-         (strcasecmp(pszValue1, "1") == 0))
+         (strcasecmp(pszValue1, "1") == 0 || strcasecmp(pszValue1, "true")==0))
           msInsertHashTable(&(layer->metadata), 
                             "wms_format", pszValue);
       /* wms_formatlist */
@@ -721,6 +745,7 @@ int msLoadMapContextLayerStyle(CPLXMLNode *psStyle, layerObj *layer,
 {
   char *pszValue, *pszValue1, *pszValue2;
   char *pszHash, *pszStyle=NULL, *pszStyleName;
+  CPLXMLNode *psStyleSLDBody;
 
   pszStyleName =(char*)CPLGetXMLValue(psStyle,"Name",NULL);
   if(pszStyleName == NULL)
@@ -734,7 +759,8 @@ int msLoadMapContextLayerStyle(CPLXMLNode *psStyle, layerObj *layer,
   /* wms_style */
   pszValue = (char*)CPLGetXMLValue(psStyle,"current",NULL);
   if(pszValue != NULL && 
-     (strcasecmp(pszValue, "1") == 0))
+     (strcasecmp(pszValue, "1") == 0 || 
+      strcasecmp(pszValue, "true") == 0))
       msInsertHashTable(&(layer->metadata), 
                         "wms_style", pszStyleName);
   /* wms_stylelist */
@@ -771,6 +797,30 @@ int msLoadMapContextLayerStyle(CPLXMLNode *psStyle, layerObj *layer,
                                      &(layer->metadata), pszStyle );
   free(pszStyle);
 
+  /* SLDBODY */
+  pszStyle = (char*)malloc(strlen(pszStyleName)+20);
+  sprintf(pszStyle, "wms_style_%s_sld_body", pszStyleName);
+
+  psStyleSLDBody = CPLGetXMLNode(psStyle, "SLD.StyledLayerDescriptor");
+  if(psStyleSLDBody != NULL && &(layer->metadata) != NULL)
+  {
+      pszValue = CPLSerializeXMLTree(psStyleSLDBody);
+      if(pszValue != NULL)
+      {
+          /* Before including SLDBody in the mapfile, we must replace the */
+          /* double quote for single quote. This is to prevent having this: */
+          /* "metadata" "<string attriute="ttt">" */
+          char *c;
+          for(c=pszValue; *c != '\0'; c++)
+              if(*c == '"')
+                  *c = '\'';
+          msInsertHashTable(&(layer->metadata), pszStyle, pszValue );
+          msFree(pszValue);
+      }
+  }
+
+  free(pszStyle);
+
   /* LegendURL */
   pszStyle = (char*) malloc(strlen(pszStyleName) + 25);
 
@@ -831,6 +881,83 @@ int msLoadMapContextLayerStyle(CPLXMLNode *psStyle, layerObj *layer,
   return MS_SUCCESS;
 }
 
+int msLoadMapContextLayerDimension(CPLXMLNode *psDimension, layerObj *layer)
+{
+    char *pszValue, *pszHash;
+  char *pszDimension=NULL, *pszDimensionName=NULL;
+
+  pszDimensionName =(char*)CPLGetXMLValue(psDimension,"name",NULL);
+  if(pszDimensionName == NULL)
+  {
+      return MS_FALSE;
+  }
+  else
+      pszDimensionName = strdup(pszDimensionName);
+
+  pszDimension = (char*)malloc(strlen(pszDimensionName)+50);
+
+  /* wms_dimension: This is the current dimension */
+  pszValue = (char*)CPLGetXMLValue(psDimension, "current", NULL);
+  if(pszValue != NULL && (strcasecmp(pszValue, "1") == 0 || 
+                          strcasecmp(pszValue, "true") == 0))
+      msInsertHashTable(&(layer->metadata), 
+                        "wms_dimension", pszDimensionName);
+  /* wms_dimensionlist */
+  pszHash = msLookupHashTable(&(layer->metadata), 
+                              "wms_dimensionlist");
+  if(pszHash != NULL)
+  {
+      pszValue = (char*)malloc(strlen(pszHash)+
+                                strlen(pszDimensionName)+2);
+      sprintf(pszValue, "%s,%s", pszHash, pszDimensionName);
+      msInsertHashTable(&(layer->metadata), 
+                        "wms_dimensionlist", pszValue);
+      free(pszValue);
+  }
+  else
+      msInsertHashTable(&(layer->metadata), 
+                        "wms_dimensionlist", pszDimensionName);
+
+  /* Units */
+  sprintf(pszDimension, "wms_dimension_%s_units", pszDimensionName);
+  msGetMapContextXMLHashValue(psDimension, "units", &(layer->metadata), 
+                              pszDimension);
+
+  /* UnitSymbol */
+  sprintf(pszDimension, "wms_dimension_%s_unitsymbol", pszDimensionName);
+  msGetMapContextXMLHashValue(psDimension, "unitSymbol", &(layer->metadata),
+                              pszDimension);
+
+  /* userValue */
+  sprintf(pszDimension, "wms_dimension_%s_uservalue", pszDimensionName);
+  msGetMapContextXMLHashValue(psDimension, "userValue", &(layer->metadata),
+                              pszDimension);
+  if(strcasecmp(pszDimensionName, "time") == 0)
+      msGetMapContextXMLHashValue(psDimension, "userValue", &(layer->metadata),
+                                  "wms_time");
+
+  /* default */
+  sprintf(pszDimension, "wms_dimension_%s_default", pszDimensionName);
+  msGetMapContextXMLHashValue(psDimension, "default", &(layer->metadata),
+                              pszDimension);
+
+  /* multipleValues */
+  sprintf(pszDimension, "wms_dimension_%s_multiplevalues", pszDimensionName);
+  msGetMapContextXMLHashValue(psDimension, "multipleValues",&(layer->metadata),
+                              pszDimension);
+
+  /* nearestValue */
+  sprintf(pszDimension, "wms_dimension_%s_nearestvalue", pszDimensionName);
+  msGetMapContextXMLHashValue(psDimension, "nearestValue", &(layer->metadata),
+                              pszDimension);
+
+  free(pszDimension);
+
+  free(pszDimensionName);
+
+  return MS_SUCCESS;
+}
+
 
 
 /*
@@ -1010,6 +1137,7 @@ int msLoadMapContextLayer(mapObj *map, CPLXMLNode *psLayer, int nVersion,
   char *pszValue;
   char *pszHash, *pszName=NULL;
   CPLXMLNode *psFormatList, *psFormat, *psStyleList, *psStyle;
+  CPLXMLNode *psDimensionList, *psDimension;
   int nStyle;
   layerObj *layer;
 
@@ -1026,14 +1154,16 @@ int msLoadMapContextLayer(mapObj *map, CPLXMLNode *psLayer, int nVersion,
   
   /* Status */
   pszValue = (char*)CPLGetXMLValue(psLayer, "hidden", "1");
-  if((pszValue != NULL) && (atoi(pszValue) == 0))
+  if((pszValue != NULL) && (atoi(pszValue) == 0 && 
+                            !strcasecmp(pszValue, "true") == 0))
       layer->status = MS_ON;
   else
       layer->status = MS_OFF;
 
   /* Queryable */
   pszValue = (char*)CPLGetXMLValue(psLayer, "queryable", "0");
-  if(pszValue !=NULL && atoi(pszValue) == 1)
+  if(pszValue !=NULL && (atoi(pszValue) == 1  || 
+                         strcasecmp(pszValue, "true") == 0))
       layer->template = strdup("ttt");
 
   /* Name and Title */
@@ -1096,6 +1226,20 @@ int msLoadMapContextLayer(mapObj *map, CPLXMLNode *psLayer, int nVersion,
   msLoadMapContextURLELements( CPLGetXMLNode(psLayer, "MetadataURL"), 
                                        &(layer->metadata), "wms_metadataurl" );
 
+
+  /* MinScale && MaxScale */
+  pszValue = (char*)CPLGetXMLValue(psLayer, "sld:MinScaleDenominator", NULL);
+  if(pszValue != NULL)
+  {
+      layer->minscale = atof(pszValue);
+  }
+
+  pszValue = (char*)CPLGetXMLValue(psLayer, "sld:MaxScaleDenominator", NULL);
+  if(pszValue != NULL)
+  {
+      layer->maxscale = atof(pszValue);
+  }
+
   /*  */
   /* Server */
   /*  */
@@ -1238,11 +1382,27 @@ int msLoadMapContextLayer(mapObj *map, CPLXMLNode *psLayer, int nVersion,
           if(strcasecmp(psStyle->pszValue, "Style") == 0)
           {
               nStyle++;
-              msLoadMapContextLayerStyle(psStyle, layer, nVersion);
+              msLoadMapContextLayerStyle(psStyle, layer, nStyle);
           }
       }
   }
 
+  /* Dimension */
+  psDimensionList = CPLGetXMLNode(psLayer, "DimensionList");
+  if(psDimensionList != NULL)
+  {
+      for(psDimension = psDimensionList->psChild; 
+          psDimension != NULL; 
+          psDimension = psDimension->psNext)
+      {
+          if(strcasecmp(psDimension->pszValue, "Dimension") == 0)
+          {
+              msLoadMapContextLayerDimension(psDimension, layer);
+          }
+      }
+  }
+  
+
   return MS_SUCCESS;
 }
 
@@ -1341,7 +1501,7 @@ int msLoadMapContext(mapObj *map, char *filename, int unique_layer_names)
   free( pszWholeText );
 
   /* We assume parser will report errors via CPL. */
-  if( psRoot == NULL || psRoot->psNext == NULL )
+  if( psRoot == NULL )
   {
       msSetError( MS_MAPCONTEXTERR, "Invalid XML file (%s)", 
                   "msLoadMapContext()", filename );
@@ -1399,6 +1559,7 @@ int msLoadMapContext(mapObj *map, char *filename, int unique_layer_names)
     case OWS_0_1_4:
     case OWS_0_1_7:
     case OWS_1_0_0:
+    case OWS_1_1_0:
       /* All is good, this is a supported version. */
       break;
     default:
@@ -1526,14 +1687,20 @@ int msWriteMapContext(mapObj *map, FILE *stream)
 #if defined(USE_WMS_LYR) && defined(USE_OGR)
   const char * version, *value;
   char * tabspace=NULL, *pszValue, *pszChar,*pszSLD=NULL,*pszURL,*pszSLD2=NULL;
-  char *pszStyle, *pszCurrent, *pszStyleItem;
+  char *pszStyle, *pszCurrent, *pszStyleItem, *pszSLDBody;
   char *pszEncodedVal;
   int i, nValue, nVersion=-1;
+  /* Dimension element */
+  char *pszDimension;
+  const char *pszDimUserValue=NULL, *pszDimUnits=NULL, *pszDimDefault=NULL;
+  const char *pszDimNearValue=NULL, *pszDimUnitSymbol=NULL;
+  const char *pszDimMultiValue=NULL;
+  int bDimensionList = 0;
 
   /* Decide which version we're going to return... */
   version = msLookupHashTable(&(map->web.metadata), "wms_context_version");
   if(version == NULL)
-    version = "1.0.0";
+    version = "1.1.0";
 
   nVersion = msOWSParseVersionString(version);
   if (nVersion < 0)
@@ -1546,6 +1713,7 @@ int msWriteMapContext(mapObj *map, FILE *stream)
     case OWS_0_1_4:
     case OWS_0_1_7:
     case OWS_1_0_0:
+    case OWS_1_1_0:
       /* All is good, this is a supported version. */
       break;
     default:
@@ -1598,11 +1766,16 @@ int msWriteMapContext(mapObj *map, FILE *stream)
   {
       msIO_fprintf( stream, " xmlns:xlink=\"http://www.w3.org/1999/xlink\"");
       msIO_fprintf( stream, " xmlns=\"http://www.opengis.net/context\"");
-      /* fprintf( stream, " xmlns:sld=\"http://www.opengis.net/sld"); */
+      msIO_fprintf( stream, " xmlns:sld=\"http://www.opengis.net/sld\"");
       pszEncodedVal = msEncodeHTMLEntities(msOWSGetSchemasLocation(map));
-      msIO_fprintf( stream, 
-               " xsi:schemaLocation=\"http://www.opengis.net/context %s/context/1.0.0/context.xsd\">\n",
-                    pszEncodedVal);
+      if( nVersion >= OWS_1_1_0 )
+          msIO_fprintf( stream, 
+                        " xsi:schemaLocation=\"http://www.opengis.net/context %s/context/1.1.0/context.xsd\">\n",
+                        pszEncodedVal);
+      else
+          msIO_fprintf( stream, 
+                        " xsi:schemaLocation=\"http://www.opengis.net/context %s/context/1.0.0/context.xsd\">\n",
+                        pszEncodedVal);
       msFree(pszEncodedVal);
   }
   else
@@ -1755,10 +1928,16 @@ int msWriteMapContext(mapObj *map, FILE *stream)
           /*  */
           /* Server definition */
           /*  */
-          msOWSPrintEncodeMetadata(stream, &(map->layers[i].metadata), 
-                                   NULL, "wms_server_version", OWS_WARN,
+          if(nVersion <= OWS_1_0_0 )
+              msOWSPrintEncodeMetadata(stream, &(map->layers[i].metadata), 
+                                       NULL, "wms_server_version", OWS_WARN,
                              "      <Server service=\"WMS\" version=\"%s\" ",
-                                   "1.1.0");
+                                       "1.0.0");
+          else
+              msOWSPrintEncodeMetadata(stream, &(map->layers[i].metadata), 
+                                       NULL, "wms_server_version", OWS_WARN,
+                          "      <Server service=\"OGC:WMS\" version=\"%s\" ",
+                                       "1.0.0");
           if(map->layers[i].name)
               msOWSPrintEncodeMetadata(stream, &(map->layers[i].metadata), 
                                        NULL, "wms_title", OWS_NOERR, 
@@ -1842,6 +2021,16 @@ int msWriteMapContext(mapObj *map, FILE *stream)
                             MS_FALSE, MS_FALSE, MS_FALSE, MS_FALSE, 
                             MS_TRUE, NULL, NULL, NULL, NULL, NULL, "      ");
 
+          /* MinScale && MaxScale */
+          if(nVersion >= OWS_1_1_0 && map->layers[i].minscale > 0)
+              msIO_fprintf(stream, 
+               "      <sld:MinScaleDenominator>%g</sld:MinScaleDenominator>\n",
+                           map->layers[i].minscale);
+          if(nVersion >= OWS_1_1_0 && map->layers[i].maxscale > 0)
+              msIO_fprintf(stream, 
+               "      <sld:MaxScaleDenominator>%g</sld:MaxScaleDenominator>\n",
+                           map->layers[i].maxscale);
+
           /* Layer SRS */
           pszValue = (char*)msOWSGetEPSGProj(&(map->layers[i].projection), 
                                              &(map->layers[i].metadata),
@@ -1854,7 +2043,8 @@ int msWriteMapContext(mapObj *map, FILE *stream)
           }
 
           /* Format */
-          if(msLookupHashTable(&(map->layers[i].metadata),"wms_formatlist")==NULL)
+          if(msLookupHashTable(&(map->layers[i].metadata),"wms_formatlist")==NULL && 
+             msLookupHashTable(&(map->layers[i].metadata),"wms_format")==NULL)
           {
               pszURL = NULL;
               if(map->layers[i].connection)
@@ -1888,6 +2078,9 @@ int msWriteMapContext(mapObj *map, FILE *stream)
 
               pszValue = msLookupHashTable(&(map->layers[i].metadata), 
                                            "wms_formatlist");
+              if(!pszValue)
+                  pszValue = msLookupHashTable(&(map->layers[i].metadata), 
+                                               "wms_format");
               pszCurrent = msLookupHashTable(&(map->layers[i].metadata), 
                                              "wms_format");
 
@@ -1913,16 +2106,20 @@ int msWriteMapContext(mapObj *map, FILE *stream)
               if(papszFormats)
                   msFreeCharArray(papszFormats, numFormats);
           }
-          /*  */
-          pszValue = msLookupHashTable(&(map->layers[i].metadata),"wms_stylelist");
+          /* Style */
+          /* First check the stylelist */
+          pszValue = msLookupHashTable(&(map->layers[i].metadata), 
+                                       "wms_stylelist");
           if(pszValue == NULL || strlen(trimLeft(pszValue)) < 1)
           {
+              /* Check if the style is in the connection URL */
               pszURL = "";
               if(map->layers[i].connection)
                   pszURL = strdup( map->layers[i].connection );
               else
                   pszURL = strdup( "" );
               pszValue = pszURL;
+              /* Grab the STYLES in the URL */
               pszValue = strstr( pszValue, "STYLES=" );
               if( pszValue )
               {
@@ -1931,14 +2128,22 @@ int msWriteMapContext(mapObj *map, FILE *stream)
                   if( pszChar )
                       pszValue[pszChar - pszValue] = '\0';
 
+                  /* Check the SLD string from the URL */
                   if(map->layers[i].connection)
                       pszSLD2 = strdup(map->layers[i].connection);
                   else
                       pszSLD2 = strdup( "" );
                   if(pszSLD2)
+                  {
                       pszSLD = strstr(pszSLD2, "SLD=");
+                      pszSLDBody = strstr(pszSLD2, "SLD_BODY=");
+                  }
                   else
+                  {
                       pszSLD = NULL;
+                      pszSLDBody = NULL;
+                  }
+                  /* Check SLD */
                   if( pszSLD )
                   {
                       pszChar = strchr(pszSLD, '&');
@@ -1946,9 +2151,19 @@ int msWriteMapContext(mapObj *map, FILE *stream)
                           pszSLD[pszChar - pszSLD] = '\0';
                       pszSLD += 4;
                   }
-                  if( (pszValue && (strcasecmp(pszValue, "") != 0)) || 
-                      (pszSLD && (strcasecmp(pszSLD, "") != 0)))
+                  /* Check SLDBody  */
+                  if( pszSLDBody )
                   {
+                      pszChar = strchr(pszSLDBody, '&');
+                      if( pszChar )
+                          pszSLDBody[pszChar - pszSLDBody] = '\0';
+                      pszSLDBody += 9;
+                  }
+                  if( (pszValue && (strcasecmp(pszValue, "") != 0)) || 
+                      (pszSLD && (strcasecmp(pszSLD, "") != 0)) || 
+                      (pszSLDBody && (strcasecmp(pszSLDBody, "") != 0)))
+                  {
+                      /* Write Name and Title */
                       msIO_fprintf( stream, "      <StyleList>\n");
                       msIO_fprintf( stream, "        <Style current=\"1\">\n");
                       if( pszValue && (strcasecmp(pszValue, "") != 0))
@@ -1960,16 +2175,24 @@ int msWriteMapContext(mapObj *map, FILE *stream)
                                   pszEncodedVal);
                           msFree(pszEncodedVal);
                       }
+                      /* Write the SLD string from the URL */
                       if( pszSLD && (strcasecmp(pszSLD, "") != 0))
                       {
                           pszEncodedVal = msEncodeHTMLEntities(pszSLD);
                           msIO_fprintf( stream, "          <SLD>\n" );
                           msIO_fprintf( stream, 
                          "            <OnlineResource xlink:type=\"simple\" ");
-                          msIO_fprintf(stream,"xlink:href=\"%s\"/>", pszEncodedVal);
+                          msIO_fprintf(stream,"xlink:href=\"%s\"/>", 
+                                       pszEncodedVal);
                           msIO_fprintf( stream, "          </SLD>\n" );
                           free(pszEncodedVal);
                       }
+                      else if(pszSLDBody && (strcasecmp(pszSLDBody, "") != 0))
+                      {
+                          msIO_fprintf( stream, "          <SLD>\n" );
+                          msIO_fprintf( stream, "            %s\n",pszSLDBody);
+                          msIO_fprintf( stream, "          </SLD>\n" );
+                      }
                       msIO_fprintf( stream, "        </Style>\n");
                       msIO_fprintf( stream, "      </StyleList>\n");
                   }
@@ -1987,46 +2210,67 @@ int msWriteMapContext(mapObj *map, FILE *stream)
           }
           else
           {
+              /* If the style information is not in the connection URL, */
+              /* read the metadata. */
               pszValue = msLookupHashTable(&(map->layers[i].metadata), 
                                            "wms_stylelist");
               pszCurrent = msLookupHashTable(&(map->layers[i].metadata), 
                                              "wms_style");
               msIO_fprintf( stream, "      <StyleList>\n");
+              /* Loop in each style in the style list */
               while(pszValue != NULL)
               {
                   pszStyle = strdup(pszValue);
                   pszChar = strchr(pszStyle, ',');
                   if(pszChar != NULL)
                       pszStyle[pszChar - pszStyle] = '\0';
-                  if( strcasecmp(pszStyle, "") != 0)
-                  {
-                      if(pszCurrent && (strcasecmp(pszStyle, pszCurrent) == 0))
-                          msIO_fprintf( stream,"        <Style current=\"1\">\n" );
-                      else
-                          msIO_fprintf( stream, "        <Style>\n" );
+                  if(strcasecmp(pszStyle, "") == 0)
+                      continue;
 
-                      pszStyleItem = (char*)malloc(strlen(pszStyle)+10+5);
-                      sprintf(pszStyleItem, "wms_style_%s_sld", pszStyle);
+                  if(pszCurrent && (strcasecmp(pszStyle, pszCurrent) == 0))
+                      msIO_fprintf( stream,"        <Style current=\"1\">\n" );
+                  else
+                      msIO_fprintf( stream, "        <Style>\n" );
+
+                  /* Write SLDURL if it is in the metadata */
+                  pszStyleItem = (char*)malloc(strlen(pszStyle)+10+10);
+                  sprintf(pszStyleItem, "wms_style_%s_sld", pszStyle);
+                  if(msLookupHashTable(&(map->layers[i].metadata),
+                                       pszStyleItem) != NULL)
+                  {
+                      msIO_fprintf(stream, "          <SLD>\n");
+                      msOWSPrintEncodeMetadata(stream, 
+                                               &(map->layers[i].metadata),
+                                               NULL, pszStyleItem, 
+                                               OWS_NOERR, 
+     "            <OnlineResource xlink:type=\"simple\" xlink:href=\"%s\"/>\n",
+                                               NULL);
+                      msIO_fprintf(stream, "          </SLD>\n");
+                      free(pszStyleItem);
+                  }
+                  else
+                  {
+                      /* If the URL is not there, check for SLDBody */
+                      sprintf(pszStyleItem, "wms_style_%s_sld_body", pszStyle);
                       if(msLookupHashTable(&(map->layers[i].metadata),
                                            pszStyleItem) != NULL)
                       {
                           msIO_fprintf(stream, "          <SLD>\n");
-                          msOWSPrintEncodeMetadata(stream, 
-                                                   &(map->layers[i].metadata),
-                                                   NULL, pszStyleItem, 
-                                                   OWS_NOERR, 
-     "            <OnlineResource xlink:type=\"simple\" xlink:href=\"%s\"/>\n",
-                                                   NULL);
+                          msOWSPrintMetadata(stream,&(map->layers[i].metadata),
+                                             NULL, pszStyleItem, OWS_NOERR, 
+                                             "            %s\n", NULL);
                           msIO_fprintf(stream, "          </SLD>\n");
                           free(pszStyleItem);
                       }
                       else
                       {
+                          /* If the SLD is not specified, then write the */
+                          /* name, Title and LegendURL */
                           free(pszStyleItem);
                           /* Name */
                           pszEncodedVal = msEncodeHTMLEntities(pszStyle);
                           msIO_fprintf(stream, "          <Name>%s</Name>\n", 
-                                  pszEncodedVal);
+                                       pszEncodedVal);
                           msFree(pszEncodedVal);
                           pszStyleItem = (char*)malloc(strlen(pszStyle)+10+8);
                           sprintf(pszStyleItem, "wms_style_%s_title",pszStyle);
@@ -2035,7 +2279,7 @@ int msWriteMapContext(mapObj *map, FILE *stream)
                                                    &(map->layers[i].metadata), 
                                                    NULL, pszStyleItem, 
                                                    OWS_NOERR, 
-                                              "          <Title>%s</Title>\n", 
+                                        "          <Title>%s</Title>\n",
                                                    NULL);
                           free(pszStyleItem);
 
@@ -2055,9 +2299,10 @@ int msWriteMapContext(mapObj *map, FILE *stream)
                                             NULL, NULL, NULL, "          ");
                           free(pszStyleItem);
                       }
-
-                      msIO_fprintf( stream,"        </Style>\n" );
                   }
+
+                  msIO_fprintf( stream,"        </Style>\n" );
+ 
                   free(pszStyle);
                   pszValue = strchr(pszValue, ',');
                   if(pszValue)  
@@ -2066,6 +2311,85 @@ int msWriteMapContext(mapObj *map, FILE *stream)
               msIO_fprintf( stream, "      </StyleList>\n");
           }
 
+          /* Dimension element */;
+          pszCurrent = NULL;
+
+          pszValue = msLookupHashTable(&(map->layers[i].metadata), 
+                                       "wms_dimensionlist");
+          pszCurrent = msLookupHashTable(&(map->layers[i].metadata), 
+                                         "wms_dimension");
+          while(pszValue != NULL)
+          {
+              /* Extract the dimension name from the list */
+              pszDimension = strdup(pszValue);
+              pszChar = strchr(pszDimension, ',');
+              if(pszChar != NULL)
+                  pszDimension[pszChar - pszDimension] = '\0';
+              if(strcasecmp(pszDimension, "") == 0)
+              {
+                  free(pszDimension);
+                  pszValue = strchr(pszValue, ',');
+                  if(pszValue)  
+                      pszValue++;
+                  continue;
+              }
+
+              /* From the dimension list, extract the required dimension */
+              msOWSGetDimensionInfo(&(map->layers[i]), pszDimension, 
+                                    &pszDimUserValue, &pszDimUnits, 
+                                    &pszDimDefault, &pszDimNearValue, 
+                                    &pszDimUnitSymbol, &pszDimMultiValue);
+
+              if(pszDimUserValue == NULL || pszDimUnits == NULL || 
+                 pszDimUnitSymbol == NULL)
+              {
+                  free(pszDimension);
+                  pszValue = strchr(pszValue, ',');
+                  if(pszValue)  
+                      pszValue++;
+                  continue;
+              }
+
+              if(!bDimensionList)
+              {
+                  bDimensionList = 1;
+                  msIO_fprintf( stream, "      <DimensionList>\n");
+              }
+
+              /* name */
+              msIO_fprintf( stream, "        <Dimension name=\"%s\"", 
+                            pszDimension);
+              /* units */
+              msIO_fprintf( stream, " units=\"%s\"",      pszDimUnits);
+              /* unitSymbol */
+              msIO_fprintf( stream, " unitSymbol=\"%s\"", pszDimUnitSymbol);
+              /* userValue */
+              msIO_fprintf( stream, " userValue=\"%s\"",  pszDimUserValue);
+              /* default */
+              if(pszDimDefault)
+                  msIO_fprintf( stream, " default=\"%s\"", pszDimDefault);
+              /* multipleValues */
+              if(pszDimMultiValue)
+                  msIO_fprintf(stream, " multipleValues=\"%s\"", 
+                               pszDimMultiValue);
+              /* nearestValue */
+              if(pszDimNearValue)
+                  msIO_fprintf( stream," nearestValue=\"%s\"",pszDimNearValue);
+      
+              if(pszCurrent && strcasecmp(pszDimension, pszCurrent) == 0)
+                  msIO_fprintf( stream, " current=\"1\"");
+
+              msIO_fprintf( stream, "/>\n");
+
+              free(pszDimension);
+              pszValue = strchr(pszValue, ',');
+              if(pszValue)  
+                  pszValue++;
+          }
+
+          if(bDimensionList)
+              msIO_fprintf( stream, "      </DimensionList>\n");
+
           msIO_fprintf(stream, "    </Layer>\n");
       }
   }
@@ -2096,3 +2420,4 @@ int msWriteMapContext(mapObj *map, FILE *stream)
 #endif
 }
 
+
