@@ -27,7 +27,37 @@
  ******************************************************************************
  *
  * $Log: mapdraw.c,v $
- * Revision 1.96.2.1  2006/01/16 20:41:22  sdlime
+ * Revision 1.107  2006/08/17 04:37:17  sdlime
+ * In keeping with naming conventions (like it or not) label->angle_follow becomes label->autofollow...
+ *
+ * Revision 1.106  2006/08/15 05:01:54  sdlime
+ * Fixed a couple of query map bugs, 1858 that alters which style is re-colored with POLYGON layers, msDrawQueryMap now respects layer order.
+ *
+ * Revision 1.105  2006/05/08 17:41:04  frank
+ * fixed layer debug time reporting
+ *
+ * Revision 1.104  2006/04/28 03:13:02  sdlime
+ * Fixed a few issues with relative coordinates. Added support for all nine relative positions. (bug 1547)
+ *
+ * Revision 1.103  2006/04/27 04:05:17  sdlime
+ * Initial support for relative coordinates. (bug 1547)
+ *
+ * Revision 1.102  2006/04/13 12:35:35  umberto
+ * Fix for segfault on line 454: lp used before assigned
+ *
+ * Revision 1.101  2006/04/08 03:31:52  frank
+ * emit layer timing info if layer debug set (as well as map)
+ *
+ * Revision 1.100  2006/02/24 05:53:49  sdlime
+ * Applied another round of patches for bug 1620.
+ *
+ * Revision 1.99  2006/02/18 20:59:13  sdlime
+ * Initial code for curved labels. (bug 1620)
+ *
+ * Revision 1.98  2006/01/16 20:37:15  sdlime
+ * Changed label size calls to not adjust baseline offset.
+ *
+ * Revision 1.97  2006/01/16 20:21:18  sdlime
  * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
  *
  * Revision 1.96  2005/11/29 14:12:16  sdlime
@@ -103,7 +133,7 @@
 #include "map.h"
 #include "maptime.h"
 
-MS_CVSID("$Id: mapdraw.c,v 1.96.2.1 2006/01/16 20:41:22 sdlime Exp $")
+MS_CVSID("$Id: mapdraw.c,v 1.107 2006/08/17 04:37:17 sdlime Exp $")
 
 /*
  * Functions to reset any pen (color index) values previously set. Used primarily to reset things when
@@ -439,15 +469,15 @@ imageObj *msDrawMap(mapObj *map)
 
     for(i=0; i<map->numlayers; i++) {
 
-        if (map->debug)
-            msGettimeofday(&starttime, NULL);
-
         if (map->layerorder[i] != -1) {
             lp = &(map->layers[ map->layerorder[i]]);
 
             if(lp->postlabelcache) /* wait to draw */
                 continue;
 
+            if (map->debug || lp->debug )
+                msGettimeofday(&starttime, NULL);
+
             if (!msLayerIsVisible(map, lp))
                 continue;
 
@@ -511,7 +541,7 @@ imageObj *msDrawMap(mapObj *map)
             }
         }
 
-        if (map->debug)
+        if (map->debug || lp->debug)
         {
             msGettimeofday(&endtime, NULL);
             msDebug("msDrawMap(): Layer %d (%s), %.3fs\n", 
@@ -573,7 +603,7 @@ imageObj *msDrawMap(mapObj *map)
     if (!msLayerIsVisible(map, lp))
       continue;
 
-    if (map->debug)
+    if (map->debug || lp->debug)
         msGettimeofday(&starttime, NULL);
 
     if (lp->connectiontype == MS_WMS)  
@@ -610,7 +640,7 @@ imageObj *msDrawMap(mapObj *map)
         return(NULL);
     }
 
-    if (map->debug)
+    if (map->debug || lp->debug)
     {
         msGettimeofday(&endtime, NULL);
         msDebug("msDrawMap(): Layer %d (%s), %.3fs\n", 
@@ -706,7 +736,7 @@ imageObj *msDrawQueryMap(mapObj *map)
   }
 
   for(i=0; i<map->numlayers; i++) {
-    lp = &(map->layers[i]);
+    lp = &(map->layers[ map->layerorder[i]]);
 
     if(lp->postlabelcache) /* wait to draw */
       continue;
@@ -725,7 +755,7 @@ imageObj *msDrawQueryMap(mapObj *map)
     return(NULL);
 
   for(i=0; i<map->numlayers; i++) { /* for each layer, check for postlabelcache layers */
-    lp = &(map->layers[i]);
+    lp = &(map->layers[ map->layerorder[i]]);
 
     if(!lp->postlabelcache)
       continue;
@@ -1091,15 +1121,25 @@ int msDrawQueryLayer(mapObj *map, layerObj *layer, imageObj *image)
   /* reset layer pen values just in case the map has been previously processed */
   msClearLayerPenValues(layer);
 
-  /* if MS_HILITE, alter the first class (always at least 1 class), and set a MINDISTANCE for the labelObj to avoid duplicates */
+  /* if MS_HILITE, alter the one style (always at least 1 style), and set a MINDISTANCE for the labelObj to avoid duplicates */
   if(map->querymap.style == MS_HILITE) {
     for(i=0; i<layer->numclasses; i++) {
-      if(MS_VALID_COLOR(layer->class[i].styles[layer->class[i].numstyles-1].color)) {
-        colorbuffer[i] = layer->class[i].styles[layer->class[i].numstyles-1].color; /* save the color from the TOP style */
-        layer->class[i].styles[layer->class[i].numstyles-1].color = map->querymap.color;
-      } else if(MS_VALID_COLOR(layer->class[i].styles[layer->class[i].numstyles-1].outlinecolor)) {
-	colorbuffer[i] = layer->class[i].styles[layer->class[i].numstyles-1].outlinecolor; /* if no color, save the outlinecolor from the TOP style */
-        layer->class[i].styles[layer->class[i].numstyles-1].outlinecolor = map->querymap.color;
+      if(layer->type == MS_LAYER_POLYGON) { /* alter BOTTOM style since that's almost always the fill */
+        if(MS_VALID_COLOR(layer->class[i].styles[0].color)) {
+          colorbuffer[i] = layer->class[i].styles[0].color; /* save the color from the BOTTOM style */
+          layer->class[i].styles[0].color = map->querymap.color;
+        } else if(MS_VALID_COLOR(layer->class[i].styles[0].outlinecolor)) {
+          colorbuffer[i] = layer->class[i].styles[0].outlinecolor; /* if no color, save the outlinecolor from the BOTTOM style */
+          layer->class[i].styles[0].outlinecolor = map->querymap.color;
+        }
+      } else {
+        if(MS_VALID_COLOR(layer->class[i].styles[layer->class[i].numstyles-1].color)) {
+          colorbuffer[i] = layer->class[i].styles[layer->class[i].numstyles-1].color; /* save the color from the TOP style */
+          layer->class[i].styles[layer->class[i].numstyles-1].color = map->querymap.color;
+        } else if(MS_VALID_COLOR(layer->class[i].styles[layer->class[i].numstyles-1].outlinecolor)) {
+	  colorbuffer[i] = layer->class[i].styles[layer->class[i].numstyles-1].outlinecolor; /* if no color, save the outlinecolor from the TOP style */
+          layer->class[i].styles[layer->class[i].numstyles-1].outlinecolor = map->querymap.color;
+        }
       }
 
       mindistancebuffer[i] = layer->class[i].label.mindistance;
@@ -1182,10 +1222,17 @@ int msDrawQueryLayer(mapObj *map, layerObj *layer, imageObj *image)
   /* if MS_HILITE, restore color and mindistance values */
   if(map->querymap.style == MS_HILITE) {
     for(i=0; i<layer->numclasses; i++) {
-      if(MS_VALID_COLOR(layer->class[i].styles[layer->class[i].numstyles-1].color))
-        layer->class[i].styles[layer->class[i].numstyles-1].color = colorbuffer[i];        
-      else if(MS_VALID_COLOR(layer->class[i].styles[layer->class[i].numstyles-1].outlinecolor))
-	layer->class[i].styles[layer->class[i].numstyles-1].outlinecolor = colorbuffer[i]; /* if no color, restore outlinecolor for the TOP style */
+      if(layer->type == MS_LAYER_POLYGON) {
+	if(MS_VALID_COLOR(layer->class[i].styles[0].color))
+          layer->class[i].styles[0].color = colorbuffer[i];
+        else if(MS_VALID_COLOR(layer->class[i].styles[0].outlinecolor))
+          layer->class[i].styles[0].outlinecolor = colorbuffer[i]; /* if no color, restore outlinecolor for the BOTTOM style */
+      } else {
+        if(MS_VALID_COLOR(layer->class[i].styles[layer->class[i].numstyles-1].color))
+          layer->class[i].styles[layer->class[i].numstyles-1].color = colorbuffer[i];        
+        else if(MS_VALID_COLOR(layer->class[i].styles[layer->class[i].numstyles-1].outlinecolor))
+	  layer->class[i].styles[layer->class[i].numstyles-1].outlinecolor = colorbuffer[i]; /* if no color, restore outlinecolor for the TOP style */
+      }
     }
     layer->class[i].label.mindistance = mindistancebuffer[i];
   }
@@ -1310,6 +1357,9 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
   double r; /* circle radius */
   int csz; /* clip over size */
   double buffer;
+  int status = MS_FAILURE;
+  
+  labelPathObj *annopath = NULL; /* Curved label path. Bug #1620 implementation */
 
 /* Steve's original code
   cliprect.minx = map->extent.minx - 2*map->cellsize; // set clipping rectangle just a bit larger than the map extent
@@ -1357,11 +1407,12 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
       layer->project = MS_FALSE;
 #endif
 
-    if(layer->transform) {
+    if(layer->transform == MS_TRUE) {
       center.x = MS_MAP2IMAGE_X(center.x, map->extent.minx, map->cellsize);
       center.y = MS_MAP2IMAGE_Y(center.y, map->extent.maxy, map->cellsize);
       r /= map->cellsize;      
-    }
+    } else
+      msOffsetPointRelativeTo(&center, layer);
 
     /* shade symbol drawing will call outline function if color not set */
     if(style != -1)
@@ -1385,14 +1436,56 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
 
     switch(shape->type) {
     case(MS_SHAPE_LINE):
-      if(layer->transform) {
+      if(layer->transform == MS_TRUE) {
         msClipPolylineRect(shape, cliprect);
         if(shape->numlines == 0) return(MS_SUCCESS);
         msTransformShape(shape, map->extent, map->cellsize, image);
+      } else
+	msOffsetShapeRelativeTo(shape, layer);
+
+      /* Bug #1620 implementation */
+      if ( layer->class[c].label.autofollow == MS_TRUE ) {
+
+        annopath = msPolylineLabelPath(shape, layer->class[c].label.minfeaturesize, &(map->fontset), shape->text, &(layer->class[c].label), layer->scalefactor, &status);
+
+        if( annopath ) {
+
+          labelObj label;
+        
+          label = layer->class[c].label;
+          
+          /* Label path derived from line overrides the rotation and position
+             values, so ignore the specified angle and set the position to
+             auto. */
+          label.position = MS_AUTO;
+        
+          if((layer->labelsizeitemindex != -1) && (label.type == MS_TRUETYPE)) label.size = atoi(shape->values[layer->labelsizeitemindex]);
+        
+          if(layer->labelcache) {
+            if(msAddLabel(map, layer->index, c, shape->index, shape->tileindex, NULL, annopath, shape->text, length, &label) != MS_SUCCESS) return(MS_FAILURE);
+          } else {
+            /* FIXME: Not sure how this should work with the label path yet */
+            /*
+              if(MS_VALID_COLOR(layer->class[c].styles[0].color)) {
+              for(s=0; s<layer->class[c].numstyles; s++)
+              msDrawMarkerSymbol(&map->symbolset, image, &(label_line->point[0]), &(layer->class[c].styles[s]), layer->scalefactor);
+              }
+            */
+            /* FIXME: need to call msDrawTextLineGD() from here eventually */
+            /* msDrawLabel(image, label_line->point[0], shape->text, &label, &map->fontset, layer->scalefactor); */
+
+            /* Free the labelpath */
+            msFreeLabelPathObj(annopath);
+          }
+
+        }
       }
 
-      if(msPolylineLabelPoint(shape, &annopnt, layer->class[c].label.minfeaturesize, &angle, &length) == MS_SUCCESS) {
+      /* Use regular label algorithm if angle is AUTO or a number, or if ANGLE FOLLOW failed */
+      if ( layer->class[c].label.autofollow == MS_FALSE || (!annopath && status != MS_FAILURE) ) {
 
+        /* Regular labels */
+        if(msPolylineLabelPoint(shape, &annopnt, layer->class[c].label.minfeaturesize, &angle, &length) == MS_SUCCESS) {
         labelObj label;
         
         label = layer->class[c].label;
@@ -1407,24 +1500,27 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
 	if(label.autoangle) label.angle = angle;
 
         if(layer->labelcache) {
-	  if(msAddLabel(map, layer->index, c, shape->tileindex, shape->index, &annopnt, shape->text, length, &label) != MS_SUCCESS) return(MS_FAILURE);
+            if(msAddLabel(map, layer->index, c, shape->index, shape->tileindex, &annopnt, NULL, shape->text, length, &label) != MS_SUCCESS) return(MS_FAILURE);
 	} else {
           if(MS_VALID_COLOR(layer->class[c].styles[0].color)) {
             for(s=0; s<layer->class[c].numstyles; s++)
               msDrawMarkerSymbol(&map->symbolset, image, &annopnt, &(layer->class[c].styles[s]), layer->scalefactor);
 	  }
 	  msDrawLabel(image, annopnt, shape->text, &label, &map->fontset, layer->scalefactor);
+
         }
       }
+      }
 
       break;
     case(MS_SHAPE_POLYGON):
 
-      if(layer->transform) {
+      if(layer->transform == MS_TRUE) {
         msClipPolygonRect(shape, cliprect);
         if(shape->numlines == 0) return(MS_SUCCESS);
         msTransformShape(shape, map->extent, map->cellsize, image);
-      }
+      } else
+	msOffsetShapeRelativeTo(shape, layer);
 
       if(msPolygonLabelPoint(shape, &annopnt, layer->class[c].label.minfeaturesize) == MS_SUCCESS) {
 
@@ -1439,7 +1535,7 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
             label.angle -= map->gt.rotation_angle;
 
         if(layer->labelcache) {
-	  if(msAddLabel(map, layer->index, c, shape->tileindex, shape->index, &annopnt, shape->text, length, &label) != MS_SUCCESS) return(MS_FAILURE);
+          if(msAddLabel(map, layer->index, c, shape->index, shape->tileindex, &annopnt, NULL, shape->text, length, &label) != MS_SUCCESS) return(MS_FAILURE);
         } else {
 	  if(MS_VALID_COLOR(layer->class[c].styles[0].color)) {
             for(s=0; s<layer->class[c].numstyles; s++)
@@ -1457,11 +1553,12 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
 
 	  point = &(shape->line[j].point[i]);
 
-	  if(layer->transform) {
+	  if(layer->transform == MS_TRUE) {
 	    if(!msPointInRect(point, &map->extent)) return(MS_SUCCESS);
 	    point->x = MS_MAP2IMAGE_X(point->x, map->extent.minx, map->cellsize);
 	    point->y = MS_MAP2IMAGE_Y(point->y, map->extent.maxy, map->cellsize);
-	  }
+	  } else
+            msOffsetPointRelativeTo(point, layer);
 
           label = layer->class[c].label;
 
@@ -1475,7 +1572,7 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
 
 	  if(shape->text) {
 	    if(layer->labelcache) {
-	      if(msAddLabel(map, layer->index, c, shape->tileindex, shape->index, point, shape->text, -1, &label) != MS_SUCCESS) return(MS_FAILURE);
+	      if(msAddLabel(map, layer->index, c, shape->index, shape->tileindex, point, NULL, shape->text, -1, &label) != MS_SUCCESS) return(MS_FAILURE);
 	    } else {
 	      if(MS_VALID_COLOR(layer->class[c].styles[0].color)) {
                 for(s=0; s<layer->class[c].numstyles; s++)
@@ -1503,11 +1600,12 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
 
 	point = &(shape->line[j].point[i]);
 
-	if(layer->transform) {
+	if(layer->transform == MS_TRUE) {
 	  if(!msPointInRect(point, &map->extent)) continue; /* next point */
 	  point->x = MS_MAP2IMAGE_X(point->x, map->extent.minx, map->cellsize);
 	  point->y = MS_MAP2IMAGE_Y(point->y, map->extent.maxy, map->cellsize);
-	}
+	} else
+          msOffsetPointRelativeTo(point, layer);
 
 	for(s=0; s<layer->class[c].numstyles; s++) {
 	  if(layer->class[c].styles[s].angleitemindex != -1) layer->class[c].styles[s].angle = atof(shape->values[layer->class[c].styles[s].angleitemindex]);
@@ -1527,7 +1625,7 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
               label.angle -= map->gt.rotation_angle;
 
 	  if(layer->labelcache) {
-	    if(msAddLabel(map, layer->index, c, shape->tileindex, shape->index, point, shape->text, -1, &label) != MS_SUCCESS) return(MS_FAILURE);
+	    if(msAddLabel(map, layer->index, c, shape->index, shape->tileindex, point, NULL, shape->text, -1, &label) != MS_SUCCESS) return(MS_FAILURE);
 	  } else
 	    msDrawLabel(image, *point, shape->text, &label, &map->fontset, layer->scalefactor);
 	}
@@ -1548,11 +1646,12 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
       layer->project = MS_FALSE;
 #endif
 
-    if(layer->transform) {
+    if(layer->transform == MS_TRUE) {
       msClipPolylineRect(shape, cliprect);
       if(shape->numlines == 0) return(MS_SUCCESS);
       msTransformShape(shape, map->extent, map->cellsize, image);
-    }
+    } else
+      msOffsetShapeRelativeTo(shape, layer);
  
     if(style != -1) {
       if(layer->class[c].styles[style].angleitemindex != -1) layer->class[c].styles[style].angle = atof(shape->values[layer->class[c].styles[style].angleitemindex]);
@@ -1567,6 +1666,39 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
     }
 
     if(shape->text) {
+
+      /* Bug #1620 implementation */
+      if ( layer->class[c].label.autofollow == MS_TRUE ) {
+
+        annopath = msPolylineLabelPath(shape, layer->class[c].label.minfeaturesize, &(map->fontset), shape->text, &(layer->class[c].label), layer->scalefactor, &status);
+        if( annopath ) {
+
+          labelObj label;
+
+          label = layer->class[c].label;
+          /* Label path derived from line overrides the rotation and position
+             values, so ignore the specified angle and set the position to
+             auto. */
+          label.position = MS_AUTO;
+          
+          if((layer->labelsizeitemindex != -1) && (label.type == MS_TRUETYPE)) 
+            label.size = atoi(shape->values[layer->labelsizeitemindex]);
+          
+          if(layer->labelcache) {
+            if(msAddLabel(map, layer->index, c, shape->index, shape->tileindex, NULL, annopath, shape->text, length, &label) != MS_SUCCESS) return(MS_FAILURE);
+          } else {
+            /* FIXME: need to call msDrawTextLineGD() from here eventually */
+            /* msDrawLabel(image, label_line->point[0], shape->text, &label, &map->fontset, layer->scalefactor); */
+            
+            /* Free the labelpath */
+            msFreeLabelPathObj(annopath);
+          }
+        }
+      }
+
+      /* Use regular label algorithm if angle is AUTO or a number, or if ANGLE FOLLOW failed */
+      if ( layer->class[c].label.autofollow == MS_FALSE || (!annopath && status != MS_FAILURE) ) {
+
       if(msPolylineLabelPoint(shape, &annopnt, layer->class[c].label.minfeaturesize, &angle, &length) == MS_SUCCESS) {
         labelObj label = layer->class[c].label;
 
@@ -1582,11 +1714,12 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
 	if(label.autoangle) label.angle = angle;
 
 	if(layer->labelcache) {
-	  if(msAddLabel(map, layer->index, c, shape->tileindex, shape->index, &annopnt, shape->text, length, &label) != MS_SUCCESS) return(MS_FAILURE);
+            if(msAddLabel(map, layer->index, c, shape->index, shape->tileindex, &annopnt, NULL, shape->text, length, &label) != MS_SUCCESS) return(MS_FAILURE);
 	} else
           msDrawLabel(image, annopnt, shape->text, &label, &map->fontset, layer->scalefactor);
       }
     }
+    }
     break;
 
   case MS_LAYER_POLYGON:
@@ -1609,7 +1742,7 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
       layer->project = MS_FALSE;
 #endif
 
-    if(layer->transform) {
+    if(layer->transform == MS_TRUE) {
       /*
        * add a small buffer around the cliping rectangle to
        * avoid lines around the edges : Bug 179
@@ -1624,7 +1757,8 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
       msClipPolygonRect(shape, cliprect);
       if(shape->numlines == 0) return(MS_SUCCESS);
       msTransformShape(shape, map->extent, map->cellsize, image);
-    }
+    } else
+      msOffsetShapeRelativeTo(shape, layer);
  
     for(s=0; s<layer->class[c].numstyles; s++) {
       if(layer->class[c].styles[s].angleitemindex != -1) layer->class[c].styles[s].angle = atof(shape->values[layer->class[c].styles[s].angleitemindex]);
@@ -1645,7 +1779,7 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
             label.angle -= map->gt.rotation_angle;
 
 	if(layer->labelcache) {
-	  if(msAddLabel(map, layer->index, c, shape->tileindex, shape->index, &annopnt, shape->text, -1, &label) != MS_SUCCESS) return(MS_FAILURE);
+	  if(msAddLabel(map, layer->index, c, shape->index, shape->tileindex, &annopnt, NULL, shape->text, -1, &label) != MS_SUCCESS) return(MS_FAILURE);
 	} else
 	  msDrawLabel(image, annopnt, shape->text, &label, &map->fontset, layer->scalefactor);
       }
@@ -1680,15 +1814,16 @@ int msDrawPoint(mapObj *map, layerObj *layer, pointObj *point, imageObj *image,
 
   switch(layer->type) {
   case MS_LAYER_ANNOTATION:
-    if(layer->transform) {
+    if(layer->transform == MS_TRUE) {
       if(!msPointInRect(point, &map->extent)) return(0);
       point->x = MS_MAP2IMAGE_X(point->x, map->extent.minx, map->cellsize);
       point->y = MS_MAP2IMAGE_Y(point->y, map->extent.maxy, map->cellsize);
-    }
+    } else
+      msOffsetPointRelativeTo(point, layer);
 
     if(labeltext) {
       if(layer->labelcache) {
-	if(msAddLabel(map, layer->index, c, -1, -1, point, labeltext, -1,NULL) != MS_SUCCESS) return(MS_FAILURE);
+        if(msAddLabel(map, layer->index, c, -1, -1, point, NULL, labeltext, -1,NULL) != MS_SUCCESS) return(MS_FAILURE);
       } else {
 	if(MS_VALID_COLOR(layer->class[c].styles[0].color)) {
           for(s=0; s<layer->class[c].numstyles; s++)
@@ -1700,18 +1835,19 @@ int msDrawPoint(mapObj *map, layerObj *layer, pointObj *point, imageObj *image,
     break;
 
   case MS_LAYER_POINT:
-    if(layer->transform) {
+    if(layer->transform == MS_TRUE) {
       if(!msPointInRect(point, &map->extent)) return(0);
       point->x = MS_MAP2IMAGE_X(point->x, map->extent.minx, map->cellsize);
       point->y = MS_MAP2IMAGE_Y(point->y, map->extent.maxy, map->cellsize);
-    }
+    } else
+      msOffsetPointRelativeTo(point, layer);
 
     for(s=0; s<layer->class[c].numstyles; s++)
       msDrawMarkerSymbol(&map->symbolset, image, point, &(layer->class[c].styles[s]), layer->scalefactor);
 
     if(labeltext) {
       if(layer->labelcache) {
-	if(msAddLabel(map, layer->index, c, -1, -1, point, labeltext, -1,NULL) != MS_SUCCESS) return(MS_FAILURE);
+        if(msAddLabel(map, layer->index, c, -1, -1, point, NULL, labeltext, -1,NULL) != MS_SUCCESS) return(MS_FAILURE);
       } else
 	msDrawLabel(image, *point, labeltext, &layer->class[c].label, &map->fontset, layer->scalefactor);
     }
