@@ -27,9 +27,20 @@
  ******************************************************************************
  *
  * $Log: maperror.c,v $
- * Revision 1.71.2.1  2006/03/19 19:14:03  tamas
+ * Revision 1.75  2006/03/16 22:28:38  tamas
  * Fixed msGetErrorString so as not to truncate the length of the error messages
- * Added msAddErrorDisplayString to read the displayable messages
+ * Added msAddErrorDisplayString to read the displayable messages separatedly
+ *
+ * Revision 1.74  2006/03/15 18:17:14  dan
+ * Added SUPPORTS=SOS_SERVER to msGetVersion() (bug 1710)
+ *
+ * Revision 1.73  2006/03/14 03:17:19  assefa
+ * Add SOS error code (Bug 1710).
+ * Correct error codes numbers for MS_TIMEERR and MS_GMLERR.
+ *
+ * Revision 1.72  2006/02/18 21:11:35  hobu
+ * make sure we cast the tv_sec to a long because on os x, the type is
+ * really __darwin_suseconds_t, which is ultimately a long.
  *
  * Revision 1.71  2005/12/12 16:57:56  sean
  * write whether point/shape z and m is supported in version message
@@ -94,7 +105,7 @@
 #include <fcntl.h>
 #endif
 
-MS_CVSID("$Id: maperror.c,v 1.71.2.1 2006/03/19 19:14:03 tamas Exp $")
+MS_CVSID("$Id: maperror.c,v 1.75 2006/03/16 22:28:38 tamas Exp $")
 
 static char *ms_errorCodes[MS_NUMERRORCODES] = {"",
 						"Unable to access file.",
@@ -132,7 +143,8 @@ static char *ms_errorCodes[MS_NUMERRORCODES] = {"",
 						"GEOS library error.",
 						"Invalid rectangle.",
 						"Date/time error.",
-						"GML encoding error."
+						"GML encoding error.",
+                                                "SOS server error."
 };
 
 #ifndef USE_THREAD
@@ -581,6 +593,9 @@ char *msGetVersion() {
 #ifdef USE_WCS_SVR
   strcat(version, " SUPPORTS=WCS_SERVER");
 #endif
+#ifdef USE_SOS_SVR
+  strcat(version, " SUPPORTS=SOS_SERVER");
+#endif
 #ifdef USE_FASTCGI
   strcat(version, " SUPPORTS=FASTCGI");
 #endif
@@ -649,7 +664,7 @@ void msDebug( const char * pszFormat, ... )
         struct timeval tv;
         msGettimeofday(&tv, NULL);
         msIO_fprintf(stderr, "[%s].%ld ", 
-                     chop(ctime(&(tv.tv_sec))), tv.tv_usec);
+                     chop(ctime(&(tv.tv_sec))), (long)tv.tv_usec);
     }
 #endif
 
