@@ -27,6 +27,12 @@
  ******************************************************************************
  *
  * $Log: mapgdal.c,v $
+ * Revision 1.30  2005/10/31 15:58:11  frank
+ * ensure TLOCK_GDAL released on error exits
+ *
+ * Revision 1.29  2005/09/13 23:51:04  frank
+ * Pop all outstanding error handler in msGDALCleanup()
+ *
  * Revision 1.28  2005/02/18 03:06:45  dan
  * Turned all C++ (//) comments into C comments (bug 1238)
  *
@@ -122,7 +128,7 @@
 #include "map.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapgdal.c,v 1.28 2005/02/18 03:06:45 dan Exp $")
+MS_CVSID("$Id: mapgdal.c,v 1.30 2005/10/31 15:58:11 frank Exp $")
 
 #ifdef USE_GDAL
 
@@ -162,9 +168,12 @@ void msGDALCleanup( void )
 {
     if( bGDALInitialized )
     {
+        int iRepeat = 5;
+
         msAcquireLock( TLOCK_GDAL );
-        
-        CPLPopErrorHandler();
+
+        while( iRepeat-- )
+            CPLPopErrorHandler();
 
 #if GDAL_RELEASE_DATE > 20021001
         GDALDestroyDriverManager();
@@ -268,6 +277,7 @@ int msSaveImageGDAL( mapObj *map, imageObj *image, char *filename )
     hMemDriver = GDALGetDriverByName( "MEM" );
     if( hMemDriver == NULL )
     {
+        msReleaseLock( TLOCK_GDAL );
         msSetError( MS_MISCERR, "Failed to find MEM driver.",
                     "msSaveImageGDAL()" );
         return MS_FAILURE;
@@ -278,6 +288,7 @@ int msSaveImageGDAL( mapObj *map, imageObj *image, char *filename )
                          eDataType, NULL );
     if( hMemDS == NULL )
     {
+        msReleaseLock( TLOCK_GDAL );
         msSetError( MS_MISCERR, "Failed to create MEM dataset.",
                     "msSaveImageGDAL()" );
         return MS_FAILURE;
@@ -448,6 +459,7 @@ int msSaveImageGDAL( mapObj *map, imageObj *image, char *filename )
     if( hOutputDriver == NULL )
     {
         GDALClose( hMemDS );
+        msReleaseLock( TLOCK_GDAL );
         msSetError( MS_MISCERR, "Failed to find %s driver.",
                     "msSaveImageGDAL()", format->driver+5 );
         return MS_FAILURE;
@@ -465,6 +477,7 @@ int msSaveImageGDAL( mapObj *map, imageObj *image, char *filename )
     if( hOutputDS == NULL )
     {
         GDALClose( hMemDS );
+        msReleaseLock( TLOCK_GDAL );
         msSetError( MS_MISCERR, "Failed to create output %s file.\n%s",
                     "msSaveImageGDAL()", format->driver+5, 
                     CPLGetLastErrorMsg() );
