@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapgraticule.c,v 1.15.2.2 2006/01/16 20:41:22 sdlime Exp $
+ * $Id: mapgraticule.c,v 1.18 2006/01/16 20:37:15 sdlime Exp $
  *
  * Name:     mapgraticule.c
  * Project:  MapServer
@@ -29,10 +29,13 @@
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  * $Log: mapgraticule.c,v $
- * Revision 1.15.2.2  2006/01/16 20:41:22  sdlime
+ * Revision 1.18  2006/01/16 20:37:15  sdlime
+ * Changed label size calls to not adjust baseline offset.
+ *
+ * Revision 1.17  2006/01/16 20:21:18  sdlime
  * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
  *
- * Revision 1.15.2.1  2006/01/11 04:52:33  sdlime
+ * Revision 1.16  2006/01/11 04:45:36  sdlime
  * Argh! Friggin' typos on my part. Fixed bug 1256.
  *
  * Revision 1.15  2005/11/30 21:42:43  julien
@@ -75,7 +78,7 @@
 #include <assert.h>
 #include "mapproject.h"
 
-MS_CVSID("$Id: mapgraticule.c,v 1.15.2.2 2006/01/16 20:41:22 sdlime Exp $")
+MS_CVSID("$Id: mapgraticule.c,v 1.18 2006/01/16 20:37:15 sdlime Exp $")
 
 /**********************************************************************************************************************
  *
