@@ -27,11 +27,24 @@
  ******************************************************************************
  *
  * $Log: mapimagemap.c,v $
- * Revision 1.23.4.2  2006/01/25 16:10:40  dan
+ * Revision 1.29  2006/08/29 19:34:59  frank
+ * Fix C++'ism (declare values after first statement in block).
+ *
+ * Revision 1.28  2006/08/29 19:23:39  frank
+ * fixed syntax error in mapimagemap.c - affects vc++
+ *
+ * Revision 1.27  2006/08/29 02:51:24  sdlime
+ * Applied patch supplied by Vilson Farias for extra commas with imagemap output (bug 770).
+ *
+ * Revision 1.26  2006/02/18 21:24:42  hobu
+ * strlen(img->img.imagemap) is a size_t on osx.  Cast to
+ * int on os x.
+ *
+ * Revision 1.25  2006/01/25 16:11:34  dan
  * Prevent systematic buffer overflow in imagemap code when vsnprintf()
  * is not available (bug 1613)
  *
- * Revision 1.23.4.1  2006/01/16 20:41:22  sdlime
+ * Revision 1.24  2006/01/16 20:21:18  sdlime
  * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
  *
  * Revision 1.23  2005/06/14 16:03:33  dan
@@ -56,7 +69,7 @@
 
 #include "map.h"
 #include "dxfcolor.h"
-
+ 
 #include <stdarg.h>
 #include <time.h>
 
@@ -65,7 +78,7 @@
 #include <io.h>
 #endif
 
-MS_CVSID("$Id: mapimagemap.c,v 1.23.4.2 2006/01/25 16:10:40 dan Exp $")
+MS_CVSID("$Id: mapimagemap.c,v 1.29 2006/08/29 19:34:59 frank Exp $")
 
 #define MYDEBUG 0
 #define DEBUG_IF if (MYDEBUG > 2) 
@@ -1237,7 +1250,10 @@ DEBUG_IF printf("msDrawLineSymbolIM<BR>\n");
 		    } else if (dxf){
 		      im_iprintf (&imgStr, "  0\nPOLYLINE\n 70\n     0\n 62\n%6d\n  8\n%s\n", matchdxfcolor(style->color), lname);
 		    } else {
-		      char *title=(p->numvalues) ? p->values[0] : "";
+                      char *title;
+
+                      first = 1;
+		      title=(p->numvalues) ? p->values[0] : "";
 		      im_iprintf (&imgStr, "<area ");
 		      if (strcmp(polyHrefFmt,"%.s")!=0) {
 			im_iprintf (&imgStr, "href=\"");
@@ -1459,6 +1475,7 @@ DEBUG_IF printf("msDrawShadeSymbolIM\n<BR>");
 		      im_iprintf (&imgStr, "  0\nPOLYLINE\n 73\n     1\n 62\n%6d\n  8\n%s\n", matchdxfcolor(style->color), lname);
 		    } else {
 		      char *title=(p->numvalues) ? p->values[0] : "";
+                      first = 1;
 		      im_iprintf (&imgStr, "<area ");
 		      if (strcmp(polyHrefFmt,"%.s")!=0) {
 			im_iprintf (&imgStr, "href=\"");
@@ -2117,7 +2134,7 @@ if(filename != NULL && strlen(filename) > 0) {
   {
 DEBUG_IF printf("ALLOCD %d<BR>\n", img->size);
 /* DEBUG_IF printf("F %s<BR>\n", img->img.imagemap); */
-DEBUG_IF printf("FLEN %d<BR>\n", strlen(img->img.imagemap));
+DEBUG_IF printf("FLEN %d<BR>\n", (int)strlen(img->img.imagemap));
 	  if (dxf == 2){
 	    msIO_fprintf(stream, "%s", layerlist); 
 	  } else if (dxf){
