@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapio.h,v 1.4 2005/07/22 17:26:11 frank Exp $
+ * $Id: mapio.h,v 1.10 2006/08/09 14:27:00 frank Exp $
  *
  * Project:  MapServer
  * Purpose:  Definitions for MapServer IO redirection capability.
@@ -28,6 +28,24 @@
  ******************************************************************************
  *
  * $Log: mapio.h,v $
+ * Revision 1.10  2006/08/09 14:27:00  frank
+ * fixed data length handling in strip content header
+ *
+ * Revision 1.9  2006/07/13 20:18:12  frank
+ * added msIO_stripStdoutBufferContentType
+ *
+ * Revision 1.8  2006/07/05 05:54:53  frank
+ * implement per-thread io contexts
+ *
+ * Revision 1.7  2006/06/19 15:13:37  frank
+ * add io context labelling, avoid depending on function pointer compares
+ *
+ * Revision 1.6  2006/05/22 19:20:59  frank
+ * added some of the RFC 16 entry points
+ *
+ * Revision 1.5  2006/05/08 20:28:43  frank
+ * force stdin into binary mode when reading from stdin on win32 (bug 1768)
+ *
  * Revision 1.4  2005/07/22 17:26:11  frank
  * bug 1259: fixed POST support in fastcgi mode
  *
@@ -46,10 +64,13 @@
 #ifndef MAPIO_H
 #define MAPIO_H
 
-#ifdef USE_FASTCGI
+#define USE_MAPIO
+
+#if defined(USE_FASTCGI) && !defined(USE_MAPIO)
 #define USE_MAPIO
 #endif
 
+
 /*
 ** We deliberately emulate stdio functions in the msIO cover functions. 
 ** This makes it easy for folks to understand the semantics, and means we
@@ -88,6 +109,7 @@ gdIOCtx MS_DLL_EXPORT *msIO_getGDIOCtx( FILE *fp );
 typedef int (*msIO_llReadWriteFunc)( void *cbData, void *data, int byteCount );
 
 typedef struct msIOContext_t {
+    const char           *label;
     int                   write_channel; /* 1 for stdout/stderr, 0 for stdin */
     msIO_llReadWriteFunc  readWriteFunc;
     void                  *cbData;
@@ -105,8 +127,28 @@ msIOContext MS_DLL_EXPORT *msIO_getHandler( FILE * );
 int msIO_contextRead( msIOContext *context, void *data, int byteCount );
 int msIO_contextWrite( msIOContext *context, const void *data, int byteCount );
 
+/*
+** For redirecting IO to a memory buffer. 
+*/
+
+typedef struct {
+    unsigned char *data;
+    int            data_len;    /* really buffer length */
+    int            data_offset; /* really buffer used */
+} msIOBuffer;
+
+int MS_DLL_EXPORT msIO_bufferRead( void *, void *, int );
+int MS_DLL_EXPORT msIO_bufferWrite( void *, void *, int );
+
+void MS_DLL_EXPORT msIO_resetHandlers(void);
+void MS_DLL_EXPORT msIO_installStdoutToBuffer(void);
+void MS_DLL_EXPORT msIO_installStdinFromBuffer(void);
+void MS_DLL_EXPORT msIO_Cleanup();
+char MS_DLL_EXPORT *msIO_stripStdoutBufferContentType(void);
+
 /* this is just for setting normal stdout's to binary mode on windows */
 
 int msIO_needBinaryStdout( void );
+int msIO_needBinaryStdin( void );
 
 #endif /* nef MAPIO_H */
