@@ -1,7 +1,7 @@
 /******************************************************************************
  *
  * Project:  MapServer
- * Purpose:  Labelling Implementation.
+ * Purpose:  Labeling Implementation.
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
@@ -27,9 +27,27 @@
  ******************************************************************************
  *
  * $Log: maplabel.c,v $
- * Revision 1.79.2.1  2006/01/16 20:41:22  sdlime
+ * Revision 1.86  2006/09/06 05:26:18  sdlime
+ * Fixed a bug with curved labels where character advance array was not being gdFree'd. (bug 1899)
+ *
+ * Revision 1.85  2006/08/17 04:32:16  sdlime
+ * Disable path following labels unless GD 2.0.29 or greater is available.
+ *
+ * Revision 1.84  2006/03/23 20:28:52  sdlime
+ * Most recent patch for curved labels. (bug 1620)
+ *
+ * Revision 1.83  2006/03/22 23:31:20  sdlime
+ * Applied latest patch for curved labels. (bug 1620)
+ *
+ * Revision 1.82  2006/02/18 20:59:13  sdlime
+ * Initial code for curved labels. (bug 1620)
+ *
+ * Revision 1.81  2006/01/16 20:21:18  sdlime
  * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
  *
+ * Revision 1.80  2006/01/12 05:26:23  sdlime
+ * Fixed spelling error in module name and replaced a // style comment.
+ *
  * Revision 1.79  2005/11/28 04:24:59  sdlime
  * Changed msAddLabel() to use msCopyStyle() rather than doing on its own. This should fix some de-allocation errors folks have been having. (bug 1398)
  *
@@ -64,9 +82,9 @@
 
 #include "map.h"
 
-MS_CVSID("$Id: maplabel.c,v 1.79.2.1 2006/01/16 20:41:22 sdlime Exp $")
+MS_CVSID("$Id: maplabel.c,v 1.86 2006/09/06 05:26:18 sdlime Exp $")
 
-int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int tileindex, pointObj *point, char *string, double featuresize, labelObj *label )
+int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int tileindex, pointObj *point, labelPathObj *labelpath, char *string, double featuresize, labelObj *label )
 {
   int i;
   char wrap[2];
@@ -98,9 +116,22 @@ int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int
   cachePtr->tileindex = tileindex;
   cachePtr->shapeindex = shapeindex;
 
+  /* Store the label point or the label path (Bug #1620) */
+  if ( point ) {
+
   cachePtr->point = *point; /* the actual label point */
   cachePtr->point.x = MS_NINT(cachePtr->point.x);
   cachePtr->point.y = MS_NINT(cachePtr->point.y);
+    cachePtr->labelpath = NULL;
+    
+  } else if ( labelpath ) {
+    int i;
+    cachePtr->labelpath = labelpath;
+    /* Use the middle point of the labelpath for mindistance calculations */
+    i = labelpath->path.numpoints / 2;
+    cachePtr->point.x = MS_NINT(labelpath->path.point[i].x);
+    cachePtr->point.y = MS_NINT(labelpath->path.point[i].y);
+  }
 
   cachePtr->text = strdup(string); /* the actual text */
 
@@ -288,7 +319,7 @@ int msLoadFontSet(fontSetObj *fontset, mapObj *map)
 ** LL corner of the text to be rendered, this is first line for TrueType fonts.
 */
 
- /* assumes an angle of 0 regardless of the settings in the labelObj */
+/* assumes an angle of 0 regardless of what's in the label object */
 int msGetLabelSize(char *string, labelObj *label, rectObj *rect, fontSetObj *fontset, double scalefactor, int adjustBaseline)
 {
   int size;
@@ -311,6 +342,7 @@ int msGetLabelSize(char *string, labelObj *label, rectObj *rect, fontSetObj *fon
       return(-1);
     }
 
+
     error = gdImageStringFT(NULL, bbox, 0, font, size, 0, 0, 0, string);
     if(error) {
       msSetError(MS_TTFERR, error, "msGetLabelSize()");
@@ -322,10 +354,10 @@ int msGetLabelSize(char *string, labelObj *label, rectObj *rect, fontSetObj *fon
     rect->maxx = bbox[2];
     rect->maxy = bbox[1];
 
-    // bug 1449 fix (adjust baseline)
+    /* bug 1449 fix (adjust baseline) */
     if(adjustBaseline) {
       label->offsety += MS_NINT(((bbox[5] + bbox[1]) + size) / 2);
-      label->offsetx += MS_NINT(bbox[0] / 2); // optional?
+      label->offsetx += MS_NINT(bbox[0] / 2);
     }
 #else
     msSetError(MS_TTFERR, "TrueType font support is not available.", "msGetLabelSize()");
@@ -362,6 +394,73 @@ int msGetLabelSize(char *string, labelObj *label, rectObj *rect, fontSetObj *fon
   return(0);
 }
 
+/*
+ * Return the size of the label, plus an array of individual character
+ * offsets, as calculated by gdImageStringFTEx().  The callee is
+ * responsible for freeing *offsets.
+ */
+int msGetLabelSizeEx(char *string, labelObj *label, rectObj *rect, fontSetObj *fontset, double scalefactor, int adjustBaseline, double **offsets) 
+{
+#if defined (USE_GD_FT) && defined (GD_HAS_FTEX_XSHOW)
+    int size;
+    char *s;
+    int k;
+    int bbox[8];
+    char *error=NULL, *font=NULL;
+    gdFTStringExtra strex;
+    
+    size = MS_NINT(label->size*scalefactor);
+    size = MS_MAX(size, label->minsize);
+    size = MS_MIN(size, label->maxsize);
+
+    font = msLookupHashTable(&(fontset->fonts), label->font);
+    if(!font) {
+      if(label->font) 
+        msSetError(MS_TTFERR, "Requested font (%s) not found.", "msGetLabelSizeEx()", label->font);
+      else
+        msSetError(MS_TTFERR, "Requested font (NULL) not found.", "msGetLabelSizeEx()");
+      return(-1);
+    }
+
+    strex.flags = gdFTEX_XSHOW;
+    error = gdImageStringFTEx(NULL, bbox, 0, font, size, 0, 0, 0, string, &strex);
+    if(error) {
+      msSetError(MS_TTFERR, error, "msGetLabelSizeEx()");
+      return(-1);
+    }
+
+    *offsets = (double*)malloc( strlen(string) * sizeof(double) );
+    
+    s = strex.xshow;
+    k = 0;
+    while ( *s && k < strlen(string) ) {
+      (*offsets)[k++] = atof(s);      
+      while ( *s && *s != ' ')
+        s++;
+      if ( *s == ' ' )
+        s++;
+    }
+
+    gdFree(strex.xshow); /* done with character advances */
+
+    rect->minx = bbox[0];
+    rect->miny = bbox[5];
+    rect->maxx = bbox[2];
+    rect->maxy = bbox[1];
+
+    /* bug 1449 fix (adjust baseline) */
+    if(adjustBaseline) {
+      label->offsety += MS_NINT(((bbox[5] + bbox[1]) + size) / 2);
+      label->offsetx += MS_NINT(bbox[0] / 2);
+    }
+
+    return MS_SUCCESS;
+#else
+    msSetError(MS_TTFERR, "TrueType font support is not available or is not current enough (requires 2.0.29 or higher).", "msGetLabelSizeEx()");
+    return(-1);
+#endif
+}
+
 gdFontPtr msGetBitmapFont(int size)
 {
   switch(size) { /* set the font to use */
