@@ -27,6 +27,21 @@
  ******************************************************************************
  *
  * $Log: maplayer.c,v $
+ * Revision 1.112  2006/08/22 13:56:58  hobu
+ * ssh!  comment out "tile;" which was unused and
+ * causing compiler to complain
+ *
+ * Revision 1.111  2006/04/27 04:05:17  sdlime
+ * Initial support for relative coordinates. (bug 1547)
+ *
+ * Revision 1.110  2006/03/28 17:31:30  assefa
+ * When setting the time filter, do not override existing Filter paramter
+ * if possible (Bug 1261).
+ *
+ * Revision 1.109  2006/01/31 17:06:10  assefa
+ * Add connectiontype initialization logic when the layer's virtual
+ * table is initialized (Bug 1615)
+ *
  * Revision 1.108  2005/10/29 02:03:43  jani
  * MS RFC 8: Pluggable External Feature Layer Providers (bug 1477).
  *
@@ -76,7 +91,7 @@
 #include "mapogcfilter.h"
 
 #include <assert.h>
-MS_CVSID("$Id: maplayer.c,v 1.108 2005/10/29 02:03:43 jani Exp $")
+MS_CVSID("$Id: maplayer.c,v 1.112 2006/08/22 13:56:58 hobu Exp $")
 
 
 static int populateVirtualTable(layerVTableObj *vtable);
@@ -725,9 +740,8 @@ makeTimeFilter(layerObj *lp,
   
     char **atimes, **tokens = NULL;
     int numtimes,i, ntmp = 0;
-    char buffer[512];
-
-    buffer[0] = '\0';
+    char *pszBuffer = NULL;
+    int bOnlyExistingFilter = 0;
 
     if (!lp || !timestring || !timefield)
       return MS_FALSE;
@@ -738,42 +752,67 @@ makeTimeFilter(layerObj *lp,
 
     if (strstr(timestring, ",") == NULL && 
         strstr(timestring, "/") == NULL) /* discrete time */
-    {
+    {   
+        /*
         if(lp->filteritem) free(lp->filteritem);
         lp->filteritem = strdup(timefield);
         if (&lp->filter)
           freeExpression(&lp->filter);
+        */
+
+        if (&lp->filter)
+        {
+            /* if the filter is set and it's a sting type, concatenate it with
+               the time. If not just free it */
+            if (lp->filter.type == MS_EXPRESSION)
+            {
+                pszBuffer = strcatalloc(pszBuffer, "((");
+                pszBuffer = strcatalloc(pszBuffer, lp->filter.string);
+                pszBuffer = strcatalloc(pszBuffer, ") and ");
+            }
+            else
+            {
+                freeExpression(&lp->filter);
+            }
+        }
         
+        pszBuffer = strcatalloc(pszBuffer, "(");
+        if (addtimebacktics)
+          pszBuffer = strcatalloc(pszBuffer,  "`");
 
-        strcat(buffer, "(");
         if (addtimebacktics)
-          strcat(buffer,  "`");
-
-         if (addtimebacktics)
-           strcat(buffer, "[");
-        strcat(buffer, timefield);
+           pszBuffer = strcatalloc(pszBuffer, "[");
+        pszBuffer = strcatalloc(pszBuffer, (char *)timefield);
         if (addtimebacktics)
-          strcat(buffer, "]");
+          pszBuffer = strcatalloc(pszBuffer, "]");
         if (addtimebacktics)
-          strcat(buffer,  "`");
+          pszBuffer = strcatalloc(pszBuffer,  "`");
 
          
-        strcat(buffer, " = ");
+        pszBuffer = strcatalloc(pszBuffer, " = ");
         if (addtimebacktics)
-          strcat(buffer,  "`");
+          pszBuffer = strcatalloc(pszBuffer,  "`");
         else
-          strcat(buffer,  "'");
+          pszBuffer = strcatalloc(pszBuffer,  "'");
 
-        strcat(buffer, timestring);
+        pszBuffer = strcatalloc(pszBuffer, (char *)timestring);
         if (addtimebacktics)
-          strcat(buffer,  "`");
+          pszBuffer = strcatalloc(pszBuffer,  "`");
         else
-          strcat(buffer,  "'");
+          pszBuffer = strcatalloc(pszBuffer,  "'");
 
-        strcat(buffer, ")");
+        pszBuffer = strcatalloc(pszBuffer, ")");
         
-        /* loadExpressionString(&lp->filter, (char *)timestring); */
-        loadExpressionString(&lp->filter, buffer);
+        /* if there was a filter, It was concatenate with an And ans should be closed*/
+        if(&lp->filter && lp->filter.type == MS_EXPRESSION)
+        {
+            pszBuffer = strcatalloc(pszBuffer, ")");
+        }
+
+        loadExpressionString(&lp->filter, pszBuffer);
+
+        if (pszBuffer)
+          msFree(pszBuffer);
 
         return MS_TRUE;
     }
@@ -784,114 +823,128 @@ makeTimeFilter(layerObj *lp,
 
     if (numtimes >= 1)
     {
+        if (&lp->filter && lp->filter.type == MS_EXPRESSION)
+        {
+            pszBuffer = strcatalloc(pszBuffer, "((");
+            pszBuffer = strcatalloc(pszBuffer, lp->filter.string);
+            pszBuffer = strcatalloc(pszBuffer, ") and ");
+            /*this flag is used to indicate that the buffer contains only the 
+              existing filter. It is set to 0 when time filter parts are
+              added to the buffer */
+            bOnlyExistingFilter = 1;
+        }
+        else
+          freeExpression(&lp->filter);
+
         /* check to see if we have ranges by parsing the first entry */
         tokens = split(atimes[0],  '/', &ntmp);
         if (ntmp == 2) /* ranges */
         {
-                
             msFreeCharArray(tokens, ntmp);
-             for (i=0; i<numtimes; i++)
-             {
+            for (i=0; i<numtimes; i++)
+            {
                  tokens = split(atimes[i],  '/', &ntmp);
                  if (ntmp == 2)
                  {
-                     if (strlen(buffer) > 0)
-                       strcat(buffer, " OR ");
+                     if (pszBuffer && strlen(pszBuffer) > 0 && bOnlyExistingFilter == 0)
+                       pszBuffer = strcatalloc(pszBuffer, " OR ");
                      else
-                       strcat(buffer, "(");
+                       pszBuffer = strcatalloc(pszBuffer, "(");
 
-                     strcat(buffer, "(");
+                     bOnlyExistingFilter = 0;
+
+                     pszBuffer = strcatalloc(pszBuffer, "(");
                      if (addtimebacktics)
-                       strcat(buffer,  "`");
+                       pszBuffer = strcatalloc(pszBuffer,  "`");
 
                      if (addtimebacktics)
-                       strcat(buffer, "[");
-                     strcat(buffer, timefield);
+                       pszBuffer = strcatalloc(pszBuffer, "[");
+                     pszBuffer = strcatalloc(pszBuffer, (char *)timefield);
                      if (addtimebacktics)
-                       strcat(buffer, "]");
+                       pszBuffer = strcatalloc(pszBuffer, "]");
                      
                      if (addtimebacktics)
-                       strcat(buffer,  "`");
+                       pszBuffer = strcatalloc(pszBuffer,  "`");
 
-                     strcat(buffer, " >= ");
+                     pszBuffer = strcatalloc(pszBuffer, " >= ");
                      if (addtimebacktics)
-                       strcat(buffer,  "`");
+                       pszBuffer = strcatalloc(pszBuffer,  "`");
                      else
-                       strcat(buffer,  "'");
+                       pszBuffer = strcatalloc(pszBuffer,  "'");
 
-                     strcat(buffer, tokens[0]);
+                     pszBuffer = strcatalloc(pszBuffer, tokens[0]);
                      if (addtimebacktics)
-                       strcat(buffer,  "`");
+                       pszBuffer = strcatalloc(pszBuffer,  "`");
                      else
-                       strcat(buffer,  "'");
-                     strcat(buffer, " AND ");
+                       pszBuffer = strcatalloc(pszBuffer,  "'");
+                     pszBuffer = strcatalloc(pszBuffer, " AND ");
 
                      if (addtimebacktics)
-                       strcat(buffer,  "`");
+                       pszBuffer = strcatalloc(pszBuffer,  "`");
 
                      if (addtimebacktics)
-                       strcat(buffer, "[");
-                     strcat(buffer, timefield);
+                       pszBuffer = strcatalloc(pszBuffer, "[");
+                     pszBuffer = strcatalloc(pszBuffer, (char *)timefield);
                      if (addtimebacktics)
-                       strcat(buffer, "]");
+                       pszBuffer = strcatalloc(pszBuffer, "]");
                      if (addtimebacktics)
-                       strcat(buffer,  "`");
+                       pszBuffer = strcatalloc(pszBuffer,  "`");
 
-                     strcat(buffer, " <= ");
+                     pszBuffer = strcatalloc(pszBuffer, " <= ");
                      
                      if (addtimebacktics)
-                       strcat(buffer,  "`");
+                       pszBuffer = strcatalloc(pszBuffer,  "`");
                      else
-                       strcat(buffer,  "'");
-                     strcat(buffer, tokens[1]);
+                       pszBuffer = strcatalloc(pszBuffer,  "'");
+                     pszBuffer = strcatalloc(pszBuffer, tokens[1]);
                      if (addtimebacktics)
-                       strcat(buffer,  "`");
+                       pszBuffer = strcatalloc(pszBuffer,  "`");
                      else
-                       strcat(buffer,  "'");
-                     strcat(buffer, ")");
+                       pszBuffer = strcatalloc(pszBuffer,  "'");
+                     pszBuffer = strcatalloc(pszBuffer, ")");
                  }
                  
                   msFreeCharArray(tokens, ntmp);
-             }
-             if (strlen(buffer) > 0)
-               strcat(buffer, ")");
+            }
+            if (pszBuffer && strlen(pszBuffer) > 0 && bOnlyExistingFilter == 0)
+              pszBuffer = strcatalloc(pszBuffer, ")");
         }
         else if (ntmp == 1) /* multiple times */
         {
-             msFreeCharArray(tokens, ntmp);
-             strcat(buffer, "(");
-             for (i=0; i<numtimes; i++)
-             {
-                 if (i > 0)
-                   strcat(buffer, " OR ");
+            msFreeCharArray(tokens, ntmp);
+            pszBuffer = strcatalloc(pszBuffer, "(");
+            for (i=0; i<numtimes; i++)
+            {
+                if (i > 0)
+                  pszBuffer = strcatalloc(pszBuffer, " OR ");
 
-                  strcat(buffer, "(");
-                  if (addtimebacktics)
-                    strcat(buffer, "`");
+                pszBuffer = strcatalloc(pszBuffer, "(");
+                if (addtimebacktics)
+                  pszBuffer = strcatalloc(pszBuffer, "`");
                   
-                  if (addtimebacktics)
-                    strcat(buffer, "[");
-                  strcat(buffer, timefield);
-                  if (addtimebacktics)
-                    strcat(buffer, "]");
+                if (addtimebacktics)
+                  pszBuffer = strcatalloc(pszBuffer, "[");
+                pszBuffer = strcatalloc(pszBuffer, (char *)timefield);
+                if (addtimebacktics)
+                  pszBuffer = strcatalloc(pszBuffer, "]");
 
-                  if (addtimebacktics)
-                    strcat(buffer, "`");
+                if (addtimebacktics)
+                  pszBuffer = strcatalloc(pszBuffer, "`");
 
-                  strcat(buffer, " = ");
+                pszBuffer = strcatalloc(pszBuffer, " = ");
                   
-                  if (addtimebacktics)
-                    strcat(buffer, "`");
-                  else
-                    strcat(buffer,  "'");
-                  strcat(buffer, atimes[i]);
-                  if (addtimebacktics)
-                    strcat(buffer,  "`");
-                  else
-                    strcat(buffer,  "'");
-                  strcat(buffer, ")");
-             } 
-              strcat(buffer, ")");
+                if (addtimebacktics)
+                  pszBuffer = strcatalloc(pszBuffer, "`");
+                else
+                  pszBuffer = strcatalloc(pszBuffer,  "'");
+                pszBuffer = strcatalloc(pszBuffer, atimes[i]);
+                if (addtimebacktics)
+                  pszBuffer = strcatalloc(pszBuffer,  "`");
+                else
+                  pszBuffer = strcatalloc(pszBuffer,  "'");
+                pszBuffer = strcatalloc(pszBuffer, ")");
+            } 
+            pszBuffer = strcatalloc(pszBuffer, ")");
         }
         else
         {
@@ -902,14 +955,20 @@ makeTimeFilter(layerObj *lp,
         msFreeCharArray(atimes, numtimes);
 
         /* load the string to the filter */
-        if (strlen(buffer) > 0)
+        if (pszBuffer && strlen(pszBuffer) > 0)
         {
+            if(&lp->filter && lp->filter.type == MS_EXPRESSION)
+              pszBuffer = strcatalloc(pszBuffer, ")");
+            /*
             if(lp->filteritem) 
               free(lp->filteritem);
-            lp->filteritem = strdup(timefield);     
-            if (&lp->filter)
-              freeExpression(&lp->filter);
-            loadExpressionString(&lp->filter, buffer);
+            lp->filteritem = strdup(timefield);
+            */     
+
+            loadExpressionString(&lp->filter, pszBuffer);
+
+            if (pszBuffer)
+              msFree(pszBuffer);
         }
 
         return MS_TRUE;
@@ -1138,7 +1197,7 @@ static int
 createVirtualTable(layerVTableObj **vtable)
 {
     *vtable = malloc(sizeof(**vtable));
-    if ( ! vtable) {
+    if ( ! *vtable) {
         return MS_FAILURE;
     }
     return populateVirtualTable(*vtable);
@@ -1161,6 +1220,17 @@ msInitializeVirtualTable(layerObj *layer)
     }
     createVirtualTable(&layer->vtable);
 
+    
+    if(layer->features && layer->connectiontype != MS_GRATICULE ) 
+      layer->connectiontype = MS_INLINE;
+
+    if(layer->tileindex && layer->connectiontype == MS_SHAPEFILE)
+      layer->connectiontype = MS_TILED_SHAPEFILE;
+
+    if(layer->type == MS_LAYER_RASTER )
+      layer->connectiontype = MS_RASTER;
+    
+
     switch(layer->connectiontype) {
         case(MS_INLINE):
             return(msINLINELayerInitializeVirtualTable(layer));
@@ -1238,7 +1308,7 @@ int msINLINELayerGetShape(layerObj *layer, shapeObj *shape, int tile, long shape
     int i=0;
     featureListNodeObjPtr current;
 
-    tile; /* Not used */
+    /* tile ; */ /* Not used -- commented out to silence compiler warning.  hobu*/
     current = layer->features;
     while (current!=NULL && i!=shapeindex) {
         i++;
@@ -1260,11 +1330,14 @@ int msINLINELayerGetShape(layerObj *layer, shapeObj *shape, int tile, long shape
 int msINLINELayerNextShape(layerObj *layer, shapeObj *shape) 
 {
     if( ! (layer->currentfeature)) {
-        /* out of features     */
+        /* out of features */
         return(MS_DONE); 
     }
+
     msCopyShape(&(layer->currentfeature->shape), shape);
+
     layer->currentfeature = layer->currentfeature->next;
+
     return(MS_SUCCESS);
 }
 
