@@ -27,10 +27,19 @@
  ******************************************************************************
  *
  * $Log: maplegend.c,v $
- * Revision 1.57.2.2  2006/03/27 05:50:52  sdlime
+ * Revision 1.62  2006/08/04 04:36:12  sdlime
+ * Removed cut & paste artifact (oops).
+ *
+ * Revision 1.61  2006/08/04 04:33:45  sdlime
+ * Fixed a problem with full-image legends where layer sizeunits were not taken into account when rendering a legend. (bug 1147)
+ *
+ * Revision 1.60  2006/03/27 05:48:03  sdlime
  * Fixed symbol initialization error with embedded scalebars and legends. (bug 1725)
  *
- * Revision 1.57.2.1  2006/01/16 20:41:22  sdlime
+ * Revision 1.59  2006/02/18 20:59:13  sdlime
+ * Initial code for curved labels. (bug 1620)
+ *
+ * Revision 1.58  2006/01/16 20:21:18  sdlime
  * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
  *
  * Revision 1.57  2005/11/17 05:56:30  sdlime
@@ -74,7 +83,7 @@
 
 #include "map.h"
 
-MS_CVSID("$Id: maplegend.c,v 1.57.2.2 2006/03/27 05:50:52 sdlime Exp $")
+MS_CVSID("$Id: maplegend.c,v 1.62 2006/08/04 04:36:12 sdlime Exp $")
 
 #define PSF .8
 #define VMARGIN 5 /* margin at top and bottom of legend graphic */
@@ -124,7 +133,7 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *class, int width, int
     if(type == MS_LAYER_POLYGON) {
       type = MS_LAYER_LINE;
       for(i=0; i<class->numstyles; i++) {
-        if(MS_VALID_COLOR(class->styles[i].color)) { /* there is a fill */
+       if(MS_VALID_COLOR(class->styles[i].color)) { /* there is a fill */
 	  type = MS_LAYER_POLYGON;
 	  break;
         }
@@ -147,7 +156,7 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *class, int width, int
       marker.y = dstY + MS_NINT(height / 2.0);
 
       for(i=0; i<class->numstyles; i++)
-        msDrawMarkerSymbolGD(&map->symbolset, img, &marker, &(class->styles[i]), 1.0);          
+        msDrawMarkerSymbolGD(&map->symbolset, img, &marker, &(class->styles[i]), lp->scalefactor);          
       break;
     case MS_LAYER_LINE:
       zigzag.line = (lineObj *)malloc(sizeof(lineObj));
@@ -166,7 +175,7 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *class, int width, int
       zigzag.line[0].numpoints = 4;
 
       for(i=0; i<class->numstyles; i++)
-        msDrawLineSymbolGD(&map->symbolset, img, &zigzag, &(class->styles[i]), 1.0); 
+        msDrawLineSymbolGD(&map->symbolset, img, &zigzag, &(class->styles[i]), lp->scalefactor); 
 
       free(zigzag.line[0].point);
       free(zigzag.line);	
@@ -175,7 +184,7 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *class, int width, int
     case MS_LAYER_RASTER:
     case MS_LAYER_POLYGON:
       for(i=0; i<class->numstyles; i++)     
-        msDrawShadeSymbolGD(&map->symbolset, img, &box, &(class->styles[i]), 1.0);
+        msDrawShadeSymbolGD(&map->symbolset, img, &box, &(class->styles[i]), lp->scalefactor);
       break;
     default:
       return MS_FAILURE;
@@ -203,12 +212,9 @@ imageObj *msCreateLegendIcon(mapObj* map, layerObj* lp, classObj* class, int wid
   outputFormatObj *format = NULL;
   int i = 0;
 
-  if(!map->outputformat || !MS_RENDERER_GD(map->outputformat) ) 
-  {
-      msSetError(MS_GDERR, 
-                 "Map outputformat must be set to a GD format!", 
-                 "msCreateLegendIcon()");
-      return(NULL);
+  if(!map->outputformat || !MS_RENDERER_GD(map->outputformat) ) {
+    msSetError(MS_GDERR, "Map outputformat must be set to a GD format!", "msCreateLegendIcon()");
+    return(NULL);
   }
 
   /* ensure we have an image format representing the options for the legend */
@@ -236,8 +242,7 @@ imageObj *msCreateLegendIcon(mapObj* map, layerObj* lp, classObj* class, int wid
     msClearLayerPenValues(lp); /* just in case the mapfile has already been processed */
     if (class) {
       msDrawLegendIcon(map, lp, class, width, height, image->img.gd, 0, 0);
-    }
-    else {
+    } else {
       for (i=0; i<lp->numclasses; i++) {
         msDrawLegendIcon(map, lp, &lp->class[i], width, height, image->img.gd, 0, 0);
       }
@@ -247,7 +252,6 @@ imageObj *msCreateLegendIcon(mapObj* map, layerObj* lp, classObj* class, int wid
   return image;
 }
 
-
 /*
 ** Creates a GD image of a legend for a specific map. msDrawLegend()
 ** respects the current scale, and classes without a name are not
@@ -274,9 +278,9 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
   outputFormatObj *format = NULL;
 
   if (!scale_independent) {
-      map->cellsize = msAdjustExtent(&(map->extent), map->width, map->height);
-      status = msCalculateScale(map->extent, map->units, map->width, map->height, map->resolution, &map->scale);
-      if(status != MS_SUCCESS) return(NULL);
+    map->cellsize = msAdjustExtent(&(map->extent), map->width, map->height);
+    status = msCalculateScale(map->extent, map->units, map->width, map->height, map->resolution, &map->scale);
+    if(status != MS_SUCCESS) return(NULL);
   }
 
   if(msValidateContexts(map) != MS_SUCCESS) return NULL; /* make sure there are no recursive REQUIRES or LABELREQUIRES expressions */
@@ -291,8 +295,7 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
       continue;
 
     for(j=0;j<lp->numclasses;j++) {
-      if(!lp->class[j].name)
-	continue; /* skip it */
+      if(!lp->class[j].name) continue; /* skip it */
       n++;
     }
   }
@@ -313,10 +316,8 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
       continue;
 
     if(!scale_independent && map->scale > 0) {
-      if((lp->maxscale > 0) && (map->scale > lp->maxscale))
-	continue;
-      if((lp->minscale > 0) && (map->scale <= lp->minscale))
-	continue;
+      if((lp->maxscale > 0) && (map->scale > lp->maxscale)) continue;
+      if((lp->minscale > 0) && (map->scale <= lp->minscale)) continue;
     }
  
     for(j=0;j<lp->numclasses;j++) {
@@ -332,6 +333,7 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
 
       if(msGetLabelSize(lp->class[j].name, &map->legend.label, &rect, &(map->fontset), 1.0, MS_FALSE) != 0)
 	return(NULL); /* something bad happened */
+
       maxheight = MS_MAX(maxheight, MS_NINT(rect.maxy - rect.miny));
       maxwidth = MS_MAX(maxwidth, MS_NINT(rect.maxx - rect.minx));
       heights[n] = MS_NINT(rect.maxy - rect.miny);
@@ -383,6 +385,10 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
 	continue;
       if((lp->minscale > 0) && (map->scale <= lp->minscale))
 	continue;
+
+      /* Should we also consider lp->symbolscale? I don't think so. Showing the "standard" size makes the most sense. */
+      if(lp->sizeunits != MS_PIXELS)
+        lp->scalefactor = (msInchesPerUnit(lp->sizeunits,0)/msInchesPerUnit(map->units,0)) / map->cellsize;
     }
 
     for(j=0; j<lp->numclasses; j++) { /* always at least 1 class */
@@ -399,8 +405,8 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
       pnt.x = HMARGIN + map->legend.keysizex + map->legend.keyspacingx;
       
       /* TODO */
-      if (msDrawLegendIcon(map, lp, &(lp->class[j]),  map->legend.keysizex,  map->legend.keysizey, image->img.gd, HMARGIN, (int) pnt.y) != MS_SUCCESS)
-         return NULL;
+      if(msDrawLegendIcon(map, lp, &(lp->class[j]),  map->legend.keysizex,  map->legend.keysizey, image->img.gd, HMARGIN, (int) pnt.y) != MS_SUCCESS)
+        return NULL;
 
       pnt.y += MS_MAX(map->legend.keysizey, maxheight);
       /* TODO */
@@ -415,7 +421,6 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
   return(image);
 }
 
-
 /* TODO */
 int msEmbedLegend(mapObj *map, gdImagePtr img)
 {
@@ -441,6 +446,7 @@ int msEmbedLegend(mapObj *map, gdImagePtr img)
   image->img.gd = NULL;
   msFreeImage( image );
 
+
   if(!map->symbolset.symbol[s].img) return(-1); /* something went wrong creating scalebar */
 
   map->symbolset.symbol[s].type = MS_SYMBOL_PIXMAP; /* intialize a few things */
@@ -506,7 +512,7 @@ int msEmbedLegend(mapObj *map, gdImagePtr img)
   if(map->legend.postlabelcache) /* add it directly to the image */
     msDrawMarkerSymbolGD(&map->symbolset, img, &point, &(map->layers[l].class[0].styles[0]), 1.0);
   else
-    msAddLabel(map, l, 0, -1, -1, &point, " ", 1.0, NULL);
+    msAddLabel(map, l, 0, -1, -1, &point, NULL, " ", 1.0, NULL);
 
   /* Mark layer as deleted so that it doesn't interfere with html legends or */
   /* with saving maps */
