@@ -28,6 +28,19 @@
  ******************************************************************************
  *
  * $Log: mapobject.c,v $
+ * Revision 1.27  2006/09/24 03:31:35  frank
+ * Same as last fix, but for y dimension, instead of just x.
+ *
+ * Revision 1.26  2006/09/23 16:00:51  frank
+ * Fixed computation of geotransform to match BBOX (to edges of image) not
+ * map.extent (to center of edge pixels).  (bug 1916)
+ *
+ * Revision 1.25  2006/08/03 00:22:00  sdlime
+ * Fixed problem where insertLayer does not set the layer index properly if the layer is inserted between existing layers. (bug 1838)
+ *
+ * Revision 1.24  2006/03/31 15:53:03  julien
+ * bug 1733 Fix SLD nonsquare pixel and SLD with FE projection issue
+ *
  * Revision 1.23  2005/10/26 17:40:03  frank
  * Avoid unused variable warning.
  *
@@ -106,7 +119,7 @@
 #  include "cpl_conv.h"
 #endif
 
-MS_CVSID("$Id: mapobject.c,v 1.23 2005/10/26 17:40:03 frank Exp $")
+MS_CVSID("$Id: mapobject.c,v 1.27 2006/09/24 03:31:35 frank Exp $")
 
 void freeWeb(webObj *web);
 void freeScalebar(scalebarObj *scalebar);
@@ -360,18 +373,23 @@ int msMapComputeGeotransform( mapObj * map )
     center_x = map->extent.minx + geo_width*0.5;
     center_y = map->extent.miny + geo_height*0.5;
 
+    /*
+    ** Per bug 1916 we have to adjust for the fact that map extents
+    ** are based on the center of the edge pixels, not the outer
+    ** edges as is expected in a geotransform.
+    */
     map->gt.geotransform[1] = 
-        cos(rot_angle) * geo_width / map->width;
+        cos(rot_angle) * geo_width / (map->width-1);
     map->gt.geotransform[2] = 
-        sin(rot_angle) * geo_height / map->height;
+        sin(rot_angle) * geo_height / (map->height-1);
     map->gt.geotransform[0] = center_x 
         - (map->width * 0.5) * map->gt.geotransform[1]
         - (map->height * 0.5) * map->gt.geotransform[2];
 
     map->gt.geotransform[4] = 
-        sin(rot_angle) * geo_width / map->width;
+        sin(rot_angle) * geo_width / (map->width-1);
     map->gt.geotransform[5] = 
-        - cos(rot_angle) * geo_height / map->height;
+        - cos(rot_angle) * geo_height / (map->height-1);
     map->gt.geotransform[3] = center_y 
         - (map->width * 0.5) * map->gt.geotransform[4]
         - (map->height * 0.5) * map->gt.geotransform[5];
@@ -410,6 +428,7 @@ void msMapGeorefToPixel( mapObj *map, double *x, double *y )
 int msMapSetFakedExtent( mapObj *map )
 
 {
+    int i;
 /* -------------------------------------------------------------------- */
 /*      Remember the original map extents so we can restore them        */
 /*      later.                                                          */
@@ -445,6 +464,9 @@ int msMapSetFakedExtent( mapObj *map )
     map->projection.gt.geotransform[2] *= -1;
     map->projection.gt.geotransform[5] *= -1;
 
+    for(i=0; i<map->numlayers; i++)
+        map->layers[i].project = MS_TRUE;
+
     return InvGeoTransform( map->projection.gt.geotransform, 
                             map->projection.gt.invgeotransform );
 }
@@ -514,7 +536,7 @@ int msInsertLayer(mapObj *map, layerObj *layer, int nIndex)
         freeLayer(&(map->layers[nIndex]));
         initLayer(&(map->layers[nIndex]), map);
         msCopyLayer(&(map->layers[nIndex]), layer);
-        map->layers[map->numlayers].index = nIndex;
+        map->layers[nIndex].index = nIndex;
 
         /* adjust layers drawing order */
         for (i=map->numlayers; i>nIndex; i--) {
