@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapogr.cpp,v 1.93.2.2 2006/01/18 04:40:56 frank Exp $
+ * $Id: mapogr.cpp,v 1.102 2006/09/05 20:39:07 frank Exp $
  *
  * Name:     mapogr.cpp
  * Project:  MapServer
@@ -29,10 +29,33 @@
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  * $Log: mapogr.cpp,v $
- * Revision 1.93.2.2  2006/01/18 04:40:56  frank
- * finished implementation of ogr shape2wkt
+ * Revision 1.102  2006/09/05 20:39:07  frank
+ * added support for multipoint and multiline to wkt translation (bug 1618)
  *
- * Revision 1.93.2.1  2006/01/17 02:31:31  frank
+ * Revision 1.101  2006/09/05 15:32:38  frank
+ * added support for translating multipoint to ms shapes (bug 1891)
+ *
+ * Revision 1.100  2006/09/04 17:05:35  dan
+ * Added support for MULTIPOLYGON and MULTILINESTRING in msShapeFromWKT()
+ * when going through OGR (i.e. GEOS disabled) (bug 1891)
+ *
+ * Revision 1.99  2006/06/01 20:49:21  dan
+ * Enabled support for encrypted connection strings (MS-RFC-18, bug 1792)
+ *
+ * Revision 1.98  2006/05/29 20:31:20  dan
+ * Make sure oPenColor is always initialized (avoid warning on MacOSX)
+ *
+ * Revision 1.97  2006/05/19 20:53:27  dan
+ * Use lp->layerinfo for OGR connections (instead of ogrlayerinfo) (bug 331)
+ *
+ * Revision 1.96  2006/04/08 03:36:48  frank
+ * Ensure that an error in GetNextFeature() will be properly translated
+ * into a MapServer error in GetNextShape().
+ *
+ * Revision 1.95  2006/01/18 04:35:52  frank
+ * completed implementation of shape2wkt
+ *
+ * Revision 1.94  2006/01/17 02:31:51  frank
  * fixed ogr wkt support - bug 1614
  *
  * Revision 1.93  2005/11/01 19:38:13  frank
@@ -144,7 +167,7 @@
 #  include "cpl_string.h"
 #endif
 
-MS_CVSID("$Id: mapogr.cpp,v 1.93.2.2 2006/01/18 04:40:56 frank Exp $")
+MS_CVSID("$Id: mapogr.cpp,v 1.102 2006/09/05 20:39:07 frank Exp $")
 
 #ifdef USE_OGR
 
@@ -605,15 +628,15 @@ int msOGRGeometryToShape(OGRGeometryH hGeometry, shapeObj *psShape,
 {
     if (hGeometry && psShape && nType > 0)
     {
-        if (nType == wkbPoint)
-          return ogrConvertGeometry((OGRGeometry *)hGeometry,
-                                    psShape,  MS_LAYER_POINT);
-        else if (nType == wkbLineString)
-          return ogrConvertGeometry((OGRGeometry *)hGeometry,
-                                    psShape,  MS_LAYER_LINE);
-        else if (nType == wkbPolygon)
-          return ogrConvertGeometry((OGRGeometry *)hGeometry,
-                                    psShape,  MS_LAYER_POLYGON);
+        if (nType == wkbPoint || nType == wkbMultiPoint )
+            return ogrConvertGeometry((OGRGeometry *)hGeometry,
+                                      psShape,  MS_LAYER_POINT);
+        else if (nType == wkbLineString || nType == wkbMultiLineString)
+            return ogrConvertGeometry((OGRGeometry *)hGeometry,
+                                      psShape,  MS_LAYER_LINE);
+        else if (nType == wkbPolygon || nType == wkbMultiPolygon)
+            return ogrConvertGeometry((OGRGeometry *)hGeometry,
+                                      psShape,  MS_LAYER_POLYGON);
         else
             return MS_FAILURE;
     }
@@ -839,6 +862,8 @@ static msOGRFileInfo *
 msOGRFileOpen(layerObj *layer, const char *connection ) 
 
 {
+  char *conn_decrypted = NULL;
+
 /* ------------------------------------------------------------------
  * Register OGR Drivers, only once per execution
  * ------------------------------------------------------------------ */
@@ -857,25 +882,35 @@ msOGRFileOpen(layerObj *layer, const char *connection )
       bOGRDriversRegistered = MS_TRUE;
   }
 
+/* ------------------------------------------------------------------
+ * Make sure any encrypted token in the connection string are decrypted
+ * ------------------------------------------------------------------ */
+  if (connection)
+  {
+      conn_decrypted = msDecryptStringTokens(layer->map, connection);
+      if (conn_decrypted == NULL)
+          return NULL;  /* An error should already have been reported */
+  }
+
 /* ------------------------------------------------------------------
  * Parse connection string into dataset name, and layer name. 
  * ------------------------------------------------------------------ */
   char *pszDSName = NULL, *pszLayerDef = NULL;
 
-  if( connection == NULL )
+  if( conn_decrypted == NULL )
   {
       /* we don't have anything */
   }
   else if( layer->data != NULL )
   {
-      pszDSName = CPLStrdup(connection);
+      pszDSName = CPLStrdup(conn_decrypted);
       pszLayerDef = CPLStrdup(layer->data);
   }
   else
   {
       char **papszTokens = NULL;
 
-      papszTokens = CSLTokenizeStringComplex( connection, ",", TRUE, FALSE );
+      papszTokens = CSLTokenizeStringComplex( conn_decrypted, ",", TRUE, FALSE );
 
       if( CSLCount(papszTokens) > 0 )
           pszDSName = CPLStrdup( papszTokens[0] );
@@ -885,6 +920,12 @@ msOGRFileOpen(layerObj *layer, const char *connection )
       CSLDestroy(papszTokens);
   }
 
+  /* Get rid of decrypted connection string. We'll use the original (not
+   * decrypted) string for debug and error messages in the rest of the code.
+   */
+  msFree(conn_decrypted);
+  conn_decrypted = NULL;
+
   if( pszDSName == NULL )
   {
       msSetError(MS_OGRERR, 
@@ -1223,7 +1264,14 @@ msOGRFileNextShape(layerObj *layer, shapeObj *shape,
 
       if( (poFeature = psInfo->poLayer->GetNextFeature()) == NULL )
       {
-          return MS_DONE;  // No more features to read
+          if( CPLGetLastErrorType() == CE_Failure )
+          {
+              msSetError(MS_OGRERR, "%s", "msOGRFileNextShape()",
+                         CPLGetLastErrorMsg() );
+              return MS_FAILURE;
+          }
+          else
+              return MS_DONE;  // No more features to read
       }
 
       if(layer->numitems > 0) 
@@ -1480,7 +1528,7 @@ int msOGRLayerOpen(layerObj *layer, const char *pszOverrideConnection)
 
   msOGRFileInfo *psInfo;
 
-  if (layer->ogrlayerinfo != NULL)
+  if (layer->layerinfo != NULL)
   {
       return MS_SUCCESS;  // Nothing to do... layer is already opened
   }
@@ -1493,10 +1541,10 @@ int msOGRLayerOpen(layerObj *layer, const char *pszOverrideConnection)
       psInfo = msOGRFileOpen( layer, 
                               (pszOverrideConnection ? pszOverrideConnection:
                                                        layer->connection) );
-      layer->ogrlayerinfo = psInfo;
+      layer->layerinfo = psInfo;
       layer->tileitemindex = -1;
       
-      if( layer->ogrlayerinfo == NULL )
+      if( layer->layerinfo == NULL )
           return MS_FAILURE;
   }
 
@@ -1510,9 +1558,9 @@ int msOGRLayerOpen(layerObj *layer, const char *pszOverrideConnection)
       // Open tile index
 
       psInfo = msOGRFileOpen( layer, layer->tileindex );
-      layer->ogrlayerinfo = psInfo;
+      layer->layerinfo = psInfo;
       
-      if( layer->ogrlayerinfo == NULL )
+      if( layer->layerinfo == NULL )
           return MS_FAILURE;
 
       // Identify TILEITEM
@@ -1531,7 +1579,7 @@ int msOGRLayerOpen(layerObj *layer, const char *pszOverrideConnection)
                      "msOGRLayerOpen()", 
                      layer->tileitem, layer->tileindex );
           msOGRFileClose( layer, psInfo );
-          layer->ogrlayerinfo = NULL;
+          layer->layerinfo = NULL;
           return MS_FAILURE;
       }
   }
@@ -1561,7 +1609,7 @@ int msOGRLayerOpen(layerObj *layer, const char *pszOverrideConnection)
                      (pszOverrideConnection ? pszOverrideConnection:
                                               layer->connection) );
           msOGRFileClose( layer, psInfo );
-          layer->ogrlayerinfo = NULL;
+          layer->layerinfo = NULL;
           return(MS_FAILURE);
       }
   }
@@ -1596,7 +1644,7 @@ static int msOGRLayerOpenVT(layerObj *layer)
 int msOGRLayerClose(layerObj *layer) 
 {
 #ifdef USE_OGR
-  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->ogrlayerinfo;
+  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->layerinfo;
 
   if (psInfo)
   {
@@ -1604,7 +1652,7 @@ int msOGRLayerClose(layerObj *layer)
           msDebug("msOGRLayerClose(%s).\n", layer->connection);
 
       msOGRFileClose( layer, psInfo );
-      layer->ogrlayerinfo = NULL;
+      layer->layerinfo = NULL;
   }
 
   return MS_SUCCESS;
@@ -1626,7 +1674,7 @@ int msOGRLayerClose(layerObj *layer)
 static int msOGRLayerIsOpen(layerObj *layer) 
 {
 #ifdef USE_OGR
-  if (layer->ogrlayerinfo)
+  if (layer->layerinfo)
       return MS_TRUE;
 
   return MS_FALSE;
@@ -1653,7 +1701,7 @@ static int msOGRLayerIsOpen(layerObj *layer)
 int msOGRLayerWhichShapes(layerObj *layer, rectObj rect) 
 {
 #ifdef USE_OGR
-  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->ogrlayerinfo;
+  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->layerinfo;
   int   status;
 
   if (psInfo == NULL || psInfo->poLayer == NULL)
@@ -1695,7 +1743,7 @@ int msOGRLayerWhichShapes(layerObj *layer, rectObj rect)
 int msOGRLayerGetItems(layerObj *layer)
 {
 #ifdef USE_OGR
-  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->ogrlayerinfo;
+  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->layerinfo;
   
   if( layer->tileindex != NULL )
   {
@@ -1736,7 +1784,7 @@ int msOGRLayerGetItems(layerObj *layer)
 static int msOGRLayerInitItemInfo(layerObj *layer)
 {
 #ifdef USE_OGR
-  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->ogrlayerinfo;
+  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->layerinfo;
   int   i;
   OGRFeatureDefn *poDefn;
 
@@ -1846,7 +1894,7 @@ void msOGRLayerFreeItemInfo(layerObj *layer)
 int msOGRLayerNextShape(layerObj *layer, shapeObj *shape) 
 {
 #ifdef USE_OGR
-  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->ogrlayerinfo;
+  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->layerinfo;
   int  status;
 
   if (psInfo == NULL || psInfo->poLayer == NULL)
@@ -1905,7 +1953,7 @@ int msOGRLayerGetShape(layerObj *layer, shapeObj *shape, int tile,
                        long record)
 {
 #ifdef USE_OGR
-  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->ogrlayerinfo;
+  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->layerinfo;
 
   if (psInfo == NULL || psInfo->poLayer == NULL)
   {
@@ -1949,7 +1997,7 @@ int msOGRLayerGetShape(layerObj *layer, shapeObj *shape, int tile,
 int msOGRLayerGetExtent(layerObj *layer, rectObj *extent) 
 {
 #ifdef USE_OGR
-  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->ogrlayerinfo;
+  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->layerinfo;
   OGREnvelope oExtent;
 
   if (psInfo == NULL || psInfo->poLayer == NULL)
@@ -2046,7 +2094,7 @@ static int msOGRLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c,
                                   int tile, long record)
 {
 #ifdef USE_OGR
-  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->ogrlayerinfo;
+  msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->layerinfo;
 
   if (psInfo == NULL || psInfo->poLayer == NULL)
   {
@@ -2186,6 +2234,9 @@ static int msOGRLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c,
               int nPenSymbol = 0;
               int nPenSize = 1;
 
+              // Make sure pen is always initialized
+              MS_INIT_COLOR(oPenColor, -1, -1, -1);
+
               // Check for Pen Pattern "ogr-pen-1": the invisible pen
               // If that's what we have then set pen color to -1
               if (pszPenName && strstr(pszPenName, "ogr-pen-1") != NULL)
@@ -2455,6 +2506,28 @@ char *msOGRShapeToWKT(shapeObj *shape)
                         0.0 );
 #endif
     }
+    else if( shape->type == MS_SHAPE_POINT && shape->numlines == 1 
+             && shape->line[0].numpoints > 1 )
+    {
+        hGeom = OGR_G_CreateGeometry( wkbMultiPoint );
+        for( i = 0; i < shape->line[0].numpoints; i++ )
+        {
+            OGRGeometryH hPoint;
+            
+            hPoint = OGR_G_CreateGeometry( wkbPoint );
+#if GDAL_VERSION_NUM >= 1310
+            OGR_G_SetPoint_2D( hPoint, 0, 
+                               shape->line[0].point[i].x, 
+                               shape->line[0].point[i].y );
+#else
+            OGR_G_SetPoint( hPoint, 0, 
+                            shape->line[0].point[i].x, 
+                            shape->line[0].point[i].y,
+                            0.0 );
+#endif
+            OGR_G_AddGeometryDirectly( hGeom, hPoint );
+        }
+    }
     else if( shape->type == MS_SHAPE_LINE && shape->numlines == 1 )
     {
         hGeom = OGR_G_CreateGeometry( wkbLineString );
@@ -2472,6 +2545,33 @@ char *msOGRShapeToWKT(shapeObj *shape)
 #endif
         }
     }
+    else if( shape->type == MS_SHAPE_LINE && shape->numlines > 1 )
+    {
+        OGRGeometryH hMultiLine = OGR_G_CreateGeometry( wkbMultiLineString );
+        int iLine;
+
+        for( iLine = 0; iLine < shape->numlines; iLine++ )
+        {
+            hGeom = OGR_G_CreateGeometry( wkbLineString );
+            for( i = 0; i < shape->line[iLine].numpoints; i++ )
+            {
+#if GDAL_VERSION_NUM >= 1310
+                OGR_G_AddPoint_2D( hGeom, 
+                                   shape->line[iLine].point[i].x, 
+                                   shape->line[iLine].point[i].y );
+#else
+                OGR_G_AddPoint( hGeom, 
+                            shape->line[iLine].point[i].x, 
+                            shape->line[iLine].point[i].y,
+                            0.0 );
+#endif
+            }
+
+            OGR_G_AddGeometryDirectly( hMultiLine, hGeom );
+        }
+
+        hGeom = hMultiLine;
+    }
     else if( shape->type == MS_SHAPE_POLYGON )
     {
         int iLine;
