@@ -27,6 +27,9 @@
  ******************************************************************************
  *
  * $Log: mapoutput.c,v $
+ * Revision 1.36  2006/07/11 01:09:42  frank
+ * msCloneOutputFormat() needs to propagate inmapfile flag (bug 1823).
+ *
  * Revision 1.35  2005/11/10 15:41:16  frank
  * Added missing function header.
  *
@@ -142,7 +145,7 @@
 #include <assert.h>
 #include "map.h"
 
-MS_CVSID("$Id: mapoutput.c,v 1.35 2005/11/10 15:41:16 frank Exp $")
+MS_CVSID("$Id: mapoutput.c,v 1.36 2006/07/11 01:09:42 frank Exp $")
 
 static outputFormatObj *msAllocOutputFormat( mapObj *map, const char *name, 
                                              const char *driver );
@@ -762,6 +765,8 @@ outputFormatObj *msCloneOutputFormat( outputFormatObj *src )
     for( i = 0; i < src->numformatoptions; i++ )
         dst->formatoptions[i] = strdup(src->formatoptions[i]);
 
+    dst->inmapfile = src->inmapfile;
+
     return dst;
 }
 
