@@ -27,6 +27,29 @@
  ******************************************************************************
  *
  * $Log: mapows.c,v $
+ * Revision 1.67  2006/08/23 18:28:00  dan
+ * Use OWS_DEFAULT_SCHEMAS_LOCATION #define instead of hardcoded string (bug 1873)
+ *
+ * Revision 1.66  2006/08/23 17:52:06  dan
+ * schemas.opengeospatial.net has been shutdown, use schemas.opengis.net
+ * instead as the default schema repository for OGC services (bug 1873)
+ *
+ * Revision 1.65  2006/06/06 14:21:01  frank
+ * ensure msOWSDispatch() always available
+ *
+ * Revision 1.64  2006/05/09 14:33:41  assefa
+ * WFS client/OWS : correct path concatenation logic for temporary gml file
+ * created (bug 1770).
+ *
+ * Revision 1.63  2006/03/15 18:00:22  assefa
+ * Use flag SOS_SVR instead of OGC_SOS (Bug 1712).
+ *
+ * Revision 1.62  2006/03/14 04:08:34  assefa
+ * Add disptach call to SOS service.
+ *
+ * Revision 1.61  2006/02/14 03:38:47  julien
+ * Update to MapContext 1.1.0, add dimensions support in context bug 1581
+ *
  * Revision 1.60  2005/06/14 16:03:34  dan
  * Updated copyright date to 2005
  *
@@ -236,13 +259,7 @@
 #include <stdarg.h> 
 #include <assert.h>
 
-MS_CVSID("$Id: mapows.c,v 1.60 2005/06/14 16:03:34 dan Exp $")
-
-#if defined(USE_WMS_SVR) || defined (USE_WFS_SVR) || defined (USE_WCS_SVR)
-
-#if !defined(USE_PROJ)
-#error "PROJ.4 is required for WMS, WFS and WCS Server Support."
-#endif
+MS_CVSID("$Id: mapows.c,v 1.67 2006/08/23 18:28:00 dan Exp $")
 
 /*
 ** msOWSDispatch() is the entry point for any OWS request (WMS, WFS, ...)
@@ -272,9 +289,20 @@ int msOWSDispatch(mapObj *map, cgiRequestObj *request)
         return status;
 #endif
 
+#ifdef USE_SOS_SVR
+    if ((status = msSOSDispatch(map, request)) != MS_DONE )
+        return status;
+#endif
+
     return MS_DONE;  /* Not a WMS/WFS request... let MapServer handle it */
 }
 
+#if defined(USE_WMS_SVR) || defined (USE_WFS_SVR) || defined (USE_WCS_SVR) || defined(USE_SOS_SVR)
+
+#if !defined(USE_PROJ)
+#error "PROJ.4 is required for WMS, WFS and WCS Server Support."
+#endif
+
 /*
 ** msRenameLayer()
 */
@@ -480,7 +508,7 @@ const char *msOWSGetSchemasLocation(mapObj *map)
     schemas_location = msLookupHashTable(&(map->web.metadata), 
                                          "ows_schemas_location");
     if (schemas_location == NULL)
-      schemas_location = "http://schemas.opengeospatial.net"; /* ".."; */
+      schemas_location = OWS_DEFAULT_SCHEMAS_LOCATION;
 
     return schemas_location;
 }
@@ -599,6 +627,11 @@ const char *msOWSLookupMetadata(hashTableObj *metadata,
                 buf[1] = 'm';
                 buf[2] = 'l';
                 break;
+                case 'S':         /* sos_... */
+                buf[0] = 's';
+                buf[1] = 'o';
+                buf[2] = 's';
+                break;
               default:
                 /* We should never get here unless an invalid code (typo) is */
                 /* present in the code, but since this happened before... */
@@ -653,6 +686,7 @@ int msOWSPrintMetadata(FILE *stream, hashTableObj *metadata,
     return status;
 }
 
+
 /*
 ** msOWSPrintEncodeMetadata()
 **
@@ -697,6 +731,29 @@ int msOWSPrintEncodeMetadata(FILE *stream, hashTableObj *metadata,
     return status;
 }
 
+
+/*
+** msOWSGetEncodeMetadata()
+**
+** Equivalent to msOWSPrintEncodeMetadata. Returns en encoded value of the
+** metadata or the default value.
+** Caller should free the returned string.
+*/
+char *msOWSGetEncodeMetadata(hashTableObj *metadata, 
+                             const char *namespaces, const char *name, 
+                             const char *default_value)
+{
+    const char *value;
+    char * pszEncodedValue=NULL;    
+    if((value = msOWSLookupMetadata(metadata, namespaces, name)))
+      pszEncodedValue = msEncodeHTMLEntities(value);
+    else if (default_value)
+      pszEncodedValue = msEncodeHTMLEntities(default_value);
+
+    return pszEncodedValue;
+}
+      
+
 /*
 ** msOWSPrintValidateMetadata()
 **
@@ -1550,6 +1607,7 @@ char *msOWSBuildURLFilename(const char *pszPath, const char *pszURL,
     char *pszBuf, *pszPtr;
     int  i, nBufLen;
 
+
     nBufLen = strlen(pszURL) + strlen(pszExt) +1;
     if (pszPath)
         nBufLen += (strlen(pszPath)+1);
@@ -1565,9 +1623,16 @@ char *msOWSBuildURLFilename(const char *pszPath, const char *pszURL,
     if (pszPath)
     {
 #ifdef _WIN32
-        sprintf(pszBuf, "%s\\", pszPath);
+        if (pszPath[strlen(pszPath) -1] != '/' &&
+            pszPath[strlen(pszPath) -1] != '\\')
+          sprintf(pszBuf, "%s\\", pszPath);
+        else
+          sprintf(pszBuf, "%s", pszPath);
 #else
-        sprintf(pszBuf, "%s/", pszPath);
+        if (pszPath[strlen(pszPath) -1] != '/')
+          sprintf(pszBuf, "%s/", pszPath);
+        else
+          sprintf(pszBuf, "%s", pszPath);
 #endif
     }
 
@@ -1625,6 +1690,98 @@ const char *msOWSGetEPSGProj(projectionObj *proj, hashTableObj *metadata, const
   return NULL;
 }
 
+
+/*
+** msOWSGetDimensionInfo()
+**
+** Extract dimension information from a layer's metadata
+**
+** Before 4.9, only the time dimension was support. With the addition of
+** Web Map Context 1.1.0, we need to support every dimension types. 
+** This function get the dimension information from special metadata in
+** the layer, but can also return default values for the time dimension.
+** 
+*/
+void msOWSGetDimensionInfo(layerObj *layer, const char *pszDimension, 
+                           const char **papszDimUserValue, 
+                           const char **papszDimUnits, 
+                           const char **papszDimDefault, 
+                           const char **papszDimNearValue, 
+                           const char **papszDimUnitSymbol, 
+                           const char **papszDimMultiValue)
+{
+    char *pszDimensionItem;
+
+    if(pszDimension == NULL || layer == NULL)
+        return;
+
+    pszDimensionItem = (char*)malloc(strlen(pszDimension)+50);
+
+    /* units (mandatory in map context) */
+    if(papszDimUnits != NULL)
+    {
+        sprintf(pszDimensionItem, "dimension_%s_units",          pszDimension);
+        *papszDimUnits = msOWSLookupMetadata(&(layer->metadata), "MO",
+                                           pszDimensionItem);
+    }
+    /* unitSymbol (mandatory in map context) */
+    if(papszDimUnitSymbol != NULL)
+    {
+        sprintf(pszDimensionItem, "dimension_%s_unitsymbol",     pszDimension);
+        *papszDimUnitSymbol = msOWSLookupMetadata(&(layer->metadata), "MO", 
+                                                  pszDimensionItem);
+    }
+    /* userValue (mandatory in map context) */
+    if(papszDimUserValue != NULL)
+    {
+        sprintf(pszDimensionItem, "dimension_%s_uservalue",      pszDimension);
+        *papszDimUserValue = msOWSLookupMetadata(&(layer->metadata), "MO", 
+                                                 pszDimensionItem);
+    }
+    /* default */
+    if(papszDimDefault != NULL)
+    {
+        sprintf(pszDimensionItem, "dimension_%s_default",        pszDimension);
+        *papszDimDefault = msOWSLookupMetadata(&(layer->metadata), "MO",
+                                               pszDimensionItem);
+    }
+    /* multipleValues */
+    if(papszDimMultiValue != NULL)
+    {
+        sprintf(pszDimensionItem, "dimension_%s_multiplevalues", pszDimension);
+        *papszDimMultiValue = msOWSLookupMetadata(&(layer->metadata), "MO", 
+                                                  pszDimensionItem);
+    }
+    /* nearestValue */
+    if(papszDimNearValue != NULL)
+    {
+        sprintf(pszDimensionItem, "dimension_%s_nearestvalue",   pszDimension);
+        *papszDimNearValue = msOWSLookupMetadata(&(layer->metadata), "MO", 
+                                                 pszDimensionItem);
+    }
+
+    /* Use default time value if necessary */
+    if(strcasecmp(pszDimension, "time") == 0)
+    {
+        if(papszDimUserValue != NULL && *papszDimUserValue == NULL)
+            *papszDimUserValue = msOWSLookupMetadata(&(layer->metadata), 
+                                                   "MO", "time");
+        if(papszDimDefault != NULL && *papszDimDefault == NULL)
+            *papszDimDefault = msOWSLookupMetadata(&(layer->metadata), 
+                                                 "MO", "timedefault");
+        if(papszDimUnits != NULL && *papszDimUnits == NULL)
+            *papszDimUnits = "ISO8601";
+        if(papszDimUnitSymbol != NULL && *papszDimUnitSymbol == NULL)
+            *papszDimUnitSymbol = "t";
+        if(papszDimNearValue != NULL && *papszDimNearValue == NULL)
+            *papszDimNearValue = "0";
+    }
+
+    free(pszDimensionItem);
+
+    return;
+}
+
 #endif /* USE_WMS_SVR || USE_WFS_SVR  || USE_WCS_SVR */
 
 
