@@ -1,10 +1,53 @@
 /**********************************************************************
- * $Id: mapows.h,v 1.64 2006/01/05 16:25:10 assefa Exp $
+ * $Id: mapows.h,v 1.78 2006/08/28 21:16:21 assefa Exp $
  *
  * mapows.h - OGC Web Services (WMS, WFS, WCS) support function definitions
  *
  **********************************************************************
  * $Log: mapows.h,v $
+ * Revision 1.78  2006/08/28 21:16:21  assefa
+ * add flag USE_SOS_SVR as part of the flags enabling ows utility functions
+ * to be defined.
+ *
+ * Revision 1.77  2006/08/23 18:28:00  dan
+ * Use OWS_DEFAULT_SCHEMAS_LOCATION #define instead of hardcoded string (bug 1873)
+ *
+ * Revision 1.76  2006/08/22 04:45:06  sdlime
+ * Fixed a bug that did not allow for seperate metadata namespaces to be used for WMS vs. WFS for GML transformations (e.g. WFS_GEOMETRIES, WMS_GEOMETRIES).
+ *
+ * Revision 1.75  2006/08/14 18:47:28  sdlime
+ * Some more implementation of external application schema support.
+ *
+ * Revision 1.74  2006/08/11 18:56:40  sdlime
+ * Initial versions of namespace read/free functions.
+ *
+ * Revision 1.73  2006/08/11 18:40:20  sdlime
+ * Created gmlNamespaceObj structures.
+ *
+ * Revision 1.72  2006/06/06 14:21:01  frank
+ * ensure msOWSDispatch() always available
+ *
+ * Revision 1.71  2006/04/17 19:06:49  dan
+ * Set User-Agent in HTTP headers of client WMS/WFS connections (bug 1749)
+ *
+ * Revision 1.70  2006/03/14 04:08:34  assefa
+ * Add disptach call to SOS service.
+ *
+ * Revision 1.69  2006/02/14 03:39:30  julien
+ * Update to MapContext 1.1.0, add dimensions support in context bug 1581
+ *
+ * Revision 1.68  2006/02/07 17:40:23  sdlime
+ * Reverting to Dan's original solution for the template member.
+ *
+ * Revision 1.67  2006/02/07 16:13:37  sdlime
+ * Renamed gmlItemObj template to tmplate to avoid c++ compile errors.
+ *
+ * Revision 1.66  2006/02/07 13:45:32  dan
+ * Use _template instead of template in gmlItemObj in C++
+ *
+ * Revision 1.65  2006/02/06 19:50:41  sdlime
+ * Added ability to define a item template for GML output, best for use with application schema. A templated attribute is: 1) not queryable and 2) not output in the server produced schema. The layer namespace and item value can be accessed via the template: e.g. gml_area_template '<$namespace:area>$value</$namespace:area>
+ *
  * Revision 1.64  2006/01/05 16:25:10  assefa
  * Correct mapscript windows build problem when flag USE_WMS_SVR was
  * not set (Bug 1529)
@@ -55,123 +98,6 @@
  * Revision 1.49  2005/02/18 03:06:46  dan
  * Turned all C++ (//) comments into C comments (bug 1238)
  *
- * Revision 1.48  2004/12/29 22:49:57  sdlime
- * Added GML3 writing capabilities to mapgml.c. Not hooked up to anything yet.
- *
- * Revision 1.47  2004/11/25 06:19:05  dan
- * Add trailing "?" or "&" to connection string when required in WFS
- * client layers using GET method (bug 1082)
- *
- * Revision 1.46  2004/11/16 21:57:49  dan
- * Final pass at updating WMS/WFS client/server interfaces to lookup "ows_*"
- * metadata in addition to default "wms_*"/"wfs_*" metadata (bug 568)
- *
- * Revision 1.45  2004/11/15 20:35:02  dan
- * Added msLayerIsOpen() to all vector layer types (bug 1051)
- *
- * Revision 1.44  2004/11/02 21:01:00  assefa
- * Add a 2nd optional argument to msLoadMapContext function (Bug 1023).
- *
- * Revision 1.43  2004/10/25 17:30:38  julien
- * Print function for OGC URLs components. msOWSPrintURLType() (Bug 944)
- *
- * Revision 1.42  2004/10/15 20:29:03  assefa
- * Add support for OGC mapcontext through mapserver cgi : Bug 946.
- *
- * Revision 1.41  2004/09/25 17:16:31  julien
- * Don't encode XML tag (Bug 897)
- * Don't compile mapgml.c function if not necessary (Bug 896)
- *
- * Revision 1.40  2004/09/23 19:18:10  julien
- * Encode all metadata and parameter printed in an XML document (Bug 802)
- *
- * Revision 1.39  2004/09/08 14:33:30  sean
- * declared MS_DLL_EXPORT for hex2int and msGMLWriteQuery (bug 851).
- *
- * Revision 1.38  2004/08/03 23:26:24  dan
- * Cleanup OWS version tests in the code, mapwms.c (bug 799)
- *
- * Revision 1.37  2004/08/03 22:12:34  dan
- * Cleanup OWS version tests in the code, started with mapcontext.c (bug 799)
- *
- * Revision 1.36  2004/06/22 22:22:16  sean
- * set MS_DLL_EXPORT for msWMSLoadGetMapParams
- *
- * Revision 1.35  2004/06/22 20:55:20  sean
- * Towards resolving issue 737 changed hashTableObj to a structure which contains a hashObj **items.  Changed all hash table access functions to operate on the target table by reference.  msFreeHashTable should not be used on the hashTableObj type members of mapserver structures, use msFreeHashItems instead.
- *
- * Revision 1.34  2004/05/22 15:51:10  sean
- * Prototype msWMSLoadGetMapParams
- *
- * Revision 1.33  2004/05/11 19:13:46  sean
- * Changes to function prototypes to reduce number of SWIG-mapscript build warnings (bug 568) and committed changes to fix bug 650, WMS layer drawing
- *
- * Revision 1.32  2004/05/03 03:45:42  dan
- * Include map= param in default onlineresource of GetCapabilties if it
- * was explicitly set in QUERY_STRING (bug 643)
- *
- * Revision 1.31  2004/04/19 22:08:39  sdlime
- * Added msOWSGetEPSGProj() to mapows.h/.c and updated the original from mapproject.c to use Dan's namespaces.
- *
- * Revision 1.30  2004/04/14 07:31:40  dan
- * Removed msOWSGetMetadata(), replaced by msOWSLookupMetadata()
- *
- * Revision 1.29  2004/04/14 05:14:54  dan
- * Added ability to pass a default value to msOWSPrintMetadataList() (bug 616)
- *
- * Revision 1.28  2004/04/14 04:54:30  dan
- * Created msOWSLookupMetadata() and added namespaces lookup in all
- * msOWSPrint*Metadata() functions. Also pass namespaces=NULL everywhere
- * that calls those functions for now to avoid breaking something just
- * before the release. (bug 615, 568)
- *
- * Revision 1.27  2004/04/12 18:38:24  assefa
- * Add dll export support for windows.
- *
- * Revision 1.26  2004/03/30 00:12:28  dan
- * Added ability to combine multiple WMS connections to the same server
- * into a single request when the layers are adjacent and compatible.(bug 116)
- *
- * Revision 1.25  2004/03/29 18:34:25  assefa
- * Windows compilation problem : gettimeofday and timval struct (Bug 602)
- *
- * Revision 1.24  2004/03/29 14:41:55  dan
- * Use CURL's internal timer instead of custom gettimeofday() calls for
- * timing WMS/WFS requests
- *
- * Revision 1.23  2004/03/18 23:11:12  dan
- * Added detailed reporting (using msDebug) of layer rendering times
- *
- * Revision 1.22  2004/03/11 22:45:39  dan
- * Added pszPostContentType in httpRequestObj instead of using hardcoded
- * text/html mime type for all post requests.
- *
- * Revision 1.21  2004/02/24 06:20:37  sdlime
- * Added msOWSGetMetadata() function.
- *
- * Revision 1.20  2004/02/05 06:05:54  sdlime
- * Added first WCS prototype to mapows.h
- *
- * Revision 1.19  2003/10/30 22:37:01  assefa
- * Add function msWFSExecuteGetFeature on a wfs layer.
- *
- * Revision 1.18  2003/10/06 13:03:19  assefa
- * Use of namespace. Correct execption return.
- *
- * Revision 1.17  2003/09/19 21:54:19  assefa
- * Add support fot the Post request.
- *
- * Revision 1.16  2003/04/09 07:13:49  dan
- * Added GetContext (custom) request in WMS interface.
- * Added missing gml: namespace in 0.1.7 context output.
- *
- * Revision 1.15  2003/03/26 20:24:38  dan
- * Do not call msDebug() unless debug flag is turned on
- *
- * Revision 1.14  2003/01/10 06:39:06  sdlime
- * Moved msEncodeHTMLEntities() and msDecodeHTMLEntities() from mapows.c 
- * to mapstring.c so they can be used a bit more freely.
- *
  * ...
  *
  * Revision 1.1  2002/09/03 03:19:51  dan
@@ -184,6 +110,12 @@
 
 #include <time.h>
 
+/* This is the URL to the official OGC Schema Repository. We use it by 
+ * default for OGC services unless the ows_schemas_lcoation web metadata 
+ * is set in the mapfile.
+ */
+#define OWS_DEFAULT_SCHEMAS_LOCATION   "http://schemas.opengis.net"
+
 /*====================================================================
  *   maphttp.c
  *====================================================================*/
@@ -192,16 +124,17 @@
 
 typedef struct http_request_info
 {
-    int         nLayerId;
-    char      * pszGetUrl;
-    char      * pszOutputFile;
-    int         nTimeout;
-    rectObj     bbox;
-    int         nStatus;       /* 200=success, value < 0 if request failed */
-    char      * pszContentType;/* Content-Type of the response */
-    char      * pszErrBuf;     /* Buffer where curl can write errors */
-    char        *pszPostRequest;     /* post request content (NULL for GET) */
-    char        *pszPostContentType; /* post request MIME type */
+    int     nLayerId;
+    char    *pszGetUrl;
+    char    *pszOutputFile;
+    int     nTimeout;
+    rectObj bbox;
+    int     nStatus;            /* 200=success, value < 0 if request failed */
+    char    *pszContentType;    /* Content-Type of the response */
+    char    *pszErrBuf;         /* Buffer where curl can write errors */
+    char    *pszPostRequest;    /* post request content (NULL for GET) */
+    char    *pszPostContentType;/* post request MIME type */
+    char    *pszUserAgent;      /* User-Agent, auto-generated if not set */
 
     /* For debugging/profiling */
     int         debug;         /* Debug mode?  MS_TRUE/MS_FALSE */
@@ -252,9 +185,10 @@ int  msHTTPGetFile(const char *pszGetUrl, const char *pszOutputFile,
 /*====================================================================
  *   mapows.c
  *====================================================================*/
-#if defined(USE_WMS_SVR) || defined (USE_WFS_SVR) || defined (USE_WCS_SVR)
-
 MS_DLL_EXPORT int msOWSDispatch(mapObj *map, cgiRequestObj *request);
+
+#if defined(USE_WMS_SVR) || defined (USE_WFS_SVR) || defined (USE_WCS_SVR) || defined(USE_SOS_SVR)
+
 MS_DLL_EXPORT int msOWSMakeAllLayersUnique(mapObj *map);
 MS_DLL_EXPORT char *msOWSTerminateOnlineResource(const char *src_url);
 MS_DLL_EXPORT char *msOWSGetOnlineResource(mapObj *map, const char *namespaces, const char *metadata_name, cgiRequestObj *req);
@@ -296,6 +230,10 @@ int msOWSPrintEncodeMetadata(FILE *stream, hashTableObj *metadata,
                              const char *namespaces, const char *name, 
                              int action_if_not_found, 
                              const char *format, const char *default_value) ;
+char *msOWSGetEncodeMetadata(hashTableObj *metadata, 
+                             const char *namespaces, const char *name, 
+                             const char *default_value);
+
 int msOWSPrintValidateMetadata(FILE *stream, hashTableObj *metadata, 
                                const char *namespaces, const char *name, 
                                int action_if_not_found, 
@@ -356,6 +294,15 @@ void msOWSProcessException(layerObj *lp, const char *pszFname,
 char *msOWSBuildURLFilename(const char *pszPath, const char *pszURL, 
                             const char *pszExt);
 const char *msOWSGetEPSGProj(projectionObj *proj, hashTableObj *metadata, const char *namespaces, int bReturnOnlyFirstOne);
+
+void msOWSGetDimensionInfo(layerObj *layer, const char *pszDimension, 
+                           const char **pszDimUserValue, 
+                           const char **pszDimUnits, 
+                           const char **pszDimDefault, 
+                           const char **pszDimNearValue, 
+                           const char **pszDimUnitSymbol, 
+                           const char **pszDimMultiValue);
+
 #endif
 
 /*====================================================================
@@ -369,10 +316,18 @@ const char *msOWSGetEPSGProj(projectionObj *proj, hashTableObj *metadata, const
 #define OWS_GML_OCCUR_UNBOUNDED -1
 
 #if defined(USE_WMS_SVR) || defined (USE_WFS_SVR)
+
+/* TODO, there must be a better way to generalize these lists of objects... */
+
 typedef struct {
   char *name;     /* name of the item */
   char *alias;    /* is the item aliased for presentation? (NULL if not) */
   char *type;     /* raw type for this item (NULL for a "string") (TODO: should this be a lookup table instead?) */
+#ifndef __cplusplus 
+  char *template;  /* presentation string for this item, needs to be a complete XML tag */
+#else
+  char *_template;  /* presentation string for this item, needs to be a complete XML tag */
+#endif
   int encode;     /* should the value be HTML encoded? Default is MS_TRUE */
   int visible;    /* should this item be output, default is MS_FALSE */  
 } gmlItemObj;
@@ -416,15 +371,28 @@ typedef struct {
   int numgroups;
 } gmlGroupListObj;
 
+typedef struct {
+	char *prefix;
+	char *uri;
+	char *schemalocation;
+} gmlNamespaceObj;
+
+typedef struct {
+  gmlNamespaceObj *namespaces;
+  int numnamespaces;
+} gmlNamespaceListObj;
+
 MS_DLL_EXPORT int msItemInGroups(char *name, gmlGroupListObj *groupList);
-MS_DLL_EXPORT gmlItemListObj *msGMLGetItems(layerObj *layer);
+MS_DLL_EXPORT gmlItemListObj *msGMLGetItems(layerObj *layer, const char *metadata_namespaces);
 MS_DLL_EXPORT void msGMLFreeItems(gmlItemListObj *itemList);
-MS_DLL_EXPORT gmlConstantListObj *msGMLGetConstants(layerObj *layer);
+MS_DLL_EXPORT gmlConstantListObj *msGMLGetConstants(layerObj *layer, const char *metadata_namespaces);
 MS_DLL_EXPORT void msGMLFreeConstants(gmlConstantListObj *constantList);
-MS_DLL_EXPORT gmlGeometryListObj *msGMLGetGeometries(layerObj *layer);
+MS_DLL_EXPORT gmlGeometryListObj *msGMLGetGeometries(layerObj *layer, const char *metadata_namespaces);
 MS_DLL_EXPORT void msGMLFreeGeometries(gmlGeometryListObj *geometryList);
-MS_DLL_EXPORT gmlGroupListObj *msGMLGetGroups(layerObj *layer);
+MS_DLL_EXPORT gmlGroupListObj *msGMLGetGroups(layerObj *layer, const char *metadata_namespaces);
 MS_DLL_EXPORT void msGMLFreeGroups(gmlGroupListObj *groupList);
+MS_DLL_EXPORT gmlNamespaceListObj *msGMLGetNamespaces(webObj *web, const char *metadata_namespaces);
+MS_DLL_EXPORT void msGMLFreeNamespaces(gmlNamespaceListObj *namespaceList);
 #endif
 
 /* export to fix bug 851 */
@@ -513,5 +481,14 @@ MS_DLL_EXPORT int msLoadMapContextURL(mapObj *map, char *urlfilename, int unique
 
 int msWCSDispatch(mapObj *map, cgiRequestObj *requestobj); /* only 1 public function */
 
+
+
+/*====================================================================
+ *   mapogsos.c
+ *====================================================================*/
+
+int msSOSDispatch(mapObj *map, cgiRequestObj *requestobj); /* only 1 public function */
+
+
 #endif /* MAPOWS_H */
 
