@@ -31,40 +31,46 @@
    enum yytokentype {
      NUMBER = 258,
      STRING = 259,
-     TIME = 260,
-     REGEX = 261,
-     OR = 262,
-     AND = 263,
-     NOT = 264,
-     IN = 265,
-     GE = 266,
-     LE = 267,
-     GT = 268,
-     LT = 269,
-     NE = 270,
-     EQ = 271,
-     RE = 272,
-     LENGTH = 273,
-     NEG = 274
+     ISTRING = 260,
+     TIME = 261,
+     REGEX = 262,
+     IREGEX = 263,
+     OR = 264,
+     AND = 265,
+     NOT = 266,
+     IEQ = 267,
+     IN = 268,
+     GE = 269,
+     LE = 270,
+     GT = 271,
+     LT = 272,
+     NE = 273,
+     EQ = 274,
+     RE = 275,
+     LENGTH = 276,
+     NEG = 277
    };
 #endif
 #define NUMBER 258
 #define STRING 259
-#define TIME 260
-#define REGEX 261
-#define OR 262
-#define AND 263
-#define NOT 264
-#define IN 265
-#define GE 266
-#define LE 267
-#define GT 268
-#define LT 269
-#define NE 270
-#define EQ 271
-#define RE 272
-#define LENGTH 273
-#define NEG 274
+#define ISTRING 260
+#define TIME 261
+#define REGEX 262
+#define IREGEX 263
+#define OR 264
+#define AND 265
+#define NOT 266
+#define IEQ 267
+#define IN 268
+#define GE 269
+#define LE 270
+#define GT 271
+#define LT 272
+#define NE 273
+#define EQ 274
+#define RE 275
+#define LENGTH 276
+#define NEG 277
 
 
 
@@ -78,7 +84,7 @@ typedef union YYSTYPE {
   struct tm tmval;
 } YYSTYPE;
 /* Line 1248 of yacc.c.  */
-#line 81 "mapparser.h"
+#line 87 "mapparser.h"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
