@@ -28,7 +28,10 @@
  ******************************************************************************
  *
  * $Log: mappdf.c,v $
- * Revision 1.37.2.1  2006/01/16 20:41:22  sdlime
+ * Revision 1.39  2006/01/16 20:37:15  sdlime
+ * Changed label size calls to not adjust baseline offset.
+ *
+ * Revision 1.38  2006/01/16 20:21:18  sdlime
  * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
  *
  * Revision 1.37  2005/12/14 01:21:38  assefa
@@ -71,7 +74,7 @@
 #endif
 #include "map.h"
 
-MS_CVSID("$Id: mappdf.c,v 1.37.2.1 2006/01/16 20:41:22 sdlime Exp $")
+MS_CVSID("$Id: mappdf.c,v 1.39 2006/01/16 20:37:15 sdlime Exp $")
 
 /* -------------------------------------------------------------------- */
 /*      prototypes.                                                     */
