@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mappool.c,v 1.12 2005/07/16 21:18:34 jerryp Exp $
+ * $Id: mappool.c,v 1.14 2006/06/05 20:07:25 hobu Exp $
  *
  * Project:  MapServer
  * Purpose:  Implement new style connection pooling.
@@ -28,6 +28,13 @@
  ******************************************************************************
  *
  * $Log: mappool.c,v $
+ * Revision 1.14  2006/06/05 20:07:25  hobu
+ * only print debug output for unreferenced closes if debug is
+ * set for the connection
+ *
+ * Revision 1.13  2006/06/05 16:39:28  hobu
+ * don't output connection details into debug output
+ *
  * Revision 1.12  2005/07/16 21:18:34  jerryp
  * msConnPoolClose now only copies connections when needed.
  *
@@ -171,7 +178,7 @@ o Currently the connection pooling API makes not attempt to address
 #include "map.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mappool.c,v 1.12 2005/07/16 21:18:34 jerryp Exp $")
+MS_CVSID("$Id: mappool.c,v 1.14 2006/06/05 20:07:25 hobu Exp $")
 
 /* defines for lifetime.  
    A positive number is a time-from-last use in seconds */
@@ -320,9 +327,11 @@ static void msConnPoolClose( int conn_index )
 
     if( conn->ref_count > 0 )
     {
-        msDebug( "msConnPoolClose(): "
+        if( conn->debug )
+            msDebug( "msConnPoolClose(): "
                  "Closing connection %s even though ref_count=%d.\n", 
                  conn->connection, conn->ref_count );
+
         msSetError( MS_MISCERR, 
                     "Closing connection %s even though ref_count=%d.", 
                     "msConnPoolClose()",
