@@ -27,16 +27,36 @@
  ******************************************************************************
  *
  * $Log: mappostgis.c,v $
- * Revision 1.65.2.3  2006/03/30 05:20:46  sdlime
- * Applied patch for bug 1629.
+ * Revision 1.74  2006/08/13 14:26:52  sdlime
+ * Fixed typo in connection encryption addition.
  *
- * Revision 1.65.2.2  2006/02/25 19:10:41  pramsey
- * Added checks for MS_FAILURE to the ParseData function calls.
+ * Revision 1.73  2006/08/11 16:58:01  dan
+ * Added ability to encrypt tokens (passwords, etc.) in database connection
+ * strings (MS-RFC-18, bug 1792)
+ *
+ * Revision 1.72  2006/05/09 22:35:41  pramsey
+ * Added quotes around field names being retrieved by query requests. (Bug 1536)
+ *
+ * Revision 1.71  2006/05/03 22:35:41  pramsey
+ * Added schema separation and search path awareness to the LayerRetrievePK. (towards Bug 1571)
+ *
+ * Revision 1.70  2006/05/02 16:03:36  frank
+ * keep track of mycursor and close in layerclose (bug 1757)
+ *
+ * Revision 1.69  2006/04/28 16:07:53  pramsey
+ * Removed HTML tags from PostGIS error messages (Bug 1572)
+ *
+ * Revision 1.68  2006/02/25 19:11:41  pramsey
+ * Added checks for MS_FAILURE around ParseData calls.
  * Patch from Tamas Szekeres
  *
- * Revision 1.65.2.1  2006/01/19 15:58:16  sean
+ * Revision 1.67  2006/01/31 10:26:24  umberto
+ * Revert behaviour to pre-1.61: do not allow for use of the FILTERITEM attribute (bug 1629)
+ *
+ * Revision 1.66  2006/01/19 16:04:04  sean
  * move gBYTE_ORDER inside the pg layerinfo structure to allow for differently byte ordered connections (bug 1587).
  *
+ *
  * Revision 1.65  2005/12/14 21:25:47  sean
  * test for the case of a selection w/out srid for postgis data and fix from Daryl (bug 1443)
  *
@@ -113,7 +133,7 @@
  *
  */
 
-/* $Id: mappostgis.c,v 1.65.2.3 2006/03/30 05:20:46 sdlime Exp $ */
+/* $Id: mappostgis.c,v 1.74 2006/08/13 14:26:52 sdlime Exp $ */
 #include <assert.h>
 #include "map.h"
 #include "maptime.h"
@@ -135,7 +155,7 @@
 #include <string.h>
 #include <ctype.h> /* tolower() */
 
-MS_CVSID("$Id: mappostgis.c,v 1.65.2.3 2006/03/30 05:20:46 sdlime Exp $")
+MS_CVSID("$Id: mappostgis.c,v 1.74 2006/08/13 14:26:52 sdlime Exp $")
 
 typedef struct ms_POSTGIS_layer_info_t
 {
@@ -147,14 +167,16 @@ typedef struct ms_POSTGIS_layer_info_t
     char        *user_srid;     /* zero length = calculate, non-zero means using this value! */
     int gBYTE_ORDER;            /* no longer a global variable */
 
+    char        cursor_name[128]; /* active cursor (empty string if none) */
+
 } msPOSTGISLayerInfo;
 
 #define DATA_ERROR_MESSAGE \
     "%s" \
-    "Error with POSTGIS data variable. You specified '%s'.<br>\n" \
-    "Standard ways of specifiying are : <br>\n(1) 'geometry_column from geometry_table' <br>\n(2) 'geometry_column from (&lt;sub query&gt;) as foo using unique &lt;column name&gt; using SRID=&lt;srid#&gt;' <br><br>\n\n" \
-    "Make sure you put in the 'using unique  &lt;column name&gt;' and 'using SRID=#' clauses in.\n\n<br><br>" \
-    "For more help, please see http://postgis.refractions.net/documentation/ \n\n<br><br>" \
+    "Error with POSTGIS data variable. You specified '%s'.\n" \
+    "Standard ways of specifiying are : \n(1) 'geometry_column from geometry_table' \n(2) 'geometry_column from (sub query) as foo using unique column name using SRID=srid#' \n\n" \
+    "Make sure you put in the 'using unique  column name' and 'using SRID=#' clauses in.\n\n" \
+    "For more help, please see http://postgis.refractions.net/documentation/ \n\n" \
     "Mappostgis.c - version of Jan 23/2004.\n"
 
 
@@ -273,7 +295,7 @@ int msPOSTGISLayerOpen(layerObj *layer)
     }
 
     if(!layer->data) {
-        msSetError(MS_QUERYERR, DATA_ERROR_MESSAGE, "msPOSTGISLayerOpen()", "", "Error parsing POSTGIS data variable: nothing specified in DATA statement.<br><br>\n\nMore Help:<br><br>\n\n");
+        msSetError(MS_QUERYERR, DATA_ERROR_MESSAGE, "msPOSTGISLayerOpen()", "", "Error parsing POSTGIS data variable: nothing specified in DATA statement.\n\nMore Help:\n\n");
 
         return MS_FAILURE;
     }
@@ -288,15 +310,26 @@ int msPOSTGISLayerOpen(layerObj *layer)
     layerinfo->user_srid = NULL;
     layerinfo->conn = NULL;
     layerinfo->gBYTE_ORDER = 0;
+    layerinfo->cursor_name[0] = '\0';
 
     layerinfo->conn = (PGconn *) msConnPoolRequest(layer);
     if(!layerinfo->conn) {
+        char *conn_decrypted;
         if(layer->debug) {
             msDebug("MSPOSTGISLayerOpen -- shared connection not available.\n");
         }
-                
+
+        /* Decrypt any encrypted token in connection and attempt to connect */
+        conn_decrypted = msDecryptStringTokens(layer->map, layer->connection);
+        if (conn_decrypted == NULL) {
+          return(MS_FAILURE);  /* An error should already have been produced */
+        }
+   
         layerinfo->conn = PQconnectdb(layer->connection);
 
+        msFree(conn_decrypted);
+        conn_decrypted = NULL;
+
         if(!layerinfo->conn || PQstatus(layerinfo->conn) == CONNECTION_BAD) {
             msDebug("FAILURE!!!");
             maskeddata = (char *)malloc(strlen(layer->connection) + 1);
@@ -311,7 +344,7 @@ int msPOSTGISLayerOpen(layerObj *layer)
               }
             }
             
-            msSetError(MS_QUERYERR, "couldnt make connection to DB with connect string '%s'.\n<br>\nError reported was '%s'.\n<br>\n\nThis error occured when trying to make a connection to the specified postgresql server.  \n<br>\nMost commonly this is caused by <br>\n(1) incorrect connection string <br>\n(2) you didnt specify a 'user=...' in your connection string <br>\n(3) the postmaster (postgresql server) isnt running <br>\n(4) you are not allowing TCP/IP connection to the postmaster <br>\n(5) your postmaster is not running on the correct port - if its not on 5432 you must specify a 'port=...' <br>\n (6) the security on your system does not allow the webserver (usually user 'nobody') to make socket connections to the postmaster <br>\n(7) you forgot to specify a 'host=...' if the postmaster is on a different machine<br>\n(8) you made a typo <br>\n  ", "msPOSTGISLayerOpen()", maskeddata,PQerrorMessage(layerinfo->conn));
+            msSetError(MS_QUERYERR, "couldnt make connection to DB with connect string '%s'.\n\nError reported was '%s'.\n\n\nThis error occured when trying to make a connection to the specified postgresql server.  \n\nMost commonly this is caused by \n(1) incorrect connection string \n(2) you didnt specify a 'user=...' in your connection string \n(3) the postmaster (postgresql server) isnt running \n(4) you are not allowing TCP/IP connection to the postmaster \n(5) your postmaster is not running on the correct port - if its not on 5432 you must specify a 'port=...' \n (6) the security on your system does not allow the webserver (usually user 'nobody') to make socket connections to the postmaster \n(7) you forgot to specify a 'host=...' if the postmaster is on a different machine\n(8) you made a typo \n  ", "msPOSTGISLayerOpen()", maskeddata,PQerrorMessage(layerinfo->conn));
 
             free(maskeddata);
             free(layerinfo);
@@ -452,7 +485,7 @@ static int prepare_database(const char *geom_table, const char *geom_column, lay
 #endif
 
         if(!pos_space || !pos_paren) {
-            msSetError(MS_QUERYERR, DATA_ERROR_MESSAGE, "prepare_database()", geom_table, "Error parsing POSTGIS data variable: Something is wrong with your subselect statement.<br><br>\n\nMore Help:<br><br>\n\n");
+            msSetError(MS_QUERYERR, DATA_ERROR_MESSAGE, "prepare_database()", geom_table, "Error parsing POSTGIS data variable: Something is wrong with your subselect statement.\n\nMore Help:\n\n");
 
             return MS_FAILURE;
         }
@@ -478,12 +511,14 @@ static int prepare_database(const char *geom_table, const char *geom_column, lay
     } else {
         length = 55 + strlen(geom_column) + strlen(urid_name);
         for(t = 0; t < layer->numitems; t++) {
-            length += strlen(layer->items[t]) + 7;
+            length += strlen(layer->items[t]) + 9;
         }
         columns_wanted = (char *) malloc(length + 1);
         *columns_wanted = 0;
         for(t = 0; t < layer->numitems; t++) {
+            strcat(columns_wanted, "\"");
             strcat(columns_wanted, layer->items[t]);
+            strcat(columns_wanted, "\"");
             strcat(columns_wanted, "::text,");
         }
         temp = columns_wanted + strlen(columns_wanted);
@@ -529,6 +564,10 @@ static int prepare_database(const char *geom_table, const char *geom_column, lay
 
     /* Allocate buffer to fit the largest query string */
     query_string_0_6 = (char *) malloc(113 + 42 + strlen(columns_wanted) + strlen(data_source) + (layer->filter.string ? strlen(layer->filter.string) : 0) + 2 * strlen(geom_column) + strlen(box3d) + strlen(f_table_name) + strlen(user_srid) + 1);
+    
+    assert( layerinfo->cursor_name[0] == '\0' );
+    strcpy( layerinfo->cursor_name, "mycursor" );
+
     if(!layer->filter.string) {
         if(!strlen(user_srid)) {
             sprintf(query_string_0_6, "DECLARE mycursor BINARY CURSOR FOR SELECT %s from %s WHERE %s && setSRID(%s, find_srid('','%s','%s') )", columns_wanted, data_source, geom_column, box3d, f_table_name, geom_column);
@@ -628,8 +667,8 @@ static int prepare_database(const char *geom_table, const char *geom_column, lay
 
     /* TODO Rename tmp2 to something meaningful */
     tmp2 = (char *) malloc(149 + strlen(query_string_0_6) + strlen(error_message) + 1);
-    sprintf(tmp2, "Error executing POSTGIS DECLARE (the actual query) statement: '%s' <br><br>\n\nPostgresql reports the error as '%s'<br><br>\n\nMore Help:<br><br>\n\n", query_string_0_6, error_message);
-    msSetError(MS_QUERYERR, DATA_ERROR_MESSAGE, "prepare_database()", tmp2, "&lt;check your .map file&gt;");
+    sprintf(tmp2, "Error executing POSTGIS DECLARE (the actual query) statement: '%s' \n\nPostgresql reports the error as '%s'\n\nMore Help:\n\n", query_string_0_6, error_message);
+    msSetError(MS_QUERYERR, DATA_ERROR_MESSAGE, "prepare_database()", tmp2, "check your .map file");
 
     free(tmp2);
     free(error_message);
@@ -741,6 +780,21 @@ int msPOSTGISLayerClose(layerObj *layer)
             msDebug("msPOSTGISLayerClose -- query_result is NULL\n");
         }
 
+        if( strlen(layerinfo->cursor_name) > 0 )
+        {
+            PGresult            *query_result;
+            char                cmd_buffer[500];
+
+            sprintf( cmd_buffer, "CLOSE %s", layerinfo->cursor_name );
+
+            query_result = PQexec(layerinfo->conn, cmd_buffer );
+            if(query_result) {
+                PQclear(query_result);
+            }
+
+            layerinfo->cursor_name[0] = '\0';
+        }
+
         msConnPoolRelease(layer, layerinfo->conn);
         layerinfo->conn = NULL;
 
@@ -1215,12 +1269,14 @@ int msPOSTGISLayerGetShape(layerObj *layer, shapeObj *shape, long record)
     } else {
         length = 46 + strlen(geom_column_name);
         for(t = 0; t < layer->numitems; t++) {
-            length += strlen(layer->items[t]) + 7;
+            length += strlen(layer->items[t]) + 9;
         }
         columns_wanted = (char *) malloc(length + 1);
         *columns_wanted = 0;
         for(t = 0; t < layer->numitems; t++) {
+            strcat(columns_wanted, "\"");
             strcat(columns_wanted, layer->items[t]);
+            strcat(columns_wanted, "\"");
             strcat(columns_wanted, "::text,");
         }
         temp = columns_wanted + strlen(columns_wanted);
@@ -1264,7 +1320,7 @@ int msPOSTGISLayerGetShape(layerObj *layer, shapeObj *shape, long record)
     query_result = PQexec(layerinfo->conn, query_str);
 
     if(!query_result || PQresultStatus(query_result) != PGRES_COMMAND_OK) {
-        msSetError(MS_QUERYERR, "Error executing POSTGIS SQL statement (in FETCH ALL): %s\n-%s\n<br>More Help:<br>", "msPOSTGISLayerGetShape()", query_str, PQerrorMessage(layerinfo->conn));
+        msSetError(MS_QUERYERR, "Error executing POSTGIS SQL statement (in FETCH ALL): %s\n-%s\nMore Help:", "msPOSTGISLayerGetShape()", query_str, PQerrorMessage(layerinfo->conn));
 
         if(query_result) {
           PQclear(query_result);
@@ -1697,6 +1753,30 @@ int msPOSTGISLayerRetrievePK(layerObj *layer, char **urid_name, char* table_name
     char        *sql = 0;
     msPOSTGISLayerInfo *layerinfo = 0;
     int major, minor, point, tmpint;
+    int length;
+    char *pos_sep, *schema, *table;
+    schema = NULL;
+    table = NULL;
+
+     /* Attempt to separate table_name into schema.table */
+    pos_sep = strstr(table_name, ".");
+    if(pos_sep) 
+    {
+      length = pos_sep - table_name;
+      schema = (char *)malloc(length + 1);
+      strncpy(schema, table_name, length);
+      schema[length] = 0;
+
+      length = (int)pos_sep + strlen(pos_sep);
+      table = (char *)malloc(length);
+      strncpy(table, pos_sep + 1, length - 1);
+      table[length - 1] = 0;
+      
+      if(debug)
+      {
+        msDebug("msPOSTGISLayerRetrievePK(): Found schema %s, table %s.\n", schema, table);
+      }
+    }
     
     if(msPOSTGISLayerRetrievePGVersion(layer, debug, &major, &minor, &point) == MS_FAILURE) 
     {
@@ -1718,7 +1798,7 @@ int msPOSTGISLayerRetrievePK(layerObj *layer, char **urid_name, char* table_name
     {
         if(debug)
         {
-            msDebug("msPOSTGISLayerRetrievePK(): Version below 7.3.\n");
+            msDebug("msPOSTGISLayerRetrievePK(): Version below 7.2.\n");
         }
         return MS_FAILURE;
     }
@@ -1736,8 +1816,18 @@ int msPOSTGISLayerRetrievePK(layerObj *layer, char **urid_name, char* table_name
          * We only support single column primary keys, so multicolumn
          * pks are explicitly excluded from the query.
          */
-        sql = malloc(strlen(table_name) + 288); 
-        sprintf(sql, "select attname from pg_attribute, pg_constraint, pg_class where pg_constraint.conrelid = pg_class.oid and pg_class.oid = pg_attribute.attrelid and pg_constraint.contype = 'p' and pg_constraint.conkey[1] = pg_attribute.attnum and pg_class.relname = '%s' and pg_constraint.conkey[2] is null", table_name);
+        if(schema && table)
+        {
+            sql = malloc(strlen(schema) + strlen(table) + 376); 
+            sprintf(sql, "select attname from pg_attribute, pg_constraint, pg_class, pg_namespace where pg_constraint.conrelid = pg_class.oid and pg_class.oid = pg_attribute.attrelid and pg_constraint.contype = 'p' and pg_constraint.conkey[1] = pg_attribute.attnum and pg_class.relname = '%s' and pg_class.relnamespace = pg_namespace.oid and pg_namespace.nspname = '%s' and pg_constraint.conkey[2] is null", table, schema);
+            free(table);
+            free(schema);
+        }
+        else
+        {
+            sql = malloc(strlen(table_name) + 325); 
+            sprintf(sql, "select attname from pg_attribute, pg_constraint, pg_class where pg_constraint.conrelid = pg_class.oid and pg_class.oid = pg_attribute.attrelid and pg_constraint.contype = 'p' and pg_constraint.conkey[1] = pg_attribute.attnum and pg_class.relname = '%s' and pg_table_is_visible(pg_class.oid) and pg_constraint.conkey[2] is null", table_name);
+      }
     }
 
     if (debug)
@@ -1782,6 +1872,16 @@ int msPOSTGISLayerRetrievePK(layerObj *layer, char **urid_name, char* table_name
       free(sql);
       return MS_FAILURE;
     }
+    if(PQntuples(query_result) > 1)
+    {
+      if(debug)
+      {
+        msDebug("msPOSTGISLayerRetrievePK: Multiple results found.\n");
+      }
+      PQclear(query_result);
+      free(sql);
+      return MS_FAILURE;
+    }
 
     if(PQgetisnull(query_result, 0, 0)) 
     {
@@ -1842,7 +1942,7 @@ static int msPOSTGISLayerParseData(layerObj *layer, char **geom_column_name, cha
     } else {
         slength = strspn(pos_srid + 12, "-0123456789");
         if(!slength) {
-            msSetError(MS_QUERYERR, DATA_ERROR_MESSAGE, "msPOSTGISLayerParseData()", "Error parsing POSTGIS data variable: You specified 'using SRID=#' but didnt have any numbers!<br><br>\n\nMore Help:<br><br>\n\n", data);
+            msSetError(MS_QUERYERR, DATA_ERROR_MESSAGE, "msPOSTGISLayerParseData()", "Error parsing POSTGIS data variable: You specified 'using SRID=#' but didnt have any numbers!\n\nMore Help:\n\n", data);
 
             return MS_FAILURE;
         } else {
@@ -1867,7 +1967,7 @@ static int msPOSTGISLayerParseData(layerObj *layer, char **geom_column_name, cha
     /* Scan for the table or sub-select clause */
     pos_scn = strstrIgnoreCase(data, " from ");
     if(!pos_scn) {
-        msSetError(MS_QUERYERR, DATA_ERROR_MESSAGE, "msPOSTGISLayerParseData()", "Error parsing POSTGIS data variable.  Must contain 'geometry_column from table_name' or 'geom from (subselect) as foo' (couldnt find ' from ').  More help: <br><br>\n\n", data);
+        msSetError(MS_QUERYERR, DATA_ERROR_MESSAGE, "msPOSTGISLayerParseData()", "Error parsing POSTGIS data variable.  Must contain 'geometry_column from table_name' or 'geom from (subselect) as foo' (couldnt find ' from ').  More help: \n\n", data);
 
         return MS_FAILURE;
     }
@@ -1886,7 +1986,7 @@ static int msPOSTGISLayerParseData(layerObj *layer, char **geom_column_name, cha
     (*table_name)[pos_opt - (pos_scn + 6)] = 0;
 
     if(strlen(*table_name) < 1 || strlen(*geom_column_name) < 1) {
-        msSetError(MS_QUERYERR, DATA_ERROR_MESSAGE, "msPOSTGISLayerParseData()", "Error parsing POSTGIS data variable.  Must contain 'geometry_column from table_name' or 'geom from (subselect) as foo' (couldnt find a geometry_column or table/subselect).  More help: <br><br>\n\n", data);
+        msSetError(MS_QUERYERR, DATA_ERROR_MESSAGE, "msPOSTGISLayerParseData()", "Error parsing POSTGIS data variable.  Must contain 'geometry_column from table_name' or 'geom from (subselect) as foo' (couldnt find a geometry_column or table/subselect).  More help: \n\n", data);
 
         return MS_FAILURE;
     }
