@@ -27,7 +27,53 @@
  ******************************************************************************
  *
  * $Log: mapprimitive.c,v $
- * Revision 1.60.2.1  2006/02/17 03:06:13  sdlime
+ * Revision 1.76  2006/09/01 18:03:19  umberto
+ * Removed USE_GEOS defines, bug #1890
+ *
+ * Revision 1.75  2006/08/17 04:32:16  sdlime
+ * Disable path following labels unless GD 2.0.29 or greater is available.
+ *
+ * Revision 1.74  2006/06/27 13:58:19  frank
+ * Place shapeObj->geometry assignment into #ifdef USE_GEOS.
+ *
+ * Revision 1.73  2006/06/27 06:32:09  sdlime
+ * Fixed msCopyShape to initialize the geometry property of the destination shape to NULL.
+ *
+ * Revision 1.72  2006/05/18 22:20:37  sdlime
+ * Fixed offseting code percentage of 1.0 maps to width-1 or height-1.
+ *
+ * Revision 1.71  2006/04/28 03:13:02  sdlime
+ * Fixed a few issues with relative coordinates. Added support for all nine relative positions. (bug 1547)
+ *
+ * Revision 1.70  2006/04/27 04:05:17  sdlime
+ * Initial support for relative coordinates. (bug 1547)
+ *
+ * Revision 1.69  2006/04/26 13:42:53  frank
+ * temporarily block out MS_PERCENTAGES use till it is defined.
+ *
+ * Revision 1.68  2006/04/26 03:25:47  sdlime
+ * Applied most recent patch for curved labels. (bug 1620)
+ *
+ * Revision 1.67  2006/03/22 23:31:20  sdlime
+ * Applied latest patch for curved labels. (bug 1620)
+ *
+ * Revision 1.66  2006/03/02 06:43:51  sdlime
+ * Applied latest patch for curved labels. (bug 1620)
+ *
+ * Revision 1.65  2006/02/24 05:53:49  sdlime
+ * Applied another round of patches for bug 1620.
+ *
+ * Revision 1.64  2006/02/18 21:14:09  hobu
+ * INFINITY is already defined on in the math headers on osx.
+ * Don't redefine it if it is already there.
+ *
+ * Revision 1.63  2006/02/18 20:59:13  sdlime
+ * Initial code for curved labels. (bug 1620)
+ *
+ * Revision 1.62  2006/02/17 03:05:23  sdlime
+ * Slightly more efficient version (no modulus operator) of the routine to test if a ring is an outer ring. (bug 1648)
+ *
+ * Revision 1.61  2006/02/16 07:51:31  sdlime
  * Fixed a flaw in routine that computes outer ring list. In certain cases it could miss an outer ring with holes in certain places. (bug 1648)
  *
  * Revision 1.60  2005/11/01 05:35:50  frank
@@ -96,7 +142,7 @@
 #include "mapprimitive.h"
 #include <assert.h>
 
-MS_CVSID("$Id: mapprimitive.c,v 1.60.2.1 2006/02/17 03:06:13 sdlime Exp $")
+MS_CVSID("$Id: mapprimitive.c,v 1.76 2006/09/01 18:03:19 umberto Exp $")
 
 typedef enum {CLIP_LEFT, CLIP_MIDDLE, CLIP_RIGHT} CLIP_STATE;
 
@@ -105,7 +151,9 @@ typedef enum {CLIP_LEFT, CLIP_MIDDLE, CLIP_RIGHT} CLIP_STATE;
 #define SWAP( a, b, t) ( (t) = (a), (a) = (b), (b) = (t) )
 #define EDGE_CHECK( x0, x, x1) ((x) < MS_MIN( (x0), (x1)) ? CLIP_LEFT : ((x) > MS_MAX( (x0), (x1)) ? CLIP_RIGHT : CLIP_MIDDLE ))
 
+#ifndef INFINITY
 #define INFINITY	(1.0e+30)
+#endif
 #define NEARZERO	(1.0e-30)	/* 1/INFINITY */
 
 void msPrintShape(shapeObj *p) 
@@ -158,9 +206,7 @@ void msInitShape(shapeObj *shape)
   shape->values = NULL;
   shape->numvalues = 0;
 
-#ifdef USE_GEOS
   shape->geometry = NULL;
-#endif
 
   /* annotation component */
   shape->text = NULL;
@@ -198,7 +244,7 @@ int msCopyShape(shapeObj *from, shapeObj *to) {
     to->numvalues = from->numvalues;
   }
 
-  /* TODO add GEOS copy, or at least initialize to->geometry to NULL */
+  to->geometry = NULL; /* GEOS code will build automatically if necessary */
 
   return(0);
 }
@@ -223,6 +269,14 @@ void msFreeShape(shapeObj *shape)
   msInitShape(shape); /* now reset */
 }
 
+void msFreeLabelPathObj(labelPathObj *path)
+{
+  msFreeShape(&(path->bounds));
+  msFree(path->path.point);
+  msFree(path->angles);
+  msFree(path);
+}
+
 void msShapeDeleteLine( shapeObj *shape, int line )
 
 {
@@ -263,9 +317,9 @@ void msComputeBounds(shapeObj *shape)
 }
 
 /* checks to see if ring r is an outer ring of shape */
-static int isOuterRing(shapeObj *shape, int r)
+static int isOuterRing(shapeObj *shape, int r) 
 {
-  int i, status=MS_TRUE;
+  int i, status=MS_TRUE; 
 
   if(shape->numlines == 1) return(MS_TRUE);
 
@@ -732,6 +786,129 @@ void msClipPolygonRect(shapeObj *shape, rectObj rect)
   return;
 }
 
+/*
+** offsets a point relative to an image position
+*/
+void msOffsetPointRelativeTo(pointObj *point, layerObj *layer)
+{
+  double x=0, y=0;
+
+  if(layer->transform == MS_TRUE) return; /* nothing to do */
+
+  if(layer->units == MS_PERCENTAGES) {
+    point->x *= (layer->map->width-1);
+    point->y *= (layer->map->height-1);
+  }
+
+  if(layer->transform == MS_FALSE || layer->transform == MS_UL) return; /* done */
+
+  switch(layer->transform) {
+  case MS_UC:
+    x = (layer->map->width-1)/2;
+    y = 0;
+    break;
+  case MS_UR:
+    x = layer->map->width-1;
+    y = 0;
+    break;
+  case MS_CL:
+    x = 0;
+    y = layer->map->height/2;
+    break;
+  case MS_CC:
+    x = layer->map->width/2;
+    y = layer->map->height/2;
+    break;
+  case MS_CR:
+    x = layer->map->width-1;
+    y = layer->map->height/2;
+    break;
+  case MS_LL:
+    x = 0;
+    y = layer->map->height-1;
+    break;
+  case MS_LC:
+    x = layer->map->width/2;
+    y = layer->map->height-1;
+    break;
+  case MS_LR:
+    x = layer->map->width-1;
+    y = layer->map->height-1;
+    break;
+  }
+
+  point->x += x;
+  point->y += y;
+
+  return;
+}
+
+/*
+** offsets a shape relative to an image position
+*/
+void msOffsetShapeRelativeTo(shapeObj *shape, layerObj *layer) 
+{
+  int i, j;
+  double x=0, y=0;
+
+  if(layer->transform == MS_TRUE) return; /* nothing to do */
+
+  if(layer->units == MS_PERCENTAGES) {
+    for (i=0; i<shape->numlines; i++) {
+      for (j=0; j<shape->line[i].numpoints; j++) {
+        shape->line[i].point[j].x *= (layer->map->width-1);
+        shape->line[i].point[j].y *= (layer->map->height-1);
+      }
+    }
+  }
+
+  if(layer->transform == MS_FALSE || layer->transform == MS_UL) return; /* done */
+
+  switch(layer->transform) {
+  case MS_UC:
+    x = (layer->map->width-1)/2;
+    y = 0;
+    break;
+  case MS_UR:
+    x = layer->map->width-1;
+    y = 0;
+    break;
+  case MS_CL:
+    x = 0;
+    y = layer->map->height/2;
+    break;
+  case MS_CC:
+    x = layer->map->width/2;
+    y = layer->map->height/2;
+    break;
+  case MS_CR:
+    x = layer->map->width-1;
+    y = layer->map->height/2;
+    break;
+  case MS_LL:
+    x = 0;
+    y = layer->map->height-1;
+    break;
+  case MS_LC:
+    x = layer->map->width/2;
+    y = layer->map->height-1;
+    break;
+  case MS_LR:
+    x = layer->map->width-1;
+    y = layer->map->height-1;
+    break;
+  }
+
+  for (i=0; i<shape->numlines; i++) {
+    for (j=0; j<shape->line[i].numpoints; j++) {
+      shape->line[i].point[j].x += x;
+      shape->line[i].point[j].y += y;
+    }
+  }
+
+  return;
+}
+
 /*
 ** converts from map coordinates to image coordinates
 */
@@ -1147,6 +1324,408 @@ int msPolylineLabelPoint(shapeObj *p, pointObj *lp, int min_length, double *angl
   return(MS_SUCCESS);
 }
 
+/*
+ * Calculate a series of label points for each character in the label for a
+ * given polyline.  The resultant series of points is stored in *labelpath.
+ * Note that the points and bounds are allocated in this function.  The
+ * polyline must be converted to image coordinates before calling this
+ * function.
+ */
+labelPathObj* msPolylineLabelPath(shapeObj *p, int min_length, fontSetObj *fontset, char *string, labelObj *label, double scalefactor, int *status)
+{
+  double line_length, max_line_length, segment_length, total_length, distance_along_segment;
+  double fwd_line_length, rev_line_length, text_length, text_start_length;
+  int segment_index, line_index;
+  
+  int i,j,k, inc, final_j;
+  double direction;
+  rectObj bbox;
+  lineObj bounds;
+  double *offsets;
+  double size, t;
+  double cx, cy; /* centre of a character, x & y values. */
+  
+  double theta;
+  double dx, dy, w, cos_t, sin_t;
+
+  double **segment_lengths = NULL;
+  labelPathObj *labelpath = NULL;
+  char *font = NULL;
+
+  /* Line smoothing kernel */
+  double kernel[] = {0.1,0.2,2,0.2,0.1}; /*{1.5, 2, 15, 2, 1.5};*/
+  double kernel_normal = 2.6; /* Must be sum of kernel elements */
+
+  double letterspacing = 1.25;
+
+  offsets = NULL;
+  
+  /* Assume success */
+  *status = MS_SUCCESS;
+
+#ifndef GD_HAS_FTEX_XSHOW
+  goto FAILURE; /* we don't have a current enough version of GD, fall back to ANGLE AUTO */
+#else
+  /* Skip the label and use the normal algorithm if it has fewer than 2 characters */
+  if ( strlen(string) < 2 ) {
+    goto FAILURE;
+  }
+  
+  segment_index = line_index = 0;
+  total_length = max_line_length = 0.0;
+  
+  /* Determine longest line */
+  segment_lengths = (double**)malloc(sizeof(double*) * p->numlines);
+  for ( i = 0; i < p->numlines; i++ ) {
+    
+    segment_lengths[i] = (double*)malloc(sizeof(double) * p->line[i].numpoints);    
+    line_length = 0;
+    
+    for ( j = 1; j < p->line[i].numpoints; j++ ) {
+      segment_length = sqrt( pow((p->line[i].point[j].x - p->line[i].point[j-1].x), 2.0) +
+                             pow((p->line[i].point[j].y - p->line[i].point[j-1].y), 2.0) );
+      line_length += segment_length;
+      segment_lengths[i][j-1] = segment_length;
+      
+    }
+
+    total_length += line_length;
+    
+    if ( line_length > max_line_length ) {
+      line_index = i;
+      max_line_length = line_length;
+    }
+  }
+
+  i = line_index;
+  
+  if ( ((min_length != -1) && (total_length < min_length)) ) {
+    /* Too short */ 
+    *status = MS_FAILURE;
+    goto FAILURE;
+  }
+
+  if ( p->line[i].numpoints < 2 ) {
+    /* Degenerate */
+    *status = MS_FAILURE;
+    goto FAILURE;
+  }
+
+  if ( p->line[i].numpoints == 2 ) {
+    /* We can just use the regular algorithm to save some cycles */
+    goto FAILURE;
+  }
+  
+  /* Determine the total length of the text */
+  if ( msGetLabelSizeEx(string, label, &bbox, fontset, scalefactor, MS_FALSE, &offsets) == MS_FAILURE ) {
+    *status = MS_FAILURE;
+    goto FAILURE;
+  }
+
+  size = label->size*scalefactor;
+  size = MS_MAX(size, label->minsize);
+  size = MS_MIN(size, label->maxsize);
+
+  font = msLookupHashTable(&(fontset->fonts), label->font);
+  if(!font) {
+    if(label->font) 
+      msSetError(MS_TTFERR, "Requested font (%s) not found.", "msPolylineLabelPath()", label->font);
+    else
+      msSetError(MS_TTFERR, "Requested font (NULL) not found.", "msPolylineLabelPath()");
+    *status = MS_FAILURE;
+    goto FAILURE;
+  }
+  
+  text_length = letterspacing * (bbox.maxx - bbox.minx);
+
+  /* If the text length is way longer than the line, skip adding the
+     label if it isn't forced (long extrapolated labels tend to be
+     ugly) */
+  if ( text_length > 1.5 * max_line_length && label->force == MS_FALSE ) {
+    *status = MS_FAILURE;
+    goto FAILURE;
+  }
+  
+  /* Allocate the labelpath */
+  labelpath = (labelPathObj*)malloc(sizeof(labelPathObj));
+  labelpath->path.numpoints = strlen(string);
+  labelpath->path.point = (pointObj*)calloc(labelpath->path.numpoints,sizeof(pointObj));
+  labelpath->angles = (double*)malloc(sizeof(double) * (labelpath->path.numpoints));
+  msInitShape(&(labelpath->bounds));
+
+  /* The bounds will have two points for each character plus an endpoint:
+     the UL corners of each bbox will be tied together and the LL corners
+     will be tied together. */
+  bounds.numpoints = 2*strlen(string) + 1;
+  bounds.point = (pointObj*)malloc(sizeof(pointObj) * bounds.numpoints);
+                                   
+  /* The points start at (max_line_length - text_length) / 2 in order to be centred */
+  text_start_length = (max_line_length - text_length) / 2.0;
+  
+  /* The text is longer than the line: extrapolate the first and last segments */
+  if ( text_start_length < 0.0 ) {
+    
+    j = 0;
+    final_j = p->line[i].numpoints - 1;
+    fwd_line_length = rev_line_length = 0;
+    
+  } else {
+
+    /* Proceed until we've traversed text_start_length in distance */
+    fwd_line_length = 0;
+    j = 0;
+    while ( fwd_line_length < text_start_length )
+      fwd_line_length += segment_lengths[i][j++];
+
+    j--;
+
+    /* Determine the last segment */ 
+    rev_line_length = 0;
+    final_j = p->line[i].numpoints - 1;
+    while ( rev_line_length < text_start_length ) {
+      rev_line_length += segment_lengths[i][final_j - 1];
+      final_j--;
+    }
+    final_j++;
+
+  }
+
+  if ( final_j == 0 )
+    final_j = 1;
+  
+  /* Determine if the line is mostly left to right or right to left */
+  direction = 0;
+  k = j;
+  while ( k < final_j ) {
+    direction += p->line[i].point[k+1].x - p->line[i].point[k].x;
+    k++;
+  }
+  
+  if ( direction > 0 ) {
+
+    /* j is already correct */
+    inc = 1;
+
+    /* Length of the segment containing the starting point */
+    segment_length = segment_lengths[i][j];
+    /* Determine how far along the segment we need to go */
+    t = 1 - (fwd_line_length - text_start_length) / segment_length;
+    
+  } else {
+
+    j = final_j;
+    inc = -1;
+
+    /* Length of the segment containing the starting point */
+    segment_length = segment_lengths[i][j-1];
+    t = 1 - (rev_line_length - text_start_length) / segment_length;
+    
+  }
+    
+  distance_along_segment = t * segment_length; /* Starting point */
+  
+  theta = 0;
+  k = 0;
+  w = 0;
+  while ( k < labelpath->path.numpoints ) {
+    int m;
+    double x,y;
+      
+    x = t * (p->line[i].point[j+inc].x - p->line[i].point[j].x) + p->line[i].point[j].x;
+    y = t * (p->line[i].point[j+inc].y - p->line[i].point[j].y) + p->line[i].point[j].y;
+
+    /* Average this label point with its neighbors according to the
+       smoothing kernel */
+    
+    if ( k == 0 ) {
+      labelpath->path.point[k].x += (kernel[0] + kernel[1]) * x;      
+      labelpath->path.point[k].y += (kernel[0] + kernel[1]) * y;      
+      
+    } else if ( k == 1 ) {
+      labelpath->path.point[k].x += kernel[0] * x;
+      labelpath->path.point[k].y += kernel[0] * y;
+      
+    } else if ( k == labelpath->path.numpoints - 2 ) {
+      labelpath->path.point[k].x += kernel[4] * x;
+      labelpath->path.point[k].y += kernel[4] * y;
+      
+    } else if ( k == labelpath->path.numpoints - 1 ) {
+      labelpath->path.point[k].x += (kernel[3] + kernel[4]) * x;
+      labelpath->path.point[k].y += (kernel[3] + kernel[4]) * y;
+      }
+      
+    for (m = 0; m < 5; m++) {
+      if ( m + k - 2 < 0 || m + k - 2 > labelpath->path.numpoints - 1 )
+        continue;
+      
+      labelpath->path.point[k+m-2].x += kernel[m]*x;
+      labelpath->path.point[k+m-2].y += kernel[m]*y;
+    }
+    
+    w = letterspacing*offsets[k];
+    
+    /* Add the character's width to the distance along the line */
+    distance_along_segment += w;
+
+    /* If we still have segments left and we've past the current
+       segment, move to the next one */
+    
+    if ( inc == 1 && j < p->line[i].numpoints - 2 ) {
+      
+      while ( j < p->line[i].numpoints - 2 && distance_along_segment > segment_lengths[i][j] ) {
+        distance_along_segment -= segment_lengths[i][j];
+        
+        /* Move to next segment */
+        j += inc;
+      }
+
+      segment_length = segment_lengths[i][j];
+      
+      
+    } else if ( inc == -1 && j > 1 ) {
+
+      while ( j > 1 && distance_along_segment > segment_lengths[i][j-1] ) {
+        distance_along_segment -= segment_lengths[i][j-1];
+        
+        /* Move to next segment */
+        j += inc;
+      }
+      
+      segment_length = segment_lengths[i][j-1];
+      
+    }
+    
+    /* Recalculate interpolation parameter */
+    t = distance_along_segment / segment_length;
+    
+    k++;
+    
+  }
+  
+  /* Pre-calc the character's centre y value.  Used for rotation adjustment.  */
+  cy = -size / 2.0;
+    
+  labelpath->path.point[0].x /= kernel_normal;
+  labelpath->path.point[0].y /= kernel_normal;
+  
+  /* Average the points and calculate each angle */
+  for (k = 1; k <= labelpath->path.numpoints; k++) {
+
+    if ( k < labelpath->path.numpoints ) {  
+      labelpath->path.point[k].x /= kernel_normal;
+      labelpath->path.point[k].y /= kernel_normal;
+      dx = labelpath->path.point[k].x - labelpath->path.point[k-1].x;
+      dy = labelpath->path.point[k].y - labelpath->path.point[k-1].y;
+    } else {
+      /* Handle the last character */
+      dx = t * (p->line[i].point[j+inc].x - p->line[i].point[j].x) + p->line[i].point[j].x - labelpath->path.point[k-1].x;
+      dy = t * (p->line[i].point[j+inc].y - p->line[i].point[j].y) + p->line[i].point[j].y - labelpath->path.point[k-1].y;
+    }
+
+    theta = -atan2(dy,dx);
+
+    /* If the difference between subsequent angles is > 80% of 180deg
+       bail because the line likely overlaps itself. */
+    if ( k > 2 && abs(theta - labelpath->angles[k-2]) > 0.4 * MS_PI ) {
+      *status = MS_FAILURE;
+      goto FAILURE;
+    }
+      
+    /* msDebug("s: %c (x,y): (%0.2f,%0.2f) t: %0.2f\n", string[k-1], labelpath->path.point[k-1].x, labelpath->path.point[k-1].y, theta); */
+
+    labelpath->angles[k-1] = theta;
+  
+    /* Move the previous point so that when the character is rotated and
+       placed it is centred on the line */
+    cos_t = cos(theta);
+    sin_t = sin(theta);
+  
+    w = letterspacing*offsets[k-1];
+
+    cx = 0; /* Center the character vertically only */
+
+    dx = - (cx * cos_t + cy * sin_t);
+    dy = - (cy * cos_t - cx * sin_t);
+  
+    labelpath->path.point[k-1].x += dx;
+    labelpath->path.point[k-1].y += dy;
+
+    /* Calculate the bounds */
+    bbox.minx = 0;
+    bbox.maxx = w;
+    bbox.maxy = 0;
+    bbox.miny = -size;
+
+    /* Add the label buffer to the bounds */
+    bbox.maxx += label->buffer;
+    bbox.maxy += label->buffer;
+    bbox.minx -= label->buffer;
+    bbox.miny -= label->buffer;
+  
+    if ( k < labelpath->path.numpoints ) {
+      /* Transform the bbox too.  We take the UL and LL corners and rotate
+         then translate them. */
+      bounds.point[k-1].x = (bbox.minx * cos_t + bbox.maxy * sin_t) + labelpath->path.point[k-1].x;
+      bounds.point[k-1].y = (bbox.maxy * cos_t - bbox.minx * sin_t) + labelpath->path.point[k-1].y;
+
+      /* Start at end and work towards the half way point */
+      bounds.point[bounds.numpoints - k - 1].x = (bbox.minx * cos_t + bbox.miny * sin_t) + labelpath->path.point[k-1].x;
+      bounds.point[bounds.numpoints - k - 1].y = (bbox.miny * cos_t - bbox.minx * sin_t) + labelpath->path.point[k-1].y;
+
+    } else {
+    
+      /* This is the last character in the string so we take the UR and LR
+         corners of the bbox */
+      bounds.point[k-1].x = (bbox.maxx * cos_t + bbox.maxy * sin_t) + labelpath->path.point[k-1].x;
+      bounds.point[k-1].y = (bbox.maxy * cos_t - bbox.maxx * sin_t) + labelpath->path.point[k-1].y;
+  
+      bounds.point[bounds.numpoints - k - 1].x = (bbox.maxx * cos_t + bbox.miny * sin_t) + labelpath->path.point[k-1].x;
+      bounds.point[bounds.numpoints - k - 1].y = (bbox.miny * cos_t - bbox.maxx * sin_t) + labelpath->path.point[k-1].y;
+    }
+
+  }
+
+  /* Close the bounds */
+  bounds.point[bounds.numpoints - 1].x = bounds.point[0].x;
+  bounds.point[bounds.numpoints - 1].y = bounds.point[0].y;
+  
+  /* Convert the bounds to a shape and store them in the labelpath */
+  if ( msAddLineDirectly(&(labelpath->bounds), &bounds) == MS_FAILURE ) {
+    *status = MS_FAILURE;
+    goto FAILURE;
+  }
+  
+  msComputeBounds(&(labelpath->bounds));
+
+  if ( segment_lengths ) {
+    for ( i = 0; i < p->numlines; i++ )
+      free(segment_lengths[i]);
+    free(segment_lengths);
+  }
+  
+  free(offsets);
+  
+  return labelpath;
+#endif
+
+FAILURE:
+  if ( segment_lengths ) {
+    for ( i = 0; i < p->numlines; i++ )
+      free(segment_lengths[i]);
+    free(segment_lengths);
+  }
+    
+  if ( offsets )
+    free(offsets);
+  
+  if ( labelpath ) {
+    msFreeLabelPathObj(labelpath);
+    labelpath = NULL;
+  }
+    
+  return NULL;
+}
+
 /* ===========================================================================
    Pretty printing of primitive objects
    ======================================================================== */
