@@ -27,6 +27,9 @@
  ******************************************************************************
  *
  * $Log: mapproject.c,v $
+ * Revision 1.51  2006/02/18 21:18:02  frank
+ * Avoid warning about lastPoint.
+ *
  * Revision 1.50  2005/12/23 05:52:48  sdlime
  * Added function to recompute a shape bounds after reprojection. (bug 1586)
  *
@@ -81,7 +84,7 @@
 #include "mapproject.h"
 #include <assert.h>
 
-MS_CVSID("$Id: mapproject.c,v 1.50 2005/12/23 05:52:48 sdlime Exp $")
+MS_CVSID("$Id: mapproject.c,v 1.51 2006/02/18 21:18:02 frank Exp $")
 
 #ifdef USE_PROJ
 static int msTestNeedWrap( pointObj pt1, pointObj pt2, pointObj pt2_geo,
@@ -463,6 +466,8 @@ msProjectShapeLine(projectionObj *in, projectionObj *out,
     if( numpoints_in > 0 )
         firstPoint = line->point[0];
 
+    memset( &lastPoint, 0, sizeof(lastPoint) );
+
 /* -------------------------------------------------------------------- */
 /*      Loop over all input points in linestring.                       */
 /* -------------------------------------------------------------------- */
