@@ -5,7 +5,7 @@
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,15 @@
  ******************************************************************************
  *
  * $Log: mapproject.h,v $
+ * Revision 1.27  2005/06/14 16:03:34  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.26  2005/02/18 03:06:47  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.25  2005/02/17 17:55:19  frank
+ * ignore USE_PROJ_API_H, just always use proj_api.h
+ *
  * Revision 1.24  2004/11/16 21:56:18  dan
  * msGetEPSGProj() obsolete, replaced by msOWSGetEPSGProj() (bug 568)
  *
@@ -46,12 +55,7 @@ extern "C" {
 #endif
 
 #ifdef USE_PROJ
-#  ifdef USE_PROJ_API_H
-#    include <proj_api.h>
-#  else
-#    include <projects.h>
-     typedef PJ *projPJ;
-#  endif
+#  include <proj_api.h>
 #endif
 
 
@@ -70,7 +74,7 @@ typedef struct {
 #else
   void *proj;
 #endif
-  geotransformObj gt; // extra transformation to apply
+  geotransformObj gt; /* extra transformation to apply */
 #endif
 } projectionObj;
 
