@@ -27,6 +27,12 @@
  ******************************************************************************
  *
  * $Log: mapquery.c,v $
+ * Revision 1.91  2006/02/02 01:00:11  sdlime
+ * Moved the code to free previous query results a bit higher in the msQueryByAttribute() function.
+ *
+ * Revision 1.90  2006/02/02 00:53:16  sdlime
+ * Fixed bug when doing repeated queries where you make a layer not queryable then it is possible the results for that layer can persist. (bug 1550)
+ *
  * Revision 1.89  2006/01/01 02:42:39  sdlime
  * The bounds that are part of the result cache are projected so there is no need to do any projection when computing the overall result set bounds.
  *
@@ -55,7 +61,7 @@
 
 #include "map.h"
 
-MS_CVSID("$Id: mapquery.c,v 1.89 2006/01/01 02:42:39 sdlime Exp $")
+MS_CVSID("$Id: mapquery.c,v 1.91 2006/02/02 01:00:11 sdlime Exp $")
 
 /*
 ** msIsLayerQueryable()  returns MS_TRUE/MS_FALSE
@@ -205,9 +211,7 @@ int _msQueryByIndex(mapObj *map, int qlayer, int tileindex, int shapeindex,
 
   msInitShape(&shape);
 
-  if (!addtoquery)
-  {
-  /* free any previous search results, do it now in case one of the next few tests fail */
+  if (!addtoquery) {  
     if(lp->resultcache) {
       if(lp->resultcache->results) free(lp->resultcache->results);
       free(lp->resultcache);
@@ -223,8 +227,7 @@ int _msQueryByIndex(mapObj *map, int qlayer, int tileindex, int shapeindex,
   status = msLayerWhichItems(lp, MS_TRUE, MS_FALSE, NULL);
   if(status != MS_SUCCESS) return(MS_FAILURE);
 
-  if (!addtoquery || lp->resultcache == NULL)
-  {
+  if (!addtoquery || lp->resultcache == NULL) {
     lp->resultcache = (resultCacheObj *)malloc(sizeof(resultCacheObj)); /* allocate and initialize the result cache */
     lp->resultcache->results = NULL;
     lp->resultcache->numresults = lp->resultcache->cachesize = 0;
@@ -296,8 +299,15 @@ int msQueryByAttributes(mapObj *map, int qlayer, char *qitem, char *qstring, int
 
   lp = &(map->layers[qlayer]);
 
+  /* free any previous search results, do now in case one of the following tests fails */
+  if(lp->resultcache) {
+    if(lp->resultcache->results) free(lp->resultcache->results);
+    free(lp->resultcache);
+    lp->resultcache = NULL;
+  }
+
   if(!msIsLayerQueryable(lp)) {
-    msSetError(MS_QUERYERR, "Requested layer has no templates defined.", "msQueryByAttributes()"); 
+    msSetError(MS_QUERYERR, "Requested layer has no templates defined so is not queryable.", "msQueryByAttributes()"); 
     return(MS_FAILURE);
   }
 
@@ -323,13 +333,6 @@ int msQueryByAttributes(mapObj *map, int qlayer, char *qitem, char *qstring, int
 
   msInitShape(&shape);
 
-  /* free any previous search results, do it now in case one of the next few tests fail */
-  if(lp->resultcache) {
-    if(lp->resultcache->results) free(lp->resultcache->results);
-    free(lp->resultcache);
-    lp->resultcache = NULL;
-  }
-
   /* open this layer */
   status = msLayerOpen(lp);
   if(status != MS_SUCCESS) return(MS_FAILURE);
@@ -450,7 +453,6 @@ int msQueryByRect(mapObj *map, int qlayer, rectObj rect)
 
   for(l=start; l>=stop; l--) {
     lp = &(map->layers[l]);
-    if(!msIsLayerQueryable(lp)) continue;
 
     /* free any previous search results, do it now in case one of the next few tests fail */
     if(lp->resultcache) {
@@ -459,6 +461,7 @@ int msQueryByRect(mapObj *map, int qlayer, rectObj rect)
       lp->resultcache = NULL;
     }
 
+    if(!msIsLayerQueryable(lp)) continue;
     if(lp->status == MS_OFF) continue;
 
     if(map->scale > 0) {
@@ -467,12 +470,11 @@ int msQueryByRect(mapObj *map, int qlayer, rectObj rect)
     }    
 
     /* Raster layers are handled specially. */
-    if( lp->type == MS_LAYER_RASTER )
-    {
-        if( msRasterQueryByRect( map, lp, rect ) == MS_FAILURE)
-            return MS_FAILURE;
+    if( lp->type == MS_LAYER_RASTER ) {
+      if( msRasterQueryByRect( map, lp, rect ) == MS_FAILURE)
+        return MS_FAILURE;
 
-        continue;
+      continue;
     }
 
     /* open this layer */
@@ -622,7 +624,6 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
     if(l == slayer) continue; /* skip the selection layer */
     
     lp = &(map->layers[l]);
-    if(!msIsLayerQueryable(lp)) continue;
 
     /* free any previous search results, do it now in case one of the next few tests fail */
     if(lp->resultcache) {
@@ -630,7 +631,8 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
       free(lp->resultcache);
       lp->resultcache = NULL;
     }
-    
+
+    if(!msIsLayerQueryable(lp)) continue;    
     if(lp->status == MS_OFF) continue;
     
     if(map->scale > 0) {
@@ -835,16 +837,15 @@ int msQueryByPoint(mapObj *map, int qlayer, int mode, pointObj p, double buffer)
 
   for(l=start; l>=stop; l--) {
     lp = &(map->layers[l]);    
-    if(!msIsLayerQueryable(lp)) continue;
 
-    /* free any previous search results, do it now in case one of the next */
-    /* few tests fail */
+    /* free any previous search results, do it now in case one of the next few tests fail */
     if(lp->resultcache) {
       if(lp->resultcache->results) free(lp->resultcache->results);
       free(lp->resultcache);
       lp->resultcache = NULL;
     }
 
+    if(!msIsLayerQueryable(lp)) continue;
     if(lp->status == MS_OFF) continue;
 
     if(map->scale > 0) {
@@ -996,7 +997,6 @@ int msQueryByShape(mapObj *map, int qlayer, shapeObj *selectshape)
  
   for(l=start; l>=stop; l--) { /* each layer */
     lp = &(map->layers[l]);
-    if(!msIsLayerQueryable(lp)) continue;
 
     /* free any previous search results, do it now in case one of the next few tests fail */
     if(lp->resultcache) {
@@ -1005,6 +1005,7 @@ int msQueryByShape(mapObj *map, int qlayer, shapeObj *selectshape)
       lp->resultcache = NULL;
     }
 
+    if(!msIsLayerQueryable(lp)) continue;
     if(lp->status == MS_OFF) continue;
     
     if(map->scale > 0) {
@@ -1188,25 +1189,24 @@ int msGetQueryResultBounds(mapObj *map, rectObj *bounds)
  */
 void msQueryFree(mapObj *map, int qlayer)
 {
-    int l; /* counters */
-    int start, stop=0;
-    layerObj *lp;
+  int l; /* counters */
+  int start, stop=0;
+  layerObj *lp;
     
-    if(qlayer < 0 || qlayer >= map->numlayers)
-      start = map->numlayers-1;
-    else
-      start = stop = qlayer;
+  if(qlayer < 0 || qlayer >= map->numlayers)
+    start = map->numlayers-1;
+  else
+    start = stop = qlayer;
 
-    for(l=start; l>=stop; l--) 
-    {
-        lp = &(map->layers[l]);
-        if(lp->resultcache) 
-        {
-            if(lp->resultcache->results) 
-              free(lp->resultcache->results);
-            free(lp->resultcache);
-            lp->resultcache = NULL;
-        }
+  for(l=start; l>=stop; l--) {
+    lp = &(map->layers[l]);
+        
+    if(lp->resultcache) {
+      if(lp->resultcache->results) 
+        free(lp->resultcache->results);
+      free(lp->resultcache);
+      lp->resultcache = NULL;
     }
+  }
 }
       
