@@ -29,6 +29,15 @@
  ******************************************************************************
  *
  * $Log: mapraster.c,v $
+ * Revision 1.137  2006/09/24 02:42:12  frank
+ * handle upsidedown images through resample logic. (bug 1904)
+ *
+ * Revision 1.136  2006/04/18 17:20:37  frank
+ * Support large (>2GB) raster files relative to SHAPEPATH. (bug 1748)
+ *
+ * Revision 1.135  2006/02/18 21:14:55  frank
+ * Fixed regex test.
+ *
  * Revision 1.134  2005/12/15 16:37:06  frank
  * Last fix was for bug 1562
  *
@@ -111,7 +120,7 @@
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapraster.c,v 1.134 2005/12/15 16:37:06 frank Exp $")
+MS_CVSID("$Id: mapraster.c,v 1.137 2006/09/24 02:42:12 frank Exp $")
 
 extern int msyyparse(void);
 extern int msyylex(void);
@@ -1372,7 +1381,7 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
               free(pszTmp);
           }
           else if (layer->filter.type == MS_REGEX || 
-                   layer->filter.type == MS_REGEX == MS_IREGEX)
+                   layer->filter.type == MS_IREGEX)
           {
               pszTmp = 
                 (char *)malloc(sizeof(char)*(strlen(layer->filter.string)+3));
@@ -1475,10 +1484,21 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
 
     msBuildPath3(szPath, map->mappath, map->shapepath, filename);
 
+    /*
+    ** Try to open the file, and read the first 8 bytes as a signature. 
+    ** If the open fails for a reason other than "bigness" then we use
+    ** the filename unaltered by path logic since it might be something
+    ** very virtual.
+    */
     f = fopen( szPath, "rb");
     if(!f) {
       memset( dd, 0, 8 );
+#ifdef EFBIG
+      if( errno != EFBIG )
+          strcpy( szPath, filename );
+#else
       strcpy( szPath, filename );
+#endif
     } else {
       fread(dd,8,1,f); /* read some bytes to try and identify the file */
       fclose(f);
@@ -1565,10 +1585,11 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
             /* 
             ** We want to resample if the source image is rotated, if
             ** the projections differ or if resampling has been explicitly
-            ** requested.
+            ** requested, or if the image has north-down instead of north-up.
             */
 #ifdef USE_PROJ
-            if( ((adfGeoTransform[2] != 0.0 || adfGeoTransform[4] != 0.0)
+            if( ((adfGeoTransform[2] != 0.0 || adfGeoTransform[4] != 0.0
+                  || adfGeoTransform[5] > 0.0 || adfGeoTransform[1] < 0.0 )
                  && layer->transform )
                 || msProjectionsDiffer( &(map->projection), 
                                         &(layer->projection) ) 
