@@ -27,7 +27,10 @@
  ******************************************************************************
  *
  * $Log: mapresample.c,v $
- * Revision 1.65.2.1  2006/03/15 20:32:48  frank
+ * Revision 1.67  2006/03/15 20:31:18  frank
+ * Fixup comment for last change.
+ *
+ * Revision 1.66  2006/03/15 20:29:12  frank
  * Fix inter-tile cracking problem (Bug 1715).
  *
  * Revision 1.65  2005/12/15 16:37:28  frank
@@ -131,7 +134,7 @@
 #include <assert.h>
 #include "mapresample.h"
 
-MS_CVSID("$Id: mapresample.c,v 1.65.2.1 2006/03/15 20:32:48 frank Exp $")
+MS_CVSID("$Id: mapresample.c,v 1.67 2006/03/15 20:31:18 frank Exp $")
 
 #ifndef MAX
 #  define MIN(a,b)      ((a<b) ? a : b)
