@@ -1,4 +1,4 @@
-// $Id: MapTest.java,v 1.1 2005/06/09 19:21:18 dan Exp $
+// $Id: MapTest.java,v 1.2 2006/06/28 13:46:22 umberto Exp $
 //
 // See README_THREADTEST.TXT for usage details.
 //
@@ -26,7 +26,7 @@ public class MapTest {
 
         Thread[]    tpool = new Thread[threads.intValue()];
         for( int i = 0; i < tpool.length; i++ ) {
-            tpool[i] = new MapThread(mapfile, iterations.intValue());
+            tpool[i] = new MapThread(mapfile, iterations.intValue(), i);
         }
         for( int i = 0; i < tpool.length; i++ ) {
             tpool[i].start();
