@@ -4,9 +4,33 @@
  *
  * This file was originally based on the SWIG interface file mapscript.i
  *
- * $Id: mapscript_i.c,v 1.95 2005/10/13 16:00:36 assefa Exp $
+ * $Id: mapscript_i.c,v 1.102 2006/09/01 02:30:15 sdlime Exp $
  *
  * $Log: mapscript_i.c,v $
+ * Revision 1.102  2006/09/01 02:30:15  sdlime
+ * Dan beat me to the bug 1428 fix. I took a bit futher by removing msLayerGetFilterString() from layerobject.c and refer to that in the mapscript getFilter/getFilterString methods.
+ *
+ * Revision 1.101  2006/08/31 20:48:47  dan
+ * Fixed MapScript getExpressionString() that was failing on expressions
+ * longer that 256 chars (SWIG) and 512 chars (PHP). Also moved all that
+ * code to a msGetExpressionString() in mapfile.c (bug 1428)
+ *
+ * Revision 1.100  2006/08/29 01:56:53  sdlime
+ * Fixed buffer overflow with POSTs and huge numbers of name/value pairs. Reduced MAX_PARAMS (now MS_MAX_CGI_PARAMS) from 10,000 to 100.
+ *
+ * Revision 1.99  2006/08/22 15:55:03  assefa
+ * Adding geos functions to php mapscript (Bug 1327)
+ *
+ * Revision 1.98  2006/05/29 19:02:01  assefa
+ * Update PHP mapscript to support addition of MapScript WxS Services
+ * (RFC 16, Bug 1790)
+ *
+ * Revision 1.97  2006/05/17 16:04:55  assefa
+ * Add geos functions union, difference and intersection (Bug 1778)
+ *
+ * Revision 1.96  2006/01/20 15:03:35  assefa
+ * Add containsshape function using uderlying GEOS function (Bug 1623).
+ *
  * Revision 1.95  2005/10/13 16:00:36  assefa
  * Add support for whichshape and nextshape (Bug 1491)
  * Adsd support for config setting/getting at the map level (Bug 1487).
@@ -404,6 +428,18 @@ char *mapObj_generateSLD(mapObj *self)
     return msSLDGenerateSLD(self, -1);
 }
 
+
+int mapObj_loadOWSParameters(mapObj *self, cgiRequestObj *request, 
+                              char *wmtver_string)
+{
+    return msMapLoadOWSParameters(self, request, wmtver_string);
+}
+
+int mapObj_OWSDispatch(mapObj *self, cgiRequestObj *req )
+{
+    return msOWSDispatch( self, req);
+}
+
 /**********************************************************************
  * class extensions for layerObj, always within the context of a map
  **********************************************************************/
@@ -558,7 +594,7 @@ int layerObj_setFilter(layerObj *self, char *string) {
   }
 
 char *layerObj_getFilter(layerObj *self) {
-    return msLayerGetFilterString(self);
+    return msGetExpressionString(&(self->filter));
   }
 
 int layerObj_setWKTProjection(layerObj *self, char *string) {
@@ -677,31 +713,12 @@ int classObj_setExpression(classObj *self, char *string) {
   }
 
 char *classObj_getExpressionString(classObj *self) {
-  char exprstring[512];
-
-  if (self->expression.string)
-  {
-      switch(self->expression.type)
-      {
-          case(MS_REGEX):
-            sprintf(exprstring, "/%s/", self->expression.string);
-            return strdup(exprstring);
-          case(MS_STRING):
-            sprintf(exprstring, "\"%s\"", self->expression.string);
-            return strdup(exprstring);
-          case(MS_EXPRESSION):
-            sprintf(exprstring, "(%s)", self->expression.string);
-            return strdup(exprstring);
-      }
-    }
-    return NULL;
+    return msGetExpressionString(&(self->expression));
 }
 
-
-
 int classObj_setText(classObj *self, layerObj *layer, char *string) {
     return msLoadExpressionString(&self->text, string);
-  }
+}
 
 int classObj_drawLegendIcon(classObj *self, mapObj *map, layerObj *layer, int width, int height, gdImagePtr dstImg, int dstX, int dstY) {
   msClearLayerPenValues(layer); // just in case the mapfile has already been processed
@@ -964,6 +981,84 @@ shapeObj *shapeObj_convexHull(shapeObj *self)
 {
     return msGEOSConvexHull(self);
 }
+
+shapeObj *shapeObj_boundary(shapeObj *self)
+{
+    return msGEOSBoundary(self);
+}
+
+int shapeObj_contains_geos(shapeObj *self, shapeObj *shape)
+{
+    return msGEOSContains(self, shape);
+}
+
+shapeObj *shapeObj_Union(shapeObj *self, shapeObj *shape)
+{
+     return msGEOSUnion(self, shape);
+}
+
+shapeObj *shapeObj_intersection(shapeObj *self, shapeObj *shape)
+{
+     return msGEOSIntersection(self, shape);
+}
+
+shapeObj *shapeObj_difference(shapeObj *self, shapeObj *shape)
+{
+     return msGEOSDifference(self, shape);
+}
+ 
+shapeObj *shapeObj_symdifference(shapeObj *self, shapeObj *shape)
+{
+     return msGEOSSymDifference(self, shape);
+}
+
+int shapeObj_overlaps(shapeObj *self, shapeObj *shape)
+{
+    return msGEOSOverlaps(self, shape);
+}
+ 
+int shapeObj_within(shapeObj *self, shapeObj *shape)
+{
+    return msGEOSWithin(self, shape);
+}
+
+int shapeObj_crosses(shapeObj *self, shapeObj *shape)
+{
+    return msGEOSCrosses(self, shape);
+}
+
+int shapeObj_touches(shapeObj *self, shapeObj *shape)
+{
+    return msGEOSTouches(self, shape);
+}
+ 
+int shapeObj_equals(shapeObj *self, shapeObj *shape)
+{
+    return msGEOSEquals(self, shape);
+}
+ 
+int shapeObj_disjoint(shapeObj *self, shapeObj *shape)
+{
+    return msGEOSDisjoint(self, shape);
+}
+ 
+
+pointObj *shapeObj_getcentroid(shapeObj *self)
+{
+    return msGEOSGetCentroid(self);
+}
+
+double shapeObj_getarea(shapeObj *self)
+{
+    return msGEOSArea(self);
+}
+
+double shapeObj_getlength(shapeObj *self)
+{
+    return msGEOSLength(self);
+}
+
+ 
 /**********************************************************************
  * class extensions for rectObj
  **********************************************************************/
@@ -1191,3 +1286,79 @@ styleObj *styleObj_clone(styleObj *style){
   
   return newstyle;
 }
+
+
+cgiRequestObj *cgirequestObj_new()
+{
+    cgiRequestObj *request;
+    request = msAllocCgiObj();
+
+    request->ParamNames = (char **) malloc(MS_MAX_CGI_PARAMS*sizeof(char*));
+    request->ParamValues = (char **) malloc(MS_MAX_CGI_PARAMS*sizeof(char*));
+
+    return request;
+}
+
+int cgirequestObj_loadParams(cgiRequestObj *self)
+{
+  self->NumParams = loadParams(self);
+  return self->NumParams;
+}
+
+
+void cgirequestObj_setParameter(cgiRequestObj *self, char *name, char *value)
+{
+    int i;
+        
+    if (self->NumParams == MS_MAX_CGI_PARAMS) {
+      msSetError(MS_CHILDERR, "Maximum number of items, %d, has been reached", "setItem()", MS_MAX_CGI_PARAMS);
+    }
+        
+    for (i=0; i<self->NumParams; i++) {
+      if (strcasecmp(self->ParamNames[i], name) == 0) {
+        free(self->ParamValues[i]);
+                self->ParamValues[i] = strdup(value);
+                break;
+      }
+    }
+    if (i == self->NumParams) {
+      self->ParamNames[self->NumParams] = strdup(name);
+      self->ParamValues[self->NumParams] = strdup(value);
+      self->NumParams++;
+    }
+}
+
+char *cgirequestObj_getName(cgiRequestObj *self, int index)
+{
+    if (index < 0 || index >= self->NumParams) {
+            msSetError(MS_CHILDERR, "Invalid index, valid range is [0, %d]", "getName()", self->NumParams-1);
+            return NULL;
+        }
+        return self->ParamNames[index];
+}
+
+char *cgirequestObj_getValue(cgiRequestObj *self, int index) 
+{
+    if (index < 0 || index >= self->NumParams) {
+            msSetError(MS_CHILDERR, "Invalid index, valid range is [0, %d]", "getValue()", self->NumParams-1);
+            return NULL;
+        }
+        return self->ParamValues[index];
+}
+ 
+char *cgirequestObj_getValueByName(cgiRequestObj *self, const char *name) 
+{
+    int i;
+    for (i=0; i<self->NumParams; i++) {
+        if (strcasecmp(self->ParamNames[i], name) == 0) {
+            return self->ParamValues[i];
+        }
+    }
+    return NULL;
+}
+void cgirequestObj_destroy(cgiRequestObj *self)
+{
+    msFreeCharArray(self->ParamNames, self->NumParams);
+    msFreeCharArray(self->ParamValues, self->NumParams);
+    free(self);
+}
