@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: php_mapscript.h,v 1.53 2005/10/13 16:00:36 assefa Exp $
+ * $Id: php_mapscript.h,v 1.57 2006/08/22 15:55:03 assefa Exp $
  *
  * Name:     php_mapscript.c
  * Project:  PHP/MapScript extension for MapServer.
@@ -30,6 +30,19 @@
  **********************************************************************
  *
  * $Log: php_mapscript.h,v $
+ * Revision 1.57  2006/08/22 15:55:03  assefa
+ * Adding geos functions to php mapscript (Bug 1327)
+ *
+ * Revision 1.56  2006/05/29 19:02:01  assefa
+ * Update PHP mapscript to support addition of MapScript WxS Services
+ * (RFC 16, Bug 1790)
+ *
+ * Revision 1.55  2006/05/17 16:04:55  assefa
+ * Add geos functions union, difference and intersection (Bug 1778)
+ *
+ * Revision 1.54  2006/01/20 15:03:35  assefa
+ * Add containsshape function using uderlying GEOS function (Bug 1623).
+ *
  * Revision 1.53  2005/10/13 16:00:36  assefa
  * Add support for whichshape and nextshape (Bug 1491)
  * Adsd support for config setting/getting at the map level (Bug 1487).
@@ -196,6 +209,9 @@ int             mapObj_selectOutputFormat(mapObj *self,
 int             mapObj_applySLD(mapObj *self, char *sld);
 int             mapObj_applySLDURL(mapObj *self, char *sld);
 char            *mapObj_generateSLD(mapObj *self);
+int             mapObj_loadOWSParameters(mapObj *self, cgiRequestObj *request, 
+                                          char *wmtver_string);
+int             mapObj_OWSDispatch(mapObj *self, cgiRequestObj *req );
 
 layerObj       *layerObj_new(mapObj *map);
 void            layerObj_destroy(layerObj* self);
@@ -302,8 +318,24 @@ int             shapeObj_contains(shapeObj *self, pointObj *point);
 int             shapeObj_intersects(shapeObj *self, shapeObj *shape);
 pointObj        *shapeObj_getpointusingmeasure(shapeObj *self, double m);
 pointObj        *shapeObj_getmeasureusingpoint(shapeObj *self, pointObj *point);
+
 shapeObj        *shapeObj_buffer(shapeObj *self, double width);
 shapeObj        *shapeObj_convexHull(shapeObj *self);
+shapeObj        *shapeObj_boundary(shapeObj *self);
+int             shapeObj_contains_geos(shapeObj *self, shapeObj *poshape);
+shapeObj        *shapeObj_Union(shapeObj *self, shapeObj *poshape);
+shapeObj        *shapeObj_intersection(shapeObj *self, shapeObj *poshape);
+shapeObj        *shapeObj_difference(shapeObj *self, shapeObj *poshape);
+shapeObj        *shapeObj_symdifference(shapeObj *self, shapeObj *poshape);
+int             shapeObj_overlaps(shapeObj *self, shapeObj *shape);
+int             shapeObj_within(shapeObj *self, shapeObj *shape);
+int             shapeObj_crosses(shapeObj *self, shapeObj *shape);
+int             shapeObj_touches(shapeObj *self, shapeObj *shape);
+int             shapeObj_equals(shapeObj *self, shapeObj *shape);
+int             shapeObj_disjoint(shapeObj *self, shapeObj *shape);
+pointObj        *shapeObj_getcentroid(shapeObj *self);
+double          shapeObj_getarea(shapeObj *self);
+double          shapeObj_getlength(shapeObj *self);
 
 rectObj        *rectObj_new();
 void            rectObj_destroy(rectObj *self);
@@ -352,4 +384,13 @@ int            *hashTableObj_remove(hashTableObj *self, const char *key);
 void           *hashTableObj_clear(hashTableObj *self);
 char           *hashTableObj_nextKey(hashTableObj *self, const char *prevkey);
 
+
+cgiRequestObj *cgirequestObj_new();
+int cgirequestObj_loadParams(cgiRequestObj *self);
+void cgirequestObj_setParameter(cgiRequestObj *self, char *name, char *value);
+char *cgirequestObj_getName(cgiRequestObj *self, int index);
+char *cgirequestObj_getValue(cgiRequestObj *self, int index);
+char *cgirequestObj_getValueByName(cgiRequestObj *self, const char *name);
+void cgirequestObj_destroy(cgiRequestObj *self);
+
 #endif /* _PHP_MAPSCRIPT_H_INCLUDED_ */
