@@ -57,6 +57,7 @@ MS_METERS = _mapscript.MS_METERS
 MS_KILOMETERS = _mapscript.MS_KILOMETERS
 MS_DD = _mapscript.MS_DD
 MS_PIXELS = _mapscript.MS_PIXELS
+MS_PERCENTAGES = _mapscript.MS_PERCENTAGES
 MS_SHAPE_POINT = _mapscript.MS_SHAPE_POINT
 MS_SHAPE_LINE = _mapscript.MS_SHAPE_LINE
 MS_SHAPE_POLYGON = _mapscript.MS_SHAPE_POLYGON
@@ -71,6 +72,7 @@ MS_LAYER_CIRCLE = _mapscript.MS_LAYER_CIRCLE
 MS_LAYER_TILEINDEX = _mapscript.MS_LAYER_TILEINDEX
 MS_TRUETYPE = _mapscript.MS_TRUETYPE
 MS_BITMAP = _mapscript.MS_BITMAP
+MS_POSITIONS_LENGTH = _mapscript.MS_POSITIONS_LENGTH
 MS_UL = _mapscript.MS_UL
 MS_LR = _mapscript.MS_LR
 MS_UR = _mapscript.MS_UR
@@ -82,6 +84,7 @@ MS_LC = _mapscript.MS_LC
 MS_CC = _mapscript.MS_CC
 MS_AUTO = _mapscript.MS_AUTO
 MS_XY = _mapscript.MS_XY
+MS_FOLLOW = _mapscript.MS_FOLLOW
 MS_TINY = _mapscript.MS_TINY
 MS_SMALL = _mapscript.MS_SMALL
 MS_MEDIUM = _mapscript.MS_MEDIUM
@@ -133,7 +136,7 @@ class fontSetObj(object):
         return "<C fontSetObj instance at %s>" % (self.this,)
     filename = property(_mapscript.fontSetObj_filename_get)
     numfonts = property(_mapscript.fontSetObj_numfonts_get)
-    fonts = property(_mapscript.fontSetObj_fonts_get, _mapscript.fontSetObj_fonts_set)
+    fonts = property(_mapscript.fontSetObj_fonts_get)
     def __init__(self, *args):
         newobj = _mapscript.new_fontSetObj(*args)
         self.this = newobj.this
@@ -236,6 +239,7 @@ class labelObj(object):
     offsety = property(_mapscript.labelObj_offsety_get, _mapscript.labelObj_offsety_set)
     angle = property(_mapscript.labelObj_angle_get, _mapscript.labelObj_angle_set)
     autoangle = property(_mapscript.labelObj_autoangle_get, _mapscript.labelObj_autoangle_set)
+    autofollow = property(_mapscript.labelObj_autofollow_get, _mapscript.labelObj_autofollow_set)
     buffer = property(_mapscript.labelObj_buffer_get, _mapscript.labelObj_buffer_set)
     antialias = property(_mapscript.labelObj_antialias_get, _mapscript.labelObj_antialias_set)
     wrap = property(_mapscript.labelObj_wrap_get, _mapscript.labelObj_wrap_set)
@@ -268,7 +272,7 @@ class webObj(object):
     log = property(_mapscript.webObj_log_get, _mapscript.webObj_log_set)
     imagepath = property(_mapscript.webObj_imagepath_get, _mapscript.webObj_imagepath_set)
     imageurl = property(_mapscript.webObj_imageurl_get, _mapscript.webObj_imageurl_set)
-    map = property(_mapscript.webObj_map_get, _mapscript.webObj_map_set)
+    map = property(_mapscript.webObj_map_get)
     template = property(_mapscript.webObj_template_get, _mapscript.webObj_template_set)
     header = property(_mapscript.webObj_header_get, _mapscript.webObj_header_set)
     footer = property(_mapscript.webObj_footer_get, _mapscript.webObj_footer_set)
@@ -282,7 +286,7 @@ class webObj(object):
     queryformat = property(_mapscript.webObj_queryformat_get, _mapscript.webObj_queryformat_set)
     legendformat = property(_mapscript.webObj_legendformat_get, _mapscript.webObj_legendformat_set)
     browseformat = property(_mapscript.webObj_browseformat_get, _mapscript.webObj_browseformat_set)
-    metadata = property(_mapscript.webObj_metadata_get, _mapscript.webObj_metadata_set)
+    metadata = property(_mapscript.webObj_metadata_get)
     def __init__(self, *args):
         newobj = _mapscript.new_webObj(*args)
         self.this = newobj.this
@@ -336,6 +340,7 @@ class styleObj(object):
             if self.thisown: destroy(self)
         except: pass
     def clone(*args): return _mapscript.styleObj_clone(*args)
+    def setSymbolByName(*args): return _mapscript.styleObj_setSymbolByName(*args)
 
 class styleObjPtr(styleObj):
     def __init__(self, this):
@@ -349,15 +354,15 @@ class classObj(object):
         return "<C classObj instance at %s>" % (self.this,)
     status = property(_mapscript.classObj_status_get, _mapscript.classObj_status_set)
     numstyles = property(_mapscript.classObj_numstyles_get, _mapscript.classObj_numstyles_set)
-    label = property(_mapscript.classObj_label_get, _mapscript.classObj_label_set)
+    label = property(_mapscript.classObj_label_get)
     name = property(_mapscript.classObj_name_get, _mapscript.classObj_name_set)
     title = property(_mapscript.classObj_title_get, _mapscript.classObj_title_set)
     template = property(_mapscript.classObj_template_get, _mapscript.classObj_template_set)
     type = property(_mapscript.classObj_type_get, _mapscript.classObj_type_set)
-    metadata = property(_mapscript.classObj_metadata_get, _mapscript.classObj_metadata_set)
+    metadata = property(_mapscript.classObj_metadata_get)
     minscale = property(_mapscript.classObj_minscale_get, _mapscript.classObj_minscale_set)
     maxscale = property(_mapscript.classObj_maxscale_get, _mapscript.classObj_maxscale_set)
-    layer = property(_mapscript.classObj_layer_get, _mapscript.classObj_layer_set)
+    layer = property(_mapscript.classObj_layer_get)
     debug = property(_mapscript.classObj_debug_get, _mapscript.classObj_debug_set)
     keyimage = property(_mapscript.classObj_keyimage_get, _mapscript.classObj_keyimage_set)
     def __init__(self, *args):
@@ -408,6 +413,7 @@ class labelCacheMemberObj(object):
     point = property(_mapscript.labelCacheMemberObj_point_get)
     poly = property(_mapscript.labelCacheMemberObj_poly_get)
     status = property(_mapscript.labelCacheMemberObj_status_get)
+    labelpath = property(_mapscript.labelCacheMemberObj_labelpath_get)
     def __init__(self, *args):
         newobj = _mapscript.new_labelCacheMemberObj(*args)
         self.this = newobj.this
@@ -565,7 +571,7 @@ class referenceMapObj(object):
     markersize = property(_mapscript.referenceMapObj_markersize_get, _mapscript.referenceMapObj_markersize_set)
     minboxsize = property(_mapscript.referenceMapObj_minboxsize_get, _mapscript.referenceMapObj_minboxsize_set)
     maxboxsize = property(_mapscript.referenceMapObj_maxboxsize_get, _mapscript.referenceMapObj_maxboxsize_set)
-    map = property(_mapscript.referenceMapObj_map_get, _mapscript.referenceMapObj_map_set)
+    map = property(_mapscript.referenceMapObj_map_get)
     def __init__(self, *args):
         newobj = _mapscript.new_referenceMapObj(*args)
         self.this = newobj.this
@@ -620,7 +626,7 @@ class legendObj(object):
     def __repr__(self):
         return "<C legendObj instance at %s>" % (self.this,)
     imagecolor = property(_mapscript.legendObj_imagecolor_get, _mapscript.legendObj_imagecolor_set)
-    label = property(_mapscript.legendObj_label_get, _mapscript.legendObj_label_set)
+    label = property(_mapscript.legendObj_label_get)
     keysizex = property(_mapscript.legendObj_keysizex_get, _mapscript.legendObj_keysizex_set)
     keysizey = property(_mapscript.legendObj_keysizey_get, _mapscript.legendObj_keysizey_set)
     keyspacingx = property(_mapscript.legendObj_keyspacingx_get, _mapscript.legendObj_keyspacingx_set)
@@ -632,7 +638,7 @@ class legendObj(object):
     position = property(_mapscript.legendObj_position_get, _mapscript.legendObj_position_set)
     postlabelcache = property(_mapscript.legendObj_postlabelcache_get, _mapscript.legendObj_postlabelcache_set)
     template = property(_mapscript.legendObj_template_get, _mapscript.legendObj_template_set)
-    map = property(_mapscript.legendObj_map_get, _mapscript.legendObj_map_set)
+    map = property(_mapscript.legendObj_map_get)
     def __init__(self, *args):
         newobj = _mapscript.new_legendObj(*args)
         self.this = newobj.this
@@ -681,9 +687,6 @@ class layerObj(object):
     labelitem = property(_mapscript.layerObj_labelitem_get, _mapscript.layerObj_labelitem_set)
     labelsizeitem = property(_mapscript.layerObj_labelsizeitem_get, _mapscript.layerObj_labelsizeitem_set)
     labelangleitem = property(_mapscript.layerObj_labelangleitem_get, _mapscript.layerObj_labelangleitem_set)
-    labelitemindex = property(_mapscript.layerObj_labelitemindex_get, _mapscript.layerObj_labelitemindex_set)
-    labelsizeitemindex = property(_mapscript.layerObj_labelsizeitemindex_get, _mapscript.layerObj_labelsizeitemindex_set)
-    labelangleitemindex = property(_mapscript.layerObj_labelangleitemindex_get, _mapscript.layerObj_labelangleitemindex_set)
     tileitem = property(_mapscript.layerObj_tileitem_get, _mapscript.layerObj_tileitem_set)
     tileindex = property(_mapscript.layerObj_tileindex_get, _mapscript.layerObj_tileindex_set)
     units = property(_mapscript.layerObj_units_get, _mapscript.layerObj_units_set)
@@ -697,7 +700,7 @@ class layerObj(object):
     styleitem = property(_mapscript.layerObj_styleitem_get, _mapscript.layerObj_styleitem_set)
     requires = property(_mapscript.layerObj_requires_get, _mapscript.layerObj_requires_set)
     labelrequires = property(_mapscript.layerObj_labelrequires_get, _mapscript.layerObj_labelrequires_set)
-    metadata = property(_mapscript.layerObj_metadata_get, _mapscript.layerObj_metadata_set)
+    metadata = property(_mapscript.layerObj_metadata_get)
     transparency = property(_mapscript.layerObj_transparency_get, _mapscript.layerObj_transparency_set)
     dump = property(_mapscript.layerObj_dump_get, _mapscript.layerObj_dump_set)
     debug = property(_mapscript.layerObj_debug_get, _mapscript.layerObj_debug_set)
@@ -779,9 +782,9 @@ class mapObj(object):
     width = property(_mapscript.mapObj_width_get, _mapscript.mapObj_width_set)
     maxsize = property(_mapscript.mapObj_maxsize_get, _mapscript.mapObj_maxsize_set)
     numlayers = property(_mapscript.mapObj_numlayers_get)
-    symbolset = property(_mapscript.mapObj_symbolset_get, _mapscript.mapObj_symbolset_set)
-    fontset = property(_mapscript.mapObj_fontset_get, _mapscript.mapObj_fontset_set)
-    labelcache = property(_mapscript.mapObj_labelcache_get, _mapscript.mapObj_labelcache_set)
+    symbolset = property(_mapscript.mapObj_symbolset_get)
+    fontset = property(_mapscript.mapObj_fontset_get)
+    labelcache = property(_mapscript.mapObj_labelcache_get)
     transparent = property(_mapscript.mapObj_transparent_get, _mapscript.mapObj_transparent_set)
     interlace = property(_mapscript.mapObj_interlace_get, _mapscript.mapObj_interlace_set)
     imagequality = property(_mapscript.mapObj_imagequality_get, _mapscript.mapObj_imagequality_set)
@@ -798,16 +801,16 @@ class mapObj(object):
     outputformatlist = property(_mapscript.mapObj_outputformatlist_get)
     outputformat = property(_mapscript.mapObj_outputformat_get)
     imagetype = property(_mapscript.mapObj_imagetype_get)
-    reference = property(_mapscript.mapObj_reference_get, _mapscript.mapObj_reference_set)
-    scalebar = property(_mapscript.mapObj_scalebar_get, _mapscript.mapObj_scalebar_set)
-    legend = property(_mapscript.mapObj_legend_get, _mapscript.mapObj_legend_set)
-    querymap = property(_mapscript.mapObj_querymap_get, _mapscript.mapObj_querymap_set)
-    web = property(_mapscript.mapObj_web_get, _mapscript.mapObj_web_set)
+    reference = property(_mapscript.mapObj_reference_get)
+    scalebar = property(_mapscript.mapObj_scalebar_get)
+    legend = property(_mapscript.mapObj_legend_get)
+    querymap = property(_mapscript.mapObj_querymap_get)
+    web = property(_mapscript.mapObj_web_get)
     layerorder = property(_mapscript.mapObj_layerorder_get, _mapscript.mapObj_layerorder_set)
     debug = property(_mapscript.mapObj_debug_get, _mapscript.mapObj_debug_set)
     datapattern = property(_mapscript.mapObj_datapattern_get, _mapscript.mapObj_datapattern_set)
     templatepattern = property(_mapscript.mapObj_templatepattern_get, _mapscript.mapObj_templatepattern_set)
-    configoptions = property(_mapscript.mapObj_configoptions_get, _mapscript.mapObj_configoptions_set)
+    configoptions = property(_mapscript.mapObj_configoptions_get)
     def __init__(self, *args):
         newobj = _mapscript.new_mapObj(*args)
         self.this = newobj.this
@@ -839,6 +842,7 @@ class mapObj(object):
     def embedScalebar(*args): return _mapscript.mapObj_embedScalebar(*args)
     def embedLegend(*args): return _mapscript.mapObj_embedLegend(*args)
     def drawLabelCache(*args): return _mapscript.mapObj_drawLabelCache(*args)
+    def getLabel(*args): return _mapscript.mapObj_getLabel(*args)
     def nextLabel(*args): return _mapscript.mapObj_nextLabel(*args)
     def queryByPoint(*args): return _mapscript.mapObj_queryByPoint(*args)
     def queryByRect(*args): return _mapscript.mapObj_queryByRect(*args)
@@ -879,6 +883,7 @@ class mapObj(object):
     def appendOutputFormat(*args): return _mapscript.mapObj_appendOutputFormat(*args)
     def removeOutputFormat(*args): return _mapscript.mapObj_removeOutputFormat(*args)
     def loadOWSParameters(*args): return _mapscript.mapObj_loadOWSParameters(*args)
+    def OWSDispatch(*args): return _mapscript.mapObj_OWSDispatch(*args)
     def zoomPoint(*args): return _mapscript.mapObj_zoomPoint(*args)
     def zoomRectangle(*args): return _mapscript.mapObj_zoomRectangle(*args)
     def zoomScale(*args): return _mapscript.mapObj_zoomScale(*args)
@@ -912,7 +917,7 @@ class imageObj(object):
     height = property(_mapscript.imageObj_height_get)
     imagepath = property(_mapscript.imageObj_imagepath_get)
     imageurl = property(_mapscript.imageObj_imageurl_get)
-    format = property(_mapscript.imageObj_format_get, _mapscript.imageObj_format_set)
+    format = property(_mapscript.imageObj_format_get)
     renderer = property(_mapscript.imageObj_renderer_get, _mapscript.imageObj_renderer_set)
     size = property(_mapscript.imageObj_size_get, _mapscript.imageObj_size_set)
     def __del__(self, destroy=_mapscript.delete_imageObj):
@@ -1012,6 +1017,7 @@ class pointObj(object):
     def setXYZ(*args): return _mapscript.pointObj_setXYZ(*args)
     def setXYZM(*args): return _mapscript.pointObj_setXYZM(*args)
     def toString(*args): return _mapscript.pointObj_toString(*args)
+    def toShape(*args): return _mapscript.pointObj_toShape(*args)
     def __str__(self):
         return self.toString()
 
@@ -1083,12 +1089,27 @@ class shapeObj(object):
     def toWKT(*args): return _mapscript.shapeObj_toWKT(*args)
     def buffer(*args): return _mapscript.shapeObj_buffer(*args)
     def convexHull(*args): return _mapscript.shapeObj_convexHull(*args)
+    def boundary(*args): return _mapscript.shapeObj_boundary(*args)
+    def getCentroid(*args): return _mapscript.shapeObj_getCentroid(*args)
+    def Union(*args): return _mapscript.shapeObj_Union(*args)
+    def intersection(*args): return _mapscript.shapeObj_intersection(*args)
+    def difference(*args): return _mapscript.shapeObj_difference(*args)
+    def symDifference(*args): return _mapscript.shapeObj_symDifference(*args)
+    def overlaps(*args): return _mapscript.shapeObj_overlaps(*args)
+    def within(*args): return _mapscript.shapeObj_within(*args)
+    def crosses(*args): return _mapscript.shapeObj_crosses(*args)
+    def intersects(*args): return _mapscript.shapeObj_intersects(*args)
+    def touches(*args): return _mapscript.shapeObj_touches(*args)
+    def equals(*args): return _mapscript.shapeObj_equals(*args)
+    def disjoint(*args): return _mapscript.shapeObj_disjoint(*args)
+    def getArea(*args): return _mapscript.shapeObj_getArea(*args)
+    def getLength(*args): return _mapscript.shapeObj_getLength(*args)
     def getValue(*args): return _mapscript.shapeObj_getValue(*args)
     def contains(*args): return _mapscript.shapeObj_contains(*args)
     def distanceToPoint(*args): return _mapscript.shapeObj_distanceToPoint(*args)
     def distanceToShape(*args): return _mapscript.shapeObj_distanceToShape(*args)
-    def intersects(*args): return _mapscript.shapeObj_intersects(*args)
     def setValue(*args): return _mapscript.shapeObj_setValue(*args)
+    def initValues(*args): return _mapscript.shapeObj_initValues(*args)
 
 class shapeObjPtr(shapeObj):
     def __init__(self, this):
@@ -1332,6 +1353,7 @@ MS_GEOSERR = _mapscript.MS_GEOSERR
 MS_RECTERR = _mapscript.MS_RECTERR
 MS_TIMEERR = _mapscript.MS_TIMEERR
 MS_GMLERR = _mapscript.MS_GMLERR
+MS_SOSERR = _mapscript.MS_SOSERR
 MS_NUMERRORCODES = _mapscript.MS_NUMERRORCODES
 MESSAGELENGTH = _mapscript.MESSAGELENGTH
 ROUTINELENGTH = _mapscript.ROUTINELENGTH
@@ -1394,7 +1416,7 @@ class hashTableObjPtr(hashTableObj):
         self.__class__ = hashTableObj
 _mapscript.hashTableObj_swigregister(hashTableObjPtr)
 
-MAX_PARAMS = _mapscript.MAX_PARAMS
+MS_MAX_CGI_PARAMS = _mapscript.MS_MAX_CGI_PARAMS
 MS_GET_REQUEST = _mapscript.MS_GET_REQUEST
 MS_POST_REQUEST = _mapscript.MS_POST_REQUEST
 class OWSRequest(object):
@@ -1413,6 +1435,7 @@ class OWSRequest(object):
         try:
             if self.thisown: destroy(self)
         except: pass
+    def loadParams(*args): return _mapscript.OWSRequest_loadParams(*args)
     def setParameter(*args): return _mapscript.OWSRequest_setParameter(*args)
     def getName(*args): return _mapscript.OWSRequest_getName(*args)
     def getValue(*args): return _mapscript.OWSRequest_getValue(*args)
@@ -1426,3 +1449,17 @@ class OWSRequestPtr(OWSRequest):
 _mapscript.OWSRequest_swigregister(OWSRequestPtr)
 
 
+msConnPoolCloseUnreferenced = _mapscript.msConnPoolCloseUnreferenced
+
+msIO_resetHandlers = _mapscript.msIO_resetHandlers
+
+msIO_installStdoutToBuffer = _mapscript.msIO_installStdoutToBuffer
+
+msIO_installStdinFromBuffer = _mapscript.msIO_installStdinFromBuffer
+
+msIO_stripStdoutBufferContentType = _mapscript.msIO_stripStdoutBufferContentType
+
+msIO_getStdoutBufferString = _mapscript.msIO_getStdoutBufferString
+
+msIO_getStdoutBufferBytes = _mapscript.msIO_getStdoutBufferBytes
+
