@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: pygdioctx.c,v 1.5 2004/11/27 16:40:47 sean Exp $
+ * $Id: pygdioctx.c,v 1.6 2005/02/18 20:10:01 sean Exp $
  *
  * Project:  MapServer
  * Purpose:  Interface Python file-like objects with GD through IOCtx
@@ -126,7 +126,7 @@ void free_PyFileIfaceObj_IOCtx(struct PyFileIfaceObj_gdIOCtx *pctx)
     if (!pctx)
         return;
     assert(pctx->ctx.gd_free != NULL);
-    pctx->ctx.gd_free((gdIOCtxPtr)pctx);  // free its internal resources
+    pctx->ctx.gd_free((gdIOCtxPtr)pctx);
     free(pctx);
 }
 
