@@ -27,17 +27,26 @@
  ******************************************************************************
  *
  * $Log: mapsde.c,v $
- * Revision 1.101.2.4  2006/03/30 16:16:37  hobu
- * make sure to return a value in msSDELayerClose
+ * Revision 1.109  2006/08/11 16:58:02  dan
+ * Added ability to encrypt tokens (passwords, etc.) in database connection
+ * strings (MS-RFC-18, bug 1792)
  *
- * Revision 1.101.2.3  2006/03/10 18:59:36  hobu
+ * Revision 1.108  2006/08/01 18:23:12  sdlime
+ * Fixed a problem in the SDE code that prohibited setting SDE to only process the attribute portion of a query.
+ *
+ * Revision 1.107  2006/03/12 05:54:35  hobu
+ * make sure to return a value in the failure case
+ * for msSDELayerClose
+ *
+ * Revision 1.106  2006/03/10 17:11:08  hobu
+ * add a PROCESSING directive to SDE to allow the user to specify
+ * the query order (attributes or spatial first).
+ *
+ * Revision 1.105  2006/03/10 17:05:56  hobu
  * Fix 1699, where point queries in msSDEWhichShape were
  * causing the SDE C API to complain about creating an invalid
  * rectangle
  *
- * Revision 1.101.2.2  2006/02/24 23:09:39  hobu
- * backport of fixes for SDE (bug 1682)
- *
  * Revision 1.104  2006/02/24 22:41:50  sdlime
  * Added missing malloc in mapsde in msSDELayerGetItems().
  *
@@ -863,6 +872,8 @@ int msSDELayerOpen(layerObj *layer) {
   
   /* If we weren't returned a connection and stream, initialize new ones */
   if (!poolinfo) {
+    char *conn_decrypted;
+
     if (layer->debug) 
       msDebug("msSDELayerOpen(): "
               "Layer %s opened from scratch.\n", layer->name);
@@ -873,14 +884,23 @@ int msSDELayerOpen(layerObj *layer) {
       return MS_FAILURE;
     } 		
 
-	  /* Split the connection parameters and make sure we have enough of them */
-    params = split(layer->connection, ',', &numparams);
+    /* Decrypt any encrypted token in the connection string */
+    conn_decrypted = msDecryptStringTokens(layer->map, layer->connection);
+    if (conn_decrypted == NULL) {
+        return(MS_FAILURE);  /* An error should already have been produced */
+    }
+    /* Split the connection parameters and make sure we have enough of them */
+    params = split(conn_decrypted, ',', &numparams);
     if(!params) {
       msSetError( MS_MEMERR, 
                   "Error spliting SDE connection information.", 
                   "msSDELayerOpen()");
+      msFree(conn_decrypted);
       return(MS_FAILURE);
     }
+    msFree(conn_decrypted);
+    conn_decrypted = NULL;
+
     if(numparams < 5) {
       msSetError( MS_SDEERR, 
                   "Not enough SDE connection parameters specified.", 
@@ -1130,7 +1150,7 @@ int  msSDELayerClose(layerObj *layer) {
   msSetError( MS_MISCERR, 
               "SDE support is not available.", 
               "msSDELayerClose()");
-  return MS_FALSE;
+  return(MS_FALSE);
 #endif
 }
 
@@ -1183,6 +1203,8 @@ int msSDELayerWhichShapes(layerObj *layer, rectObj rect) {
   SE_SHAPE shape=0;
   SE_FILTER constraint;
   SE_QUERYINFO query_info;
+  char* proc_value=NULL;
+  int query_order=SE_SPATIAL_FIRST;
 
   msSDELayerInfo *sde=NULL;
 
@@ -1221,7 +1243,7 @@ int msSDELayerWhichShapes(layerObj *layer, rectObj rect) {
   envelope.miny = MS_MAX(rect.miny, envelope.miny);
   envelope.maxx = MS_MIN(rect.maxx, envelope.maxx);
   envelope.maxy = MS_MIN(rect.maxy, envelope.maxy);
-
+  
   if( envelope.minx == envelope.maxx && envelope.miny == envelope.maxy){
         /* fudge a rectangle so we have a valid one for generate_rectangle */
         /* FIXME: use the real shape for the query and set the filter_type 
@@ -1231,7 +1253,7 @@ int msSDELayerWhichShapes(layerObj *layer, rectObj rect) {
         envelope.miny = envelope.miny - 0.001;
         envelope.maxy = envelope.maxy + 0.001;
     }
-    
+
   status = SE_shape_generate_rectangle(&envelope, shape);
   if(status != SE_SUCCESS) {
     sde_error(status, 
@@ -1334,12 +1356,17 @@ int msSDELayerWhichShapes(layerObj *layer, rectObj rect) {
               "SE_stream_query_with_info()");
     return(MS_FAILURE);
   }
+  
+  proc_value = msLayerGetProcessingKey(layer,"QUERYORDER");
+  if(proc_value && strcasecmp(proc_value, "ATTRIBUTE") == 0)
+    query_order = SE_ATTRIBUTE_FIRST;
 
   status = SE_stream_set_spatial_constraints( sde->stream, 
-                                              SE_SPATIAL_FIRST, 
+                                              query_order, 
                                               FALSE, 
                                               1, 
                                               &constraint);
+
   if(status != SE_SUCCESS) {
     sde_error(status, 
               "msSDELayerWhichShapes()", 
