@@ -5,7 +5,7 @@
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,12 @@
  ******************************************************************************
  *
  * $Log: mapsearch.c,v $
+ * Revision 1.17  2005/06/14 16:03:34  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.16  2005/02/18 03:06:47  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
  * Revision 1.15  2004/10/21 04:30:54  frank
  * Added standardized headers.  Added MS_CVSID().
  *
@@ -34,7 +40,7 @@
 
 #include "map.h"
 
-MS_CVSID("$Id: mapsearch.c,v 1.15 2004/10/21 04:30:54 frank Exp $")
+MS_CVSID("$Id: mapsearch.c,v 1.17 2005/06/14 16:03:34 dan Exp $")
 
 #define LASTVERT(v,n)  ((v) == 0 ? n-2 : v-1)
 #define NEXTVERT(v,n)  ((v) == n-2 ? 0 : v+1)
@@ -212,14 +218,14 @@ int msIntersectPolylines(shapeObj *line1, shapeObj *line2) {
 int msIntersectPolylinePolygon(shapeObj *line, shapeObj *poly) {
   int c1,v1,c2,v2;
 
-  // STEP 1: polygon might competely contain the polyline or one of it's parts (only need to check one point from each part)
+  /* STEP 1: polygon might competely contain the polyline or one of it's parts (only need to check one point from each part) */
   for(c1=0; c1<line->numlines; c1++) {
-    if(msIntersectPointPolygon(&(line->line[c1].point[0]), poly) == MS_TRUE) // this considers holes and multiple parts
+    if(msIntersectPointPolygon(&(line->line[c1].point[0]), poly) == MS_TRUE) /* this considers holes and multiple parts */
       return(MS_TRUE);
   }
 
 
-  // STEP 2: look for intersecting line segments
+  /* STEP 2: look for intersecting line segments */
   for(c1=0; c1<line->numlines; c1++)
     for(v1=1; v1<line->line[c1].numpoints; v1++)
       for(c2=0; c2<poly->numlines; c2++)
@@ -235,13 +241,13 @@ int msIntersectPolygons(shapeObj *p1, shapeObj *p2) {
 
   /* STEP 1: polygon 1 completely contains 2 (only need to check one point from each part) */
   for(c2=0; c2<p2->numlines; c2++) {
-    if(msIntersectPointPolygon(&(p2->line[c2].point[0]), p1) == MS_TRUE) // this considers holes and multiple parts
+    if(msIntersectPointPolygon(&(p2->line[c2].point[0]), p1) == MS_TRUE) /* this considers holes and multiple parts */
       return(MS_TRUE);
   }
 
   /* STEP 2: polygon 2 completely contains 1 (only need to check one point from each part) */
   for(c1=0; c1<p1->numlines; c1++) {
-    if(msIntersectPointPolygon(&(p1->line[c1].point[0]), p2) == MS_TRUE) // this considers holes and multiple parts
+    if(msIntersectPointPolygon(&(p1->line[c1].point[0]), p2) == MS_TRUE) /* this considers holes and multiple parts */
       return(MS_TRUE);
   }
 
@@ -284,7 +290,7 @@ double msDistancePointToSegment(pointObj *p, pointObj *a, pointObj *b)
 
   l = msDistancePointToPoint(a,b);
 
-  if(l == 0.0) // a = b
+  if(l == 0.0) /* a = b */
     return( msDistancePointToPoint(a,p));
 
   r = ((a->y - p->y)*(a->y - b->y) - (a->x - p->x)*(b->x - a->x))/(l*l);
@@ -300,21 +306,21 @@ double msDistancePointToSegment(pointObj *p, pointObj *a, pointObj *b)
 }
 
 #define SMALL_NUMBER 0.00000001
-#define dot(u,v) ((u).x *(v).x + (u).y *(v).y) // vector dot product
+#define dot(u,v) ((u).x *(v).x + (u).y *(v).y) /* vector dot product */
 #define norm(v) sqrt(dot(v,v))
 
 #define slope(a,b) (((a)->y - (b)->y)/((a)->x - (b)->x))
 
-// Segment to segment distance code is a modified version of that found at: 
-//
-//   http://www.geometryalgorithms.com/Archive/algorithm_0106/algorithm_0106.htm
-//
-// Copyright 2001, softSurfer (www.softsurfer.com)
-// This code may be freely used and modified for any purpose
-// providing that this copyright notice is included with it.
-// SoftSurfer makes no warranty for this code, and cannot be held
-// liable for any real or imagined damage resulting from its use.
-// Users of this code must verify correctness for their application.
+/* Segment to segment distance code is a modified version of that found at:  */
+/*  */
+/* http://www.geometryalgorithms.com/Archive/algorithm_0106/algorithm_0106.htm */
+/*  */
+/* Copyright 2001, softSurfer (www.softsurfer.com) */
+/* This code may be freely used and modified for any purpose */
+/* providing that this copyright notice is included with it. */
+/* SoftSurfer makes no warranty for this code, and cannot be held */
+/* liable for any real or imagined damage resulting from its use. */
+/* Users of this code must verify correctness for their application. */
 
 double msDistanceSegmentToSegment(pointObj *pa, pointObj *pb, pointObj *pc, pointObj *pd) 
 {
@@ -322,22 +328,22 @@ double msDistanceSegmentToSegment(pointObj *pa, pointObj *pb, pointObj *pc, poin
   vectorObj u, v, w;
   double a, b, c, d, e; 
   double D;
-  double sc, sN, sD; // N=numerator, D=demoninator
+  double sc, sN, sD; /* N=numerator, D=demoninator */
   double tc, tN, tD;
 
-  // check for strictly parallel segments first
-  // if(((pa->x == pb->x) && (pc->x == pd->x)) || (slope(pa,pb) == slope(pc,pd))) { // vertical (infinite slope) || otherwise parallel
-  //   D = msDistancePointToSegment(pa, pc, pd);
-  //   D = MS_MIN(D, msDistancePointToSegment(pb, pc, pd));
-  //   D = MS_MIN(D, msDistancePointToSegment(pc, pa, pb));
-  //   return(MS_MIN(D, msDistancePointToSegment(pd, pa, pb)));
-  // }
+  /* check for strictly parallel segments first */
+  /* if(((pa->x == pb->x) && (pc->x == pd->x)) || (slope(pa,pb) == slope(pc,pd))) { // vertical (infinite slope) || otherwise parallel */
+  /* D = msDistancePointToSegment(pa, pc, pd); */
+  /* D = MS_MIN(D, msDistancePointToSegment(pb, pc, pd)); */
+  /* D = MS_MIN(D, msDistancePointToSegment(pc, pa, pb)); */
+  /* return(MS_MIN(D, msDistancePointToSegment(pd, pa, pb))); */
+  /* } */
 
-  u.x = pb->x - pa->x; // u = pb - pa
+  u.x = pb->x - pa->x; /* u = pb - pa */
   u.y = pb->y - pa->y;
-  v.x = pd->x - pc->x; // v = pd - pc 
+  v.x = pd->x - pc->x; /* v = pd - pc  */
   v.y = pd->y - pc->y; 
-  w.x = pa->x - pc->x; // w = pa - pc
+  w.x = pa->x - pc->x; /* w = pa - pc */
   w.y = pa->y - pc->y; 
 
   a = dot(u,u);
@@ -350,13 +356,13 @@ double msDistanceSegmentToSegment(pointObj *pa, pointObj *pb, pointObj *pc, poin
   sc = sN = sD = D;
   tc = tN = tD = D;
   
-  // compute the line parameters of the two closest points
-  if(D < SMALL_NUMBER) { // lines are parallel or almost parallel
+  /* compute the line parameters of the two closest points */
+  if(D < SMALL_NUMBER) { /* lines are parallel or almost parallel */
     sN = 0.0;
     sD = 1.0;
     tN = e;
     tD = c;
-  } else { // get the closest points on the infinite lines
+  } else { /* get the closest points on the infinite lines */
     sN = b*e - c*d;
     tN = a*e - b*d;
     if(sN < 0) {
@@ -392,7 +398,7 @@ double msDistanceSegmentToSegment(pointObj *pa, pointObj *pb, pointObj *pc, poin
     }
   }
 
-  // finally do the division to get sc and tc
+  /* finally do the division to get sc and tc */
   sc = sN/sD;
   tc = tN/tD;
 
@@ -426,8 +432,8 @@ double msDistancePointToShape(pointObj *point, shapeObj *shape)
     break;
   case(MS_SHAPE_POLYGON):
     if(msIntersectPointPolygon(point, shape))
-      minDist = 0; // point is IN the shape
-    else { // treat shape just like a line
+      minDist = 0; /* point is IN the shape */
+    else { /* treat shape just like a line */
       for(j=0;j<shape->numlines;j++) {
         for(i=1; i<shape->line[j].numpoints; i++) {
           dist = msDistancePointToSegment(point, &(shape->line[j].point[i-1]), &(shape->line[j].point[i]));
@@ -449,7 +455,7 @@ double msDistanceShapeToShape(shapeObj *shape1, shapeObj *shape2)
   double dist, minDist=-1;
 
   switch(shape1->type) {
-  case(MS_SHAPE_POINT): // shape1
+  case(MS_SHAPE_POINT): /* shape1 */
     for(i=0;i<shape1->numlines;i++) {
       for(j=0; j<shape1->line[i].numpoints; j++) {
         dist = msDistancePointToShape(&(shape1->line[i].point[j]), shape2);
@@ -458,7 +464,7 @@ double msDistanceShapeToShape(shapeObj *shape1, shapeObj *shape2)
       }
     }
     break;
-  case(MS_SHAPE_LINE): // shape1
+  case(MS_SHAPE_LINE): /* shape1 */
     switch(shape2->type) {
     case(MS_SHAPE_POINT):
       for(i=0;i<shape2->numlines;i++) {
@@ -474,11 +480,11 @@ double msDistanceShapeToShape(shapeObj *shape1, shapeObj *shape2)
         for(j=1; j<shape1->line[i].numpoints; j++) {
           for(k=0;k<shape2->numlines;k++) {
             for(l=1; l<shape2->line[k].numpoints; l++) {
-              // check intersection (i.e. dist=0)
+              /* check intersection (i.e. dist=0) */
         if(msIntersectSegments(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l])) == MS_TRUE) 
                 return(0);
 
-        // no intersection, compute distance
+        /* no intersection, compute distance */
         dist = msDistanceSegmentToSegment(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l]));
         if((dist < minDist) || (minDist < 0)) 
     minDist = dist;
@@ -488,22 +494,22 @@ double msDistanceShapeToShape(shapeObj *shape1, shapeObj *shape2)
       }
       break;    
     case(MS_SHAPE_POLYGON):
-      // shape2 (the polygon) could contain shape1 or one of it's parts      
+      /* shape2 (the polygon) could contain shape1 or one of it's parts       */
       for(i=0; i<shape1->numlines; i++) {
-        if(msIntersectPointPolygon(&(shape1->line[0].point[0]), shape2) == MS_TRUE) // this considers holes and multiple parts
+        if(msIntersectPointPolygon(&(shape1->line[0].point[0]), shape2) == MS_TRUE) /* this considers holes and multiple parts */
           return(0);
       }
       
-      // check segment intersection and, if necessary, distance between segments
+      /* check segment intersection and, if necessary, distance between segments */
       for(i=0;i<shape1->numlines;i++) {
         for(j=1; j<shape1->line[i].numpoints; j++) {
           for(k=0;k<shape2->numlines;k++) {
             for(l=1; l<shape2->line[k].numpoints; l++) {
-        // check intersection (i.e. dist=0)
+        /* check intersection (i.e. dist=0) */
         if(msIntersectSegments(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l])) == MS_TRUE) 
                 return(0);
 
-        // no intersection, compute distance
+        /* no intersection, compute distance */
         dist = msDistanceSegmentToSegment(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l]));
         if((dist < minDist) || (minDist < 0)) 
     minDist = dist;
@@ -514,7 +520,7 @@ double msDistanceShapeToShape(shapeObj *shape1, shapeObj *shape2)
       break;
     }
     break;
-  case(MS_SHAPE_POLYGON): // shape1
+  case(MS_SHAPE_POLYGON): /* shape1 */
     switch(shape2->type) {
     case(MS_SHAPE_POINT):
       for(i=0;i<shape2->numlines;i++) {
@@ -526,22 +532,22 @@ double msDistanceShapeToShape(shapeObj *shape1, shapeObj *shape2)
       }
       break;
     case(MS_SHAPE_LINE):
-      // shape1 (the polygon) could contain shape2 or one of it's parts      
+      /* shape1 (the polygon) could contain shape2 or one of it's parts       */
       for(i=0; i<shape2->numlines; i++) {
-        if(msIntersectPointPolygon(&(shape2->line[i].point[0]), shape1) == MS_TRUE) // this considers holes and multiple parts
+        if(msIntersectPointPolygon(&(shape2->line[i].point[0]), shape1) == MS_TRUE) /* this considers holes and multiple parts */
           return(0);
       }
       
-      // check segment intersection and, if necessary, distance between segments
+      /* check segment intersection and, if necessary, distance between segments */
       for(i=0;i<shape1->numlines;i++) {      
         for(j=1; j<shape1->line[i].numpoints; j++) {
           for(k=0;k<shape2->numlines;k++) {
             for(l=1; l<shape2->line[k].numpoints; l++) {
-        // check intersection (i.e. dist=0)
+        /* check intersection (i.e. dist=0) */
         if(msIntersectSegments(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l])) == MS_TRUE) 
                 return(0);
 
-        // no intersection, compute distance
+        /* no intersection, compute distance */
         dist = msDistanceSegmentToSegment(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l]));
         if((dist < minDist) || (minDist < 0)) 
     minDist = dist;
@@ -551,28 +557,28 @@ double msDistanceShapeToShape(shapeObj *shape1, shapeObj *shape2)
       }
       break; 
     case(MS_SHAPE_POLYGON): 
-      // shape1 completely contains shape2 (only need to check one point from each part)
+      /* shape1 completely contains shape2 (only need to check one point from each part) */
       for(i=0; i<shape2->numlines; i++) {
-        if(msIntersectPointPolygon(&(shape2->line[i].point[0]), shape1) == MS_TRUE) // this considers holes and multiple parts
+        if(msIntersectPointPolygon(&(shape2->line[i].point[0]), shape1) == MS_TRUE) /* this considers holes and multiple parts */
           return(0);
       }
 
-      // shape2 completely contains shape1 (only need to check one point from each part)
+      /* shape2 completely contains shape1 (only need to check one point from each part) */
       for(i=0; i<shape1->numlines; i++) {
-        if(msIntersectPointPolygon(&(shape1->line[i].point[0]), shape2) == MS_TRUE) // this considers holes and multiple parts
+        if(msIntersectPointPolygon(&(shape1->line[i].point[0]), shape2) == MS_TRUE) /* this considers holes and multiple parts */
           return(0);
       }
 
-      // check segment intersection and, if necessary, distance between segments
+      /* check segment intersection and, if necessary, distance between segments */
       for(i=0;i<shape1->numlines;i++) {        
         for(j=1; j<shape1->line[i].numpoints; j++) {
           for(k=0;k<shape2->numlines;k++) {
             for(l=1; l<shape2->line[k].numpoints; l++) {
-        // check intersection (i.e. dist=0)
+        /* check intersection (i.e. dist=0) */
         if(msIntersectSegments(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l])) == MS_TRUE) 
                 return(0);        
 
-        // no intersection, compute distance
+        /* no intersection, compute distance */
         dist = msDistanceSegmentToSegment(&(shape1->line[i].point[j-1]), &(shape1->line[i].point[j]), &(shape2->line[k].point[l-1]), &(shape2->line[k].point[l]));
         if((dist < minDist) || (minDist < 0)) 
     minDist = dist;
