@@ -32,6 +32,18 @@
  ******************************************************************************
  *
  * $Log: mapshape.c,v $
+ * Revision 1.73  2006/07/12 06:17:17  sdlime
+ * Added debug statements to output information when tiles tiles are not found (mimicing mapraster.c). Useful when debuging tileindex problems.
+ *
+ * Revision 1.72  2006/06/20 17:26:40  dan
+ * Fixed 3 more instances of the same shapefile leak in tiled layers (bug 1802)
+ *
+ * Revision 1.71  2006/06/14 12:42:41  dan
+ * Fixed leak of shapefile handles (shp/shx/dbf) on tiled layers (bug 1802)
+ *
+ * Revision 1.70  2006/05/15 19:09:10  frank
+ * Support treating POLYGONZ as MS_SHAPE_POLYGON.  (bug 1784)
+ *
  * Revision 1.69  2005/10/28 01:09:42  jani
  * MS RFC 3: Layer vtable architecture (bug 1477)
  *
@@ -62,7 +74,7 @@
 #include <assert.h>
 #include "map.h"
 
-MS_CVSID("$Id: mapshape.c,v 1.69 2005/10/28 01:09:42 jani Exp $")
+MS_CVSID("$Id: mapshape.c,v 1.73 2006/07/12 06:17:17 sdlime Exp $")
 
 
 #define ByteCopy( a, b, c )     memcpy( b, a, c )
@@ -1196,7 +1208,9 @@ void msSHPReadShape( SHPHandle psSHP, int hEntity, shapeObj *shape )
 	}
       }
 
-      if(psSHP->nShapeType == SHP_POLYGON || psSHP->nShapeType == SHP_POLYGONM)
+      if(psSHP->nShapeType == SHP_POLYGON 
+         || psSHP->nShapeType == SHP_POLYGONZ
+         || psSHP->nShapeType == SHP_POLYGONM)
         shape->type = MS_SHAPE_POLYGON;
       else
         shape->type = MS_SHAPE_LINE;
@@ -1626,13 +1640,19 @@ int msTiledSHPOpenFile(layerObj *layer)
       
     /* open the shapefile */
 #ifndef IGNORE_MISSING_DATA
-    if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) 
-      if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1)
+    if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) {
+      if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1) {
+        if( layer->debug || layer->map->debug ) msDebug( "Unable to open file %s for layer %s ... fatal error.\n", filename, layer->name );
         return(MS_FAILURE);
+      }
+    }
 #else
-    if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) 
-      if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1)
-	    continue; /* check again */
+    if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) {
+      if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1) {
+	if( layer->debug || layer->map->debug ) msDebug( "Unable to open file %s for layer %s ... ignoring this missing data.\n", filename, layer->name );
+	continue; /* check again */
+      }
+    }
 #endif
 
     return(MS_SUCCESS); /* found a template, ok to proceed */
@@ -1680,18 +1700,31 @@ int msTiledSHPWhichShapes(layerObj *layer, rectObj rect)
 
       /* open the shapefile */
 #ifndef IGNORE_MISSING_DATA
-      if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1)
-	    if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1)
-	      return(MS_FAILURE);
+      if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) {
+        if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1) {
+          if( layer->debug || layer->map->debug ) msDebug( "Unable to open file %s for layer %s ... fatal error.\n", filename, layer->name );
+          return(MS_FAILURE);
+        }
+      }
 #else
-      if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1)
-        if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1)
+      if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) {
+        if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1) {
+          if( layer->debug || layer->map->debug ) msDebug( "Unable to open file %s for layer %s ... ignoring this missing data.\n", filename, layer->name );
           continue; /* check again */
+        }
+      }
 #endif
 
       status = msSHPWhichShapes(tSHP->shpfile, rect, layer->debug);
-      if(status == MS_DONE) continue; /* next tile */
-      else if(status != MS_SUCCESS) return(MS_FAILURE);
+      if(status == MS_DONE) {
+          /* Close and continue to next tile */
+          msSHPCloseFile(tSHP->shpfile);
+          continue;
+      }
+      else if(status != MS_SUCCESS) {
+          msSHPCloseFile(tSHP->shpfile);
+          return(MS_FAILURE);
+      }
        
       /* the layer functions keeps track of this */
       /* tSHP->tileshpfile->lastshape = tshape.index; */
@@ -1718,18 +1751,31 @@ int msTiledSHPWhichShapes(layerObj *layer, rectObj rect)
       
         /* open the shapefile */
 #ifndef IGNORE_MISSING_DATA
-        if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) 
-          if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1)
+        if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) {
+          if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1) {
+            if( layer->debug || layer->map->debug ) msDebug( "Unable to open file %s for layer %s ... fatal error.\n", filename, layer->name );
             return(MS_FAILURE);
+          }
+        }
 #else
-        if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) 
-          if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1)
+        if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) {
+          if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1) {
+            if( layer->debug || layer->map->debug ) msDebug( "Unable to open file %s for layer %s ... ignoring this missing data.\n", filename, layer->name );
             continue; /* check again */
+          }
+        }
 #endif
 
         status = msSHPWhichShapes(tSHP->shpfile, rect, layer->debug);
-        if(status == MS_DONE) continue; /* next tile */
-        else if(status != MS_SUCCESS) return(MS_FAILURE);
+        if(status == MS_DONE) {
+            /* Close and continue to next tile */
+            msSHPCloseFile(tSHP->shpfile);
+            continue;
+        }
+        else if(status != MS_SUCCESS) {
+            msSHPCloseFile(tSHP->shpfile);
+            return(MS_FAILURE);
+        }
 
         tSHP->tileshpfile->lastshape = i;
         break;
@@ -1788,18 +1834,31 @@ int msTiledSHPNextShape(layerObj *layer, shapeObj *shape)
 
           /* open the shapefile */
 #ifndef IGNORE_MISSING_DATA
-          if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1)
-	        if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1)
-	          return(MS_FAILURE);
+          if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) {
+            if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1) {
+              if( layer->debug || layer->map->debug ) msDebug( "Unable to open file %s for layer %s ... fatal error.\n", filename, layer->name );
+              return(MS_FAILURE);
+            }
+          }
 #else
-          if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1)
-            if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1)
+          if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) {
+            if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1) {
+              if( layer->debug || layer->map->debug ) msDebug( "Unable to open file %s for layer %s ... ignoring this missing data.\n", filename, layer->name );
               continue; /* check again */
+            }
+          }
 #endif
 
           status = msSHPWhichShapes(tSHP->shpfile, tSHP->tileshpfile->statusbounds, layer->debug);
-          if(status == MS_DONE) continue; /* next tile */
-          else if(status != MS_SUCCESS) return(MS_FAILURE);
+          if(status == MS_DONE) {
+              /* Close and continue to next tile */
+              msSHPCloseFile(tSHP->shpfile);
+              continue;
+          }
+          else if(status != MS_SUCCESS) {
+              msSHPCloseFile(tSHP->shpfile);
+              return(MS_FAILURE);
+          }
 
           /* the layer functions keeps track of this */
           /* tSHP->tileshpfile->lastshape = tshape.index; */
@@ -1815,34 +1874,46 @@ int msTiledSHPNextShape(layerObj *layer, shapeObj *shape)
       } else { /* or reference a shapefile directly   */
 
         for(i=(tSHP->tileshpfile->lastshape + 1); i<tSHP->tileshpfile->numshapes; i++) {
-	      if(msGetBit(tSHP->tileshpfile->status,i)) {
-	        if(!layer->data) /* assume whole filename is in attribute field */
-	          filename = (char*)msDBFReadStringAttribute(tSHP->tileshpfile->hDBF, i, layer->tileitemindex);
-	        else {  
-	          sprintf(tilename,"%s/%s", msDBFReadStringAttribute(tSHP->tileshpfile->hDBF, i, layer->tileitemindex) , layer->data);
-	          filename = tilename;
-	        }
+          if(msGetBit(tSHP->tileshpfile->status,i)) {
+            if(!layer->data) /* assume whole filename is in attribute field */
+              filename = (char*)msDBFReadStringAttribute(tSHP->tileshpfile->hDBF, i, layer->tileitemindex);
+            else {  
+              sprintf(tilename,"%s/%s", msDBFReadStringAttribute(tSHP->tileshpfile->hDBF, i, layer->tileitemindex) , layer->data);
+              filename = tilename;
+	    }
 
-  	        if(strlen(filename) == 0) continue; /* check again */
+            if(strlen(filename) == 0) continue; /* check again */
 
-  	        /* open the shapefile */
+            /* open the shapefile */
 #ifndef IGNORE_MISSING_DATA
-	        if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) 
-              if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1)
-	            return(MS_FAILURE);
+            if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) {
+              if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1) {
+                if( layer->debug || layer->map->debug ) msDebug( "Unable to open file %s for layer %s ... fatal error.\n", filename, layer->name );
+                return(MS_FAILURE);
+              }
+            }
 #else
-            if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) 
-              if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1)
-	            continue; /* check again */
+            if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath3(szPath, layer->map->mappath, layer->map->shapepath, filename)) == -1) {
+              if(msSHPOpenFile(tSHP->shpfile, "rb", msBuildPath(szPath, layer->map->mappath, filename)) == -1) {
+                if( layer->debug || layer->map->debug ) msDebug( "Unable to open file %s for layer %s ... ignoring this missing data.\n", filename, layer->name );
+                continue; /* check again */
+              }
+	    }
 #endif
 
-  	        status = msSHPWhichShapes(tSHP->shpfile, tSHP->tileshpfile->statusbounds, layer->debug);
-	        if(status == MS_DONE) continue; /* next tile */
-	        else if(status != MS_SUCCESS) return(MS_FAILURE);
+            status = msSHPWhichShapes(tSHP->shpfile, tSHP->tileshpfile->statusbounds, layer->debug);
+            if(status == MS_DONE) {
+              /* Close and continue to next tile */
+              msSHPCloseFile(tSHP->shpfile);
+              continue;
+            } else if(status != MS_SUCCESS) {
+              msSHPCloseFile(tSHP->shpfile);
+              return(MS_FAILURE);
+            }
 	  
-	        tSHP->tileshpfile->lastshape = i;
-	        break;
-	      }
+            tSHP->tileshpfile->lastshape = i;
+            break;
+	  }
         } /* end for loop */
       
         if(i == tSHP->tileshpfile->numshapes) return(MS_DONE); /* no more tiles */
