@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapsvg.c,v 1.21.2.1 2006/01/16 20:41:23 sdlime Exp $
+ * $Id: mapsvg.c,v 1.22 2006/01/16 20:21:18 sdlime Exp $
  *
  * Name:     mapsvg.c
  * Project:  MapServer
@@ -31,7 +31,7 @@
  ******************************************************************************
  *
  * $Log: mapsvg.c,v $
- * Revision 1.21.2.1  2006/01/16 20:41:23  sdlime
+ * Revision 1.22  2006/01/16 20:21:18  sdlime
  * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
  *
  * Revision 1.21  2005/10/26 17:53:57  frank
@@ -111,7 +111,7 @@
 #endif
 #include "map.h"
 
-MS_CVSID("$Id: mapsvg.c,v 1.21.2.1 2006/01/16 20:41:23 sdlime Exp $")
+MS_CVSID("$Id: mapsvg.c,v 1.22 2006/01/16 20:21:18 sdlime Exp $")
 
 
 static int msIO_fprintfgz(FILE *fp, int bCompressed, const char *format, ... )
