@@ -27,27 +27,46 @@
  ******************************************************************************
  *
  * $Log: mapsymbol.c,v $
- * Revision 1.90.2.5  2006/03/30 03:46:30  sdlime
- * Fixed a few more instances where PIXMAP symbols are created but sizex and sizey are not explicitly set. (bug 1725)
+ * Revision 1.100  2006/07/23 03:28:45  sdlime
+ * Fixed error in msAddImageSymbol() where imagepath is not set. (bug 1832)
  *
- * Revision 1.90.2.4  2006/03/21 06:29:29  sdlime
+ * Revision 1.99  2006/04/27 04:05:18  sdlime
+ * Initial support for relative coordinates. (bug 1547)
+ *
+ * Revision 1.98  2006/03/30 03:58:04  sdlime
+ * Fixed what I hope are the last of the routines that create PIXMAP symbols but don't set symbol sizex and sizey. (bug 1725)
+ *
+ * Revision 1.97  2006/03/21 06:27:08  sdlime
+ * Made sure msRotateSymbol sets the new symbol's sizex and sizey members for PIXMAP symbols.
+ *
+ * Revision 1.96  2006/03/16 05:28:44  sdlime
+ * Cleaned up a build warning in mapsymbol.c.
+ *
+ * Revision 1.95  2006/03/16 04:45:24  sdlime
  * Reverted to old means of scaling symbols based solely on height. Fixed possiblity of memory leak with symbol rotation. Made rotation and scaling behavior more consistent across all GD rendering functions (point, line, polygon and circle). (bugs 1684 and 1705)
  *
- * Revision 1.90.2.3  2006/03/02 01:38:43  jani
- * Revert 1.90.2.2; (bug fix #1684).
- * I accidentally committed to 4.8 branch. Sorry about that.
+ * Revision 1.94  2006/03/02 16:43:44  jani
+ * This fixes second part of bug 1684, which happens when image is loaded
+ * by symbol file.
  *
- * Revision 1.90.2.2  2006/03/02 01:34:57  jani
+ * * loadSymbol: initialize sizex, sizey when loading image as symbol.
+ *
+ * Revision 1.93  2006/03/02 01:51:13  jani
  * When we add a pixmap symbol, be sure to initialize sizex,sizey fields of
- * symbol. These fields are used later by mapgd.c:msDrawMarkerSymbolGD
- * to calculate scaling factor, if they are uninitialized (0.0),
- * the scaling factor will be 1.
+ * symbol. These fields are used later (e.g.mapgd.c:msDrawMarkerSymbolGD)
+ * to calculate scaling factor.  If they are uninitialized (0.0),
+ * the scaling factor will be 1. This worked before with the old code because
+ * msDrawMarkerSymbolGD used directly img->sy when it calculated
+ * d (the scaling factor).
  *
  * This fixes bug #1684
  *
  * * msAddImageSymbol: Initialize sizex, sizey
  *
- * Revision 1.90.2.1  2006/01/16 20:01:56  sean
+ * Revision 1.92  2006/02/18 21:04:20  hobu
+ * warning nanny on line 1223, unused variable
+ *
+ * Revision 1.91  2006/01/16 19:46:35  sean
  * close handle when saving symbolset, pointed out by Albert Rovira
  *
  * Revision 1.90  2006/01/09 18:04:19  frank
@@ -118,7 +137,7 @@
 #include "mapcopy.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapsymbol.c,v 1.90.2.5 2006/03/30 03:46:30 sdlime Exp $")
+MS_CVSID("$Id: mapsymbol.c,v 1.100 2006/07/23 03:28:45 sdlime Exp $")
 
 extern int msyylex(void); /* lexer globals */
 extern void msyyrestart(FILE *);
@@ -134,7 +153,7 @@ extern unsigned char JPEGsig[3];
 ** Symbol to string static arrays needed for writing map files.
 ** Must be kept in sync with enumerations and defines found in map.h.
 */
-extern char *msLabelPositions[]; /* Defined in mapfile.c */
+extern char *msPositionsText[]; /* Defined in mapfile.c */
 static char *msCapsJoinsCorners[7]={"NONE", "BEVEL", "BUTT", "MITER", "ROUND", "SQUARE", "TRIANGLE"};
 
 void freeImageCache(struct imageCacheObj *ic)
@@ -340,7 +359,7 @@ int loadSymbol(symbolObj *s, char *symbolpath)
 	return(-1);
       }
       s->sizex = s->img->sx;
-      s->sizey = s->img->sy;            
+      s->sizey = s->img->sy;
       break;
     case(LINECAP):
       if((s->linecap = getSymbol(4,MS_CJC_BUTT, MS_CJC_ROUND, MS_CJC_SQUARE, MS_CJC_TRIANGLE)) == -1)
@@ -452,7 +471,7 @@ void writeSymbol(symbolObj *s, FILE *stream)
     if (s->character != NULL) fprintf(stream, "    CHARACTER \"%s\"\n", s->character);
     fprintf(stream, "    GAP %d\n", s->gap);
     if (s->font != NULL) fprintf(stream, "    FONT \"%s\"\n", s->font);
-    fprintf(stream, "    POSITION %s\n", msLabelPositions[s->position]);
+    fprintf(stream, "    POSITION %s\n", msPositionsText[s->position - MS_UL]);
     break;
   case(MS_SYMBOL_CARTOLINE):
     fprintf(stream, "    TYPE CARTOLINE\n");
@@ -496,6 +515,7 @@ void writeSymbol(symbolObj *s, FILE *stream)
 /*
 ** Little helper function to allow us to build symbol files on-the-fly 
 ** from just a file name.
+**
 ** Returns the symbol index or -1 if it could not be added.
 */
 int msAddImageSymbol(symbolSetObj *symbolset, char *filename) 
@@ -517,21 +537,16 @@ int msAddImageSymbol(symbolSetObj *symbolset, char *filename)
   }
 
   
-  if (symbolset->map)
-  {
-      if((stream = fopen(msBuildPath(szPath, symbolset->map->mappath, filename), "rb")) == NULL)        
-      {
-          msSetError(MS_IOERR, "Error opening image file %s.", "msAddImageSymbol()", szPath);
-          return(-1);
-      }
-  }
-  else
-  {
-      if((stream = fopen(msBuildPath(szPath, NULL, filename), "rb")) == NULL)        
-      {
-          msSetError(MS_IOERR, "Error opening image file %s.", "msAddImageSymbol()", szPath);
-          return(-1);
-      }
+  if(symbolset->map) {
+    if((stream = fopen(msBuildPath(szPath, symbolset->map->mappath, filename), "rb")) == NULL) {
+      msSetError(MS_IOERR, "Error opening image file %s.", "msAddImageSymbol()", szPath);
+      return(-1);
+    }
+  } else {
+    if((stream = fopen(msBuildPath(szPath, NULL, filename), "rb")) == NULL) {
+      msSetError(MS_IOERR, "Error opening image file %s.", "msAddImageSymbol()", szPath);
+      return(-1);
+    }
   }
   i = symbolset->numsymbols;  
 
@@ -539,27 +554,27 @@ int msAddImageSymbol(symbolSetObj *symbolset, char *filename)
 
   fread(bytes,8,1,stream); /* read some bytes to try and identify the file */
   rewind(stream); /* reset the image for the readers */
-  if (memcmp(bytes,"GIF8",4)==0) {
+  if(memcmp(bytes,"GIF8",4)==0) {
 #ifdef USE_GD_GIF
-      gdIOCtx *ctx;
-      ctx = msNewGDFileCtx(stream);
-      symbolset->symbol[i].img = gdImageCreateFromGifCtx(ctx);
-      ctx->gd_free(ctx);
+    gdIOCtx *ctx;
+    ctx = msNewGDFileCtx(stream);
+    symbolset->symbol[i].img = gdImageCreateFromGifCtx(ctx);
+    ctx->gd_free(ctx);
 #else
-      msSetError(MS_MISCERR, "Unable to load GIF symbol.", "msAddImageSymbol()");
-      fclose(stream);
-      return(-1);
+    msSetError(MS_MISCERR, "Unable to load GIF symbol.", "msAddImageSymbol()");
+    fclose(stream);
+    return(-1);
 #endif
   } else if (memcmp(bytes,PNGsig,8)==0) {
 #ifdef USE_GD_PNG
-      gdIOCtx *ctx;
-      ctx = msNewGDFileCtx(stream);
-      symbolset->symbol[i].img = gdImageCreateFromPngCtx(ctx);
-      ctx->gd_free(ctx);
+    gdIOCtx *ctx;
+    ctx = msNewGDFileCtx(stream);
+    symbolset->symbol[i].img = gdImageCreateFromPngCtx(ctx);
+    ctx->gd_free(ctx);
 #else
-      msSetError(MS_MISCERR, "Unable to load PNG symbol.", "msAddImageSymbol()");
-      fclose(stream);
-      return(-1);
+    msSetError(MS_MISCERR, "Unable to load PNG symbol.", "msAddImageSymbol()");
+    fclose(stream);
+    return(-1);
 #endif
   }
 
@@ -571,6 +586,7 @@ int msAddImageSymbol(symbolSetObj *symbolset, char *filename)
   }
 
   symbolset->symbol[i].name = strdup(filename);
+  symbolset->symbol[i].imagepath = strdup(filename);
   symbolset->symbol[i].type = MS_SYMBOL_PIXMAP;
   symbolset->symbol[i].sizex = symbolset->symbol[i].img->sx;
   symbolset->symbol[i].sizey = symbolset->symbol[i].img->sy;
@@ -1146,6 +1162,7 @@ int msSymbolSetImageGD(symbolObj *symbol, imageObj *image)
     gdImageCopy(symbol->img, image->img.gd, 0, 0, 0, 0,
                 image->width, image->height);
 
+    symbol->type = MS_SYMBOL_PIXMAP; /* just in case the symbol wasn't a PIXMAP symbol before */
     symbol->sizex = symbol->img->sx;
     symbol->sizey = symbol->img->sy;
 
@@ -1279,14 +1296,14 @@ symbolObj *msRotateSymbol(symbolObj *symbol, double angle)
 	newSymbol->img = gdImageCreate(width, height);	
       }
 
-      newSymbol->sizex = width;
-      newSymbol->sizey = height;
+      newSymbol->sizex = maxx;
+      newSymbol->sizey = maxy;
 
       gdImageCopyRotated (newSymbol->img, symbol->img, width*0.5, height*0.5, 0, 0, gdImageSX(symbol->img), gdImageSY(symbol->img), angle);
       break;
     }
   default:
-    break; /* should never get here */   
+    break; /* should never get here */
   } /* end symbol type switch */
 
   return newSymbol;
