@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapsymbol.h,v 1.19 2005/06/14 16:03:35 dan Exp $
+ * $Id: mapsymbol.h,v 1.23 2005/12/15 05:42:05 sdlime Exp $
  *
  * Project:  MapServer
  * Purpose:  symbolObj related declarations.
@@ -28,6 +28,19 @@
  ******************************************************************************
  *
  * $Log: mapsymbol.h,v $
+ * Revision 1.23  2005/12/15 05:42:05  sdlime
+ * Fixed a problem in the symbol reader that basically turned TYPE SIMPLE into TYPE TRUETYPE. Fix what to start the symbol type enum at some larger value (e.g. 1000).
+ *
+ * Revision 1.22  2005/11/24 04:35:50  dan
+ * Use dynamic allocation for ellipse symbol's STYLE array, avoiding the
+ * static limitation on the STYLE argument values. (bug 1539)
+ *
+ * Revision 1.21  2005/10/14 05:05:19  sdlime
+ * Added MS_MAXPATTERNSIZE.
+ *
+ * Revision 1.20  2005/09/23 03:47:56  sdlime
+ * Made symbol->imagepath readonly within SWIG-based scripts. Use setImagepath instead. (bug 1472)
+ *
  * Revision 1.19  2005/06/14 16:03:35  dan
  * Updated copyright date to 2005
  *
@@ -44,7 +57,7 @@
 
 #include <gd.h>
 
-enum MS_SYMBOL_TYPE {MS_SYMBOL_SIMPLE, MS_SYMBOL_VECTOR, MS_SYMBOL_ELLIPSE, MS_SYMBOL_PIXMAP, MS_SYMBOL_TRUETYPE, MS_SYMBOL_CARTOLINE, MS_SYMBOL_HATCH};
+enum MS_SYMBOL_TYPE {MS_SYMBOL_SIMPLE=1000, MS_SYMBOL_VECTOR, MS_SYMBOL_ELLIPSE, MS_SYMBOL_PIXMAP, MS_SYMBOL_TRUETYPE, MS_SYMBOL_CARTOLINE, MS_SYMBOL_HATCH};
 
 #define MS_MAXSYMBOLS 64            /* maximum number of symbols in a symbol file */
 #define MS_MAXVECTORPOINTS 100      /* shade, marker and line symbol parameters */
@@ -114,7 +127,15 @@ typedef struct {
 #ifndef SWIG
   gdImagePtr img;
 #endif /* SWIG */
+
+#ifdef SWIG
+  %immutable;
+#endif /* SWIG */
   char *imagepath;
+#ifdef SWIG
+  %mutable;
+#endif /* SWIG */
+
   int transparent;
   int transparentcolor;
 
