@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maptime.h,v 1.11 2005/06/14 16:03:35 dan Exp $
+ * $Id: maptime.h,v 1.12 2006/03/14 03:43:30 assefa Exp $
  *
  * Project:  MapServer
  * Purpose:  Time processing related declarations.
@@ -28,6 +28,10 @@
  ******************************************************************************
  *
  * $Log: maptime.h,v $
+ * Revision 1.12  2006/03/14 03:43:30  assefa
+ * Move msValidateTimeValue to maptime so it can be used by WMS and SOS.
+ * (Bug 1710)
+ *
  * Revision 1.11  2005/06/14 16:03:35  dan
  * Updated copyright date to 2005
  *
@@ -88,4 +92,6 @@ void msSetLimitedPattersToUse(char *patternstring);
 void msUnsetLimitedPatternToUse(void);
 int msTimeGetResolution(const char *timestring);
 
+int msValidateTimeValue(char *timestring, const char *timeextent);
+
 #endif /* MAPTIME_H */
