@@ -27,6 +27,16 @@
  ******************************************************************************
  *
  * $Log: maputil.c,v $
+ * Revision 1.190  2006/08/15 17:24:56  dan
+ * Fixed problem with PHP MapScript's saveWebImage() filename collisions
+ * when mapscript was loaded in php.ini with PHP as an Apache DSO (bug 1322)
+ *
+ * Revision 1.189  2006/07/05 05:54:53  frank
+ * implement per-thread io contexts
+ *
+ * Revision 1.188  2006/06/27 13:35:45  sdlime
+ * Initial version of GEOS C-API code including SWIG bindings for most everything useful.
+ *
  * Revision 1.187  2005/08/27 04:57:25  sdlime
  * Fixed a recursion problem in context evaluation function msEvalContext(). Had to disable the [raster] option though and will try to implement some other way. (bug 1283)
  *
@@ -132,7 +142,7 @@
 #include <process.h>
 #endif
 
-MS_CVSID("$Id: maputil.c,v 1.187 2005/08/27 04:57:25 sdlime Exp $")
+MS_CVSID("$Id: maputil.c,v 1.190 2006/08/15 17:24:56 dan Exp $")
 
 extern int msyyparse(void);
 extern int msyylex(void);
@@ -1098,7 +1108,7 @@ void msForceTmpFileBase( const char *new_base )
  * 
  *    PID + timestamp + sequence# + extension
  * 
- * If msForceTmpFileBase() has been call to control the temporary filename
+ * If msForceTmpFileBase() has been called to control the temporary filename
  * then the filename will be:
  *
  *    TmpBase + sequence# + extension
@@ -1119,13 +1129,15 @@ char *msTmpFile(const char *mappath, const char *tmppath, const char *ext)
     else 
     {
         /* We'll use tmpId and tmpCount to generate unique filenames */
-        sprintf(tmpId, "%ld%d",(long)time(NULL),(int)getpid());
+        sprintf(tmpId, "%lx_%x",(long)time(NULL),(int)getpid());
     }
 
     if (ext == NULL)  ext = "";
     tmpFname = (char*)malloc(strlen(tmpId) + 10  + strlen(ext) + 1);
 
-    sprintf(tmpFname, "%s%d.%s", tmpId, tmpCount++, ext);
+    msAcquireLock( TLOCK_TMPFILE );
+    sprintf(tmpFname, "%s_%x.%s", tmpId, tmpCount++, ext);
+    msReleaseLock( TLOCK_TMPFILE );
 
     fullFname = msBuildPath3(szPath, mappath, tmppath, tmpFname);
     free(tmpFname);
@@ -1291,16 +1303,20 @@ void  msTransformShape(shapeObj *shape, rectObj extent, double cellsize,
 int msSetup()
 {
 #ifdef USE_THREAD
-    msThreadInit();
+   msThreadInit();
 #endif
 
 #ifdef USE_GD_FT
-    if (gdFontCacheSetup() != 0) 
-    {
-        return MS_FAILURE;
-    }
+  if (gdFontCacheSetup() != 0) {
+    return MS_FAILURE;
+   }
 #endif
-    return MS_SUCCESS;
+
+#ifdef USE_GEOS
+  msGEOSSetup();
+#endif
+
+  return MS_SUCCESS;
 }
   
 /* This is intended to be a function to cleanup anything that "hangs around"
@@ -1309,23 +1325,29 @@ extern void lexer_cleanup(void);
 
 void msCleanup()
 {
-    msForceTmpFileBase( NULL );
-    msConnPoolFinalCleanup();
-    lexer_cleanup();
+  msForceTmpFileBase( NULL );
+  msConnPoolFinalCleanup();
+  lexer_cleanup();
+
 #ifdef USE_OGR
-    msOGRCleanup();
+  msOGRCleanup();
 #endif    
 #ifdef USE_GDAL
-    msGDALCleanup();
+  msGDALCleanup();
 #endif    
 #ifdef USE_PROJ
-    pj_deallocate_grids();
-    msSetPROJ_LIB( NULL );
+  pj_deallocate_grids();
+  msSetPROJ_LIB( NULL );
 #endif
 #if defined(USE_WMS_LYR) || defined(USE_WFS_LYR)
-    msHTTPCleanup();
+  msHTTPCleanup();
 #endif
-    msResetErrorList();
+#ifdef USE_GEOS
+  msGEOSCleanup();
+#endif
+  msIO_Cleanup();
+
+  msResetErrorList();
 }
 
 /************************************************************************/
