@@ -27,6 +27,12 @@
  ******************************************************************************
  *
  * $Log: mapwcs.c,v $
+ * Revision 1.71  2005/09/23 04:37:32  frank
+ * added preliminary WCS INTERPOLATION support
+ *
+ * Revision 1.70  2005/06/27 19:58:30  frank
+ * Fixed some small WCS params related memory leaks.
+ *
  * Revision 1.69  2005/06/14 16:03:35  dan
  * Updated copyright date to 2005
  *
@@ -92,7 +98,7 @@
 #include "maperror.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapwcs.c,v 1.69 2005/06/14 16:03:35 dan Exp $")
+MS_CVSID("$Id: mapwcs.c,v 1.71 2005/09/23 04:37:32 frank Exp $")
 
 #ifdef USE_WCS_SVR
 
@@ -124,6 +130,7 @@ typedef struct {
   char *time;
   long width, height, depth;	/* image dimensions */
   double resx, resy, resz;      /* resolution */
+  char *interpolation;          /* interpolationMethod */
   char *format;
   char *exceptions;		/* exception MIME type, (default application=vnd.ogc.se_xml) */
 } wcsParamsObj;
@@ -145,6 +152,9 @@ static int msWCSValidateRangeSetParam(hashTableObj *metadata, char *name, const
   return MS_SUCCESS; /* take their word for it at the moment */
 }
 
+/************************************************************************/
+/*                    msWCSConvertRangeSetToString()                    */
+/************************************************************************/
 static char *msWCSConvertRangeSetToString(const char *value) {
   char **tokens;
   int numtokens;
@@ -177,6 +187,9 @@ static char *msWCSConvertRangeSetToString(const char *value) {
     return strdup(value);
 }
 
+/************************************************************************/
+/*                           msWCSException()                           */
+/************************************************************************/
 static int msWCSException(mapObj *map, const char *version) 
 {
   char *pszEncodedVal = NULL;
@@ -200,6 +213,10 @@ static int msWCSException(mapObj *map, const char *version)
   return MS_FAILURE;
 }
 
+/************************************************************************/
+/*                    msWCSPrintRequestCapability()                     */
+/************************************************************************/
+
 static void msWCSPrintRequestCapability(const char *version, const char *request_tag, const char *script_url)
 {
   msIO_printf("    <%s>\n", request_tag);
@@ -218,6 +235,9 @@ static void msWCSPrintRequestCapability(const char *version, const char *request
   msIO_printf("    </%s>\n", request_tag);
 }
 
+/************************************************************************/
+/*                         msWCSCreateParams()                          */
+/************************************************************************/
 static wcsParamsObj *msWCSCreateParams()
 {
   wcsParamsObj *params;
@@ -230,6 +250,9 @@ static wcsParamsObj *msWCSCreateParams()
   return params;
 }
 
+/************************************************************************/
+/*                          msWCSFreeParams()                           */
+/************************************************************************/
 static void msWCSFreeParams(wcsParamsObj *params)
 {
   if(params) {
@@ -243,9 +266,14 @@ static void msWCSFreeParams(wcsParamsObj *params)
     if(params->format) free(params->format);
     if(params->exceptions) free(params->exceptions);
     if(params->time) free(params->time);
+    if(params->interpolation) free(params->interpolation);
   }
 }
 
+/************************************************************************/
+/*                       msWCSIsLayerSupported()                        */
+/************************************************************************/
+
 static int msWCSIsLayerSupported(layerObj *layer)
 {
     /* only raster layers, with 'DUMP TRUE' explicitly defined, are elligible to be served via WCS, WMS rasters are not ok */
@@ -254,6 +282,10 @@ static int msWCSIsLayerSupported(layerObj *layer)
     return MS_FALSE;
 }
 
+/************************************************************************/
+/*                      msWCSGetRequestParameter()                      */
+/************************************************************************/
+
 static const char *msWCSGetRequestParameter(cgiRequestObj *request, char *name) {
   int i;
 
@@ -270,6 +302,10 @@ static const char *msWCSGetRequestParameter(cgiRequestObj *request, char *name)
   return NULL;
 }
 
+/************************************************************************/
+/*                  msWCSSetDefaultBandsRangeSetInfo()                  */
+/************************************************************************/
+
 static void msWCSSetDefaultBandsRangeSetInfo( wcsParamsObj *params,
                                               coverageMetadataObj *cm,
                                               layerObj *lp ) 
@@ -322,6 +358,10 @@ static void msWCSSetDefaultBandsRangeSetInfo( wcsParamsObj *params,
     free( bandlist );
 }    
 
+/************************************************************************/
+/*                         msWCSParseRequest()                          */
+/************************************************************************/
+
 static int msWCSParseRequest(cgiRequestObj *request, wcsParamsObj *params, mapObj *map)
 {
   int i;
@@ -336,6 +376,8 @@ static int msWCSParseRequest(cgiRequestObj *request, wcsParamsObj *params, mapOb
 	     params->version = strdup(request->ParamValues[i]);
        else if(strcasecmp(request->ParamNames[i], "REQUEST") == 0)
 	     params->request = strdup(request->ParamValues[i]);
+       else if(strcasecmp(request->ParamNames[i], "INTERPOLATION") == 0)
+	     params->interpolation = strdup(request->ParamValues[i]);
        else if(strcasecmp(request->ParamNames[i], "SERVICE") == 0)
 	     params->service = strdup(request->ParamValues[i]);
 	   else if(strcasecmp(request->ParamNames[i], "SECTION") == 0)
@@ -385,6 +427,10 @@ static int msWCSParseRequest(cgiRequestObj *request, wcsParamsObj *params, mapOb
   return MS_SUCCESS;
 }
 
+/************************************************************************/
+/*           msWCSGetCapabilities_Service_ResponsibleParty()            */
+/************************************************************************/
+
 static void msWCSGetCapabilities_Service_ResponsibleParty(mapObj *map)
 {
   int bEnableTelephone=MS_FALSE, bEnableAddress=MS_FALSE, bEnableOnlineResource=MS_FALSE;
@@ -481,6 +527,10 @@ static void msWCSGetCapabilities_Service_ResponsibleParty(mapObj *map)
   return;
 }
 
+/************************************************************************/
+/*                    msWCSGetCapabilities_Service()                    */
+/************************************************************************/
+
 static int msWCSGetCapabilities_Service(mapObj *map, wcsParamsObj *params)
 {
   /* start the Service section, only need the full start tag if this is the only section requested */
@@ -522,6 +572,10 @@ static int msWCSGetCapabilities_Service(mapObj *map, wcsParamsObj *params)
   return MS_SUCCESS;
 }
 
+/************************************************************************/
+/*                  msWCSGetCapabilities_Capability()                   */
+/************************************************************************/
+
 static int msWCSGetCapabilities_Capability(mapObj *map, wcsParamsObj *params, cgiRequestObj *req)
 {
   char *script_url=NULL, *script_url_encoded;
@@ -567,6 +621,10 @@ static int msWCSGetCapabilities_Capability(mapObj *map, wcsParamsObj *params, cg
   return MS_SUCCESS;
 }
 
+/************************************************************************/
+/*             msWCSGetCapabilities_CoverageOfferingBrief()             */
+/************************************************************************/
+
 static int msWCSGetCapabilities_CoverageOfferingBrief(layerObj *layer, wcsParamsObj *params) 
 {
   coverageMetadataObj cm;
@@ -608,6 +666,10 @@ static int msWCSGetCapabilities_CoverageOfferingBrief(layerObj *layer, wcsParams
   return MS_SUCCESS;
 }
 
+/************************************************************************/
+/*                msWCSGetCapabilities_ContentMetadata()                */
+/************************************************************************/
+
 static int msWCSGetCapabilities_ContentMetadata(mapObj *map, wcsParamsObj *params)
 {
   int i;
@@ -635,6 +697,10 @@ static int msWCSGetCapabilities_ContentMetadata(mapObj *map, wcsParamsObj *param
   return MS_SUCCESS;
 }
 
+/************************************************************************/
+/*                        msWCSGetCapabilities()                        */
+/************************************************************************/
+
 static int msWCSGetCapabilities(mapObj *map, wcsParamsObj *params, cgiRequestObj *req)
 {
   /* msIO_printf("Content-type: application/vnd.ogc.se_xml%c%c",10,10); */
@@ -673,6 +739,10 @@ static int msWCSGetCapabilities(mapObj *map, wcsParamsObj *params, cgiRequestObj
   return MS_SUCCESS;
 }
 
+/************************************************************************/
+/*               msWCSDescribeCoverage_AxisDescription()                */
+/************************************************************************/
+
 static int msWCSDescribeCoverage_AxisDescription(layerObj *layer, char *name)
 {
   const char *value;
@@ -740,6 +810,10 @@ static int msWCSDescribeCoverage_AxisDescription(layerObj *layer, char *name)
   return MS_SUCCESS;
 }
 
+/************************************************************************/
+/*               msWCSDescribeCoverage_CoverageOffering()               */
+/************************************************************************/
+
 static int msWCSDescribeCoverage_CoverageOffering(layerObj *layer, wcsParamsObj *params) 
 {
   const char *value; 
@@ -893,7 +967,12 @@ static int msWCSDescribeCoverage_CoverageOffering(layerObj *layer, wcsParamsObj
   msOWSPrintEncodeMetadata(stdout, &(layer->metadata), "COM", "formats", OWS_NOERR, "      <formats>%s</formats>\n", NULL);
   msIO_printf("    </supportedFormats>\n");
   
-  /* TODO: add SupportedInterpolations (optional) */
+  msIO_printf("    <supportedInterpolations default=\"nearest neighbour\">\n");
+  msIO_printf("      <interpolationMethod>nearest neighbor</interpolationMethod>\n" );
+  msIO_printf("      <interpolationMethod>bilinear</interpolationMethod>\n" );
+/*  msIO_printf("      <interpolationMethod>bicubic</interpolationMethod>\n" ); */
+  msIO_printf("    </supportedInterpolations>\n");
+  
 
   /* done */
   msIO_printf("  </CoverageOffering>\n");
@@ -901,6 +980,10 @@ static int msWCSDescribeCoverage_CoverageOffering(layerObj *layer, wcsParamsObj
   return MS_SUCCESS;
 }
 
+/************************************************************************/
+/*                       msWCSDescribeCoverage()                        */
+/************************************************************************/
+
 static int msWCSDescribeCoverage(mapObj *map, wcsParamsObj *params)
 {
   int i,j;
@@ -938,7 +1021,12 @@ static int msWCSDescribeCoverage(mapObj *map, wcsParamsObj *params)
   return MS_SUCCESS;
 }
 
-static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request, wcsParamsObj *params)
+/************************************************************************/
+/*                          msWCSGetCoverage()                          */
+/************************************************************************/
+
+static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request, 
+                            wcsParamsObj *params)
 {
   imageObj   *image;
   layerObj   *lp;
@@ -961,7 +1049,7 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request, wcsParamsObj *p
                 "msWCSGetCoverage()");
     return msWCSException(map, params->version);
   }
-
+  
   /* find the layer we are working with.  */
   lp = NULL;
   for(i=0; i<map->numlayers; i++) {
@@ -1149,6 +1237,25 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request, wcsParamsObj *p
       if( map->debug )
           msDebug( "RESX and RESY don't match.  Using geotransform/resample.");
   }
+
+  /* Do we have a specified interpolation method */
+  if( params->interpolation != NULL )
+  {
+      if( strncasecmp(params->interpolation,"NEAREST",7) == 0 )
+          msLayerSetProcessingKey(lp, "RESAMPLE", "NEAREST");
+      else if( strcasecmp(params->interpolation,"BILINEAR") == 0 )
+          msLayerSetProcessingKey(lp, "RESAMPLE", "BILINEAR");
+      else if( strcasecmp(params->interpolation,"AVERAGE") == 0 )
+          msLayerSetProcessingKey(lp, "RESAMPLE", "AVERAGE");
+      else
+      {
+          msSetError( MS_WCSERR, 
+                      "INTERPOLATION=%s specifies an unsupported interpolation method.",
+                      "msWCSGetCoverage()",
+                      params->interpolation );
+          return msWCSException(map, params->version);
+      }
+  }
    
   /* apply region and size to map object.  */
   map->width = params->width;
@@ -1226,7 +1333,12 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request, wcsParamsObj *p
 }
 #endif /* def USE_WCS_SVR */
 
-/* Entry point for WCS requests */
+/************************************************************************/
+/*                           msWCSDispatch()                            */
+/*                                                                      */
+/*      Entry point for WCS requests                                    */
+/************************************************************************/
+
 int msWCSDispatch(mapObj *map, cgiRequestObj *request)
 {
 #ifdef USE_WCS_SVR
@@ -1235,15 +1347,27 @@ int msWCSDispatch(mapObj *map, cgiRequestObj *request)
   /* populate the service parameters */
   params = msWCSCreateParams();
   if( msWCSParseRequest(request, params, map) == MS_FAILURE )
+  {
+      msWCSFreeParams(params); /* clean up */
+      free(params);
       return MS_FAILURE;
+  }
 
   /* If SERVICE is specified then it MUST be "WCS" */
   if(params->service && strcasecmp(params->service, "WCS") != 0)
-    return MS_DONE;
+  {
+      msWCSFreeParams(params); /* clean up */
+      free(params);
+      return MS_DONE;
+  }
 
   /* If SERVICE and REQUEST not included then not a WCS request */
   if(!params->service && !params->request)
-    return MS_DONE;
+  {
+      msWCSFreeParams(params); /* clean up */
+      free(params);
+      return MS_DONE;
+  }
 
   /*
   ** ok, it's a WCS request, check what we can at a global level and then dispatch to the various request handlers
