@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapwfs.c,v 1.73.2.1 2006/03/05 20:54:52 assefa Exp $
+ * $Id: mapwfs.c,v 1.87 2006/09/06 17:26:24 sdlime Exp $
  *
  * Name:     mapwfs.c
  * Project:  MapServer
@@ -29,8 +29,48 @@
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  * $Log: mapwfs.c,v $
- * Revision 1.73.2.1  2006/03/05 20:54:52  assefa
- * Add the possiblity to set wfs_maxfeatures to 0 (Bug 1678).
+ * Revision 1.87  2006/09/06 17:26:24  sdlime
+ * Removed advertising GML3 support in WFS 1.0.0 getCapabilities output.
+ *
+ * Revision 1.86  2006/08/22 04:45:06  sdlime
+ * Fixed a bug that did not allow for seperate metadata namespaces to be used for WMS vs. WFS for GML transformations (e.g. WFS_GEOMETRIES, WMS_GEOMETRIES).
+ *
+ * Revision 1.85  2006/08/14 21:52:20  sdlime
+ * Updated WFS GetFeature to reference external namespaces if available.
+ *
+ * Revision 1.84  2006/08/14 21:15:00  sdlime
+ * Added support to WFS  DescribeFeatureType for external schemas (initial coding).
+ *
+ * Revision 1.83  2006/08/14 20:19:54  dan
+ * Produce a warning in GetCapabilities if ???_featureid not set (bug 1782)
+ *
+ * Revision 1.82  2006/08/14 18:47:28  sdlime
+ * Some more implementation of external application schema support.
+ *
+ * Revision 1.81  2006/08/11 21:44:48  sdlime
+ * Added namespace reading (using is the hard part) to WFS server.
+ *
+ * Revision 1.80  2006/04/08 05:58:48  frank
+ * fix various memory leaks
+ *
+ * Revision 1.79  2006/04/08 03:38:18  frank
+ * Ensure that an error in FLTApplyFilterToLayer() is reported as a
+ * WFS exception.
+ *
+ * Revision 1.78  2006/03/30 21:43:47  sdlime
+ * Upgraded GML 3 version from 3.1.0 to 3.1.1 based on discussions with Tom Kralidis.
+ *
+ * Revision 1.77  2006/03/22 21:40:47  sdlime
+ * Added support to allow a service provider (WFS or WMS) tonot expose feature geometries. (bug 1718)
+ *
+ * Revision 1.76  2006/03/15 22:54:07  sdlime
+ * Fixed my brain-dead gml_group schema writing support.
+ *
+ * Revision 1.75  2006/02/24 02:55:03  assefa
+ * Add the possiblity to set wfs_maxfeatures to 0 (Bug 1678)
+ *
+ * Revision 1.74  2006/02/17 22:59:47  sdlime
+ * Updated WFS schema production code to ignore items with templates defined since we can't readily produce schema elements for them.
  *
  * Revision 1.73  2005/12/27 17:36:27  sdlime
  * Fixed a typo in msWFSGetGeometryType().
@@ -80,7 +120,7 @@
 
 #include "map.h"
 
-MS_CVSID("$Id: mapwfs.c,v 1.73.2.1 2006/03/05 20:54:52 assefa Exp $")
+MS_CVSID("$Id: mapwfs.c,v 1.87 2006/09/06 17:26:24 sdlime Exp $")
 
 #if defined(USE_WFS_SVR)
 
@@ -316,6 +356,11 @@ int msWFSDumpLayer(mapObj *map, layerObj *lp)
                      MS_TRUE, MS_FALSE, MS_FALSE, MS_TRUE, MS_TRUE, 
                      NULL, NULL, NULL, NULL, NULL, "        ");
 
+   if (msOWSLookupMetadata(&(lp->metadata), "OFG", "featureid") == NULL)
+   {
+       msIO_fprintf(stdout, "<!-- WARNING: Required Feature Id attribute (fid) not specified for this feature type. Make sure you set one of wfs_featureid, ows_feature_id or gml_featureid metadata. -->\n");
+   }
+
    msIO_printf("    </FeatureType>\n");
       
    return MS_SUCCESS;
@@ -405,10 +450,13 @@ int msWFSGetCapabilities(mapObj *map, const char *wmtver, cgiRequestObj *req)
   msIO_printf("  <Request>\n");
   msWFSPrintRequestCap(wmtver, "GetCapabilities", script_url_encoded, 
                        NULL, NULL);
-  msWFSPrintRequestCap(wmtver, "DescribeFeatureType", script_url_encoded, 
-                       "SchemaDescriptionLanguage", "XMLSCHEMA", "SFE_XMLSCHEMA", NULL);
-  msWFSPrintRequestCap(wmtver, "GetFeature", script_url_encoded, 
-                       "ResultFormat", "GML2", "GML3", NULL);
+  /* msWFSPrintRequestCap(wmtver, "DescribeFeatureType", script_url_encoded, "SchemaDescriptionLanguage", "XMLSCHEMA", "SFE_XMLSCHEMA", NULL); */
+  /* msWFSPrintRequestCap(wmtver, "GetFeature", script_url_encoded, "ResultFormat", "GML2", "GML3", NULL); */
+
+  /* don't advertise the GML3 or GML for SFE support */
+  msWFSPrintRequestCap(wmtver, "DescribeFeatureType", script_url_encoded, "SchemaDescriptionLanguage", "XMLSCHEMA", NULL);
+  msWFSPrintRequestCap(wmtver, "GetFeature", script_url_encoded, "ResultFormat", "GML2", NULL);
+
   msIO_printf("  </Request>\n");
   msIO_printf("</Capability>\n\n");
 
@@ -533,6 +581,7 @@ static void msWFSWriteGeometryElement(FILE *stream, gmlGeometryListObj *geometry
   gmlGeometryObj *geometry=NULL;
 
   if(!stream || !tab) return;
+  if(geometryList && geometryList->numgeometries == 1 && strcasecmp(geometryList->geometries[0].name, "none") == 0) return;
 
   if(!geometryList || geometryList->numgeometries == 0) { /* write a default container */
     msIO_fprintf(stream, "%s<element name=\"%s\" type=\"gml:GeometryPropertyType\" minOccurs=\"0\" maxOccurs=\"1\"/>\n", tab, OWS_GML_DEFAULT_GEOMETRY_NAME);
@@ -542,19 +591,19 @@ static void msWFSWriteGeometryElement(FILE *stream, gmlGeometryListObj *geometry
       geometry = &(geometryList->geometries[0]);
       msIO_fprintf(stream, "%s<element name=\"%s\" type=\"gml:%s\" minOccurs=\"%d\"", tab, geometry->name, msWFSGetGeometryType(geometry->type, outputformat), geometry->occurmin);
       if(geometry->occurmax == OWS_GML_OCCUR_UNBOUNDED)
-	msIO_fprintf(stream, " maxOccurs=\"UNBOUNDED\"/>\n");
+        msIO_fprintf(stream, " maxOccurs=\"unbounded\"/>\n");
       else
-	msIO_fprintf(stream, " maxOccurs=\"%d\"/>\n", geometry->occurmax);
+        msIO_fprintf(stream, " maxOccurs=\"%d\"/>\n", geometry->occurmax);
     } else {
       msIO_fprintf(stream, "%s<choice>\n", tab);
       for(i=0; i<geometryList->numgeometries; i++) {
-	geometry = &(geometryList->geometries[i]);
+        geometry = &(geometryList->geometries[i]);
 	
-	msIO_fprintf(stream, "  %s<element name=\"%s\" type=\"gml:%s\" minOccurs=\"%d\"", tab, geometry->name, msWFSGetGeometryType(geometry->type, outputformat), geometry->occurmin);
-	if(geometry->occurmax == OWS_GML_OCCUR_UNBOUNDED)
-	  msIO_fprintf(stream, " maxOccurs=\"UNBOUNDED\"/>\n");
-	else
-	  msIO_fprintf(stream, " maxOccurs=\"%d\"/>\n", geometry->occurmax);	
+      msIO_fprintf(stream, "  %s<element name=\"%s\" type=\"gml:%s\" minOccurs=\"%d\"", tab, geometry->name, msWFSGetGeometryType(geometry->type, outputformat), geometry->occurmin);
+      if(geometry->occurmax == OWS_GML_OCCUR_UNBOUNDED)
+        msIO_fprintf(stream, " maxOccurs=\"unbounded\"/>\n");
+      else
+	      msIO_fprintf(stream, " maxOccurs=\"%d\"/>\n", geometry->occurmax);	
       }
       msIO_fprintf(stream, "%s</choice>\n", tab);
     }
@@ -570,6 +619,7 @@ static void msWFSWriteItemElement(FILE *stream, gmlItemObj *item, const char *ta
 
   if(!stream || !item || !tab) return;
   if(!item->visible) return; /* not exposing this attribute */
+	if(item->template) return; /* can't adequately deal with templated items yet */
 
   if(item->alias) /* TODO: what about name spaces embedded in the alias? */
     element_name = item->alias;
@@ -598,7 +648,17 @@ static void msWFSWriteConstantElement(FILE *stream, gmlConstantObj *constant, co
   return;
 }
 
-static void msWFSWriteGroupElement(FILE *stream, gmlGroupObj *group, gmlItemListObj *itemList, gmlConstantListObj *constantList, const char *tab)
+static void msWFSWriteGroupElement(FILE *stream, gmlGroupObj *group, const char *tab, const char *namespace)
+{
+  if(group->type)
+    msIO_fprintf(stream, "%s<element name=\"%s\" type=\"%s:%s\"/>\n", tab, group->name, namespace, group->type);
+  else
+    msIO_fprintf(stream, "%s<element name=\"%s\" type=\"%s:%sType\"/>\n", tab, group->name, namespace, group->name);
+
+	return;
+}
+
+static void msWFSWriteGroupElementType(FILE *stream, gmlGroupObj *group, gmlItemListObj *itemList, gmlConstantListObj *constantList, const char *tab)
 {
   int i, j;
   char *element_tab;
@@ -609,40 +669,36 @@ static void msWFSWriteGroupElement(FILE *stream, gmlGroupObj *group, gmlItemList
   /* setup the element tab */
   element_tab = (char *) malloc(sizeof(char)*strlen(tab)+3);
   if(!element_tab) return;
-  sprintf(element_tab, "%s      ", tab);
+  sprintf(element_tab, "%s    ", tab);
 
-  if(group->type) {
-    msIO_fprintf(stream, "%s<element name=\"%s\" type=\"%s\">\n", tab, group->name, group->type);
-    msIO_fprintf(stream, "%s  <complexType name=\"%s\">\n", tab, group->type);
-  } else {
-    msIO_fprintf(stream, "%s<element name=\"%s\" type=\"%sType\">\n", tab, group->name, group->name);     
-    msIO_fprintf(stream, "%s  <complexType name=\"%sType\">\n", tab, group->name);
-  }
+  if(group->type)
+    msIO_fprintf(stream, "%s<complexType name=\"%s\">\n", tab, group->type);
+  else
+    msIO_fprintf(stream, "%s<complexType name=\"%sType\">\n", tab, group->name);
   
-  msIO_fprintf(stream, "%s    <sequence>\n", tab);
+  msIO_fprintf(stream, "%s  <sequence>\n", tab);
 
   /* now the items/constants (e.g. elements) in the group */ 
   for(i=0; i<group->numitems; i++) {
     for(j=0; j<constantList->numconstants; j++) { /* find the right gmlConstantObj */
       constant = &(constantList->constants[j]);
       if(strcasecmp(constant->name, group->items[i]) == 0) { 
-	msWFSWriteConstantElement(stream, constant, element_tab);
-	break;
+        msWFSWriteConstantElement(stream, constant, element_tab);
+	      break;
       }
     }
     if(j != constantList->numconstants) continue; /* found this item */
     for(j=0; j<itemList->numitems; j++) { /* find the right gmlItemObj */
       item = &(itemList->items[j]);
       if(strcasecmp(item->name, group->items[i]) == 0) { 
-	msWFSWriteItemElement(stream, item, element_tab);
-	break;
+        msWFSWriteItemElement(stream, item, element_tab);
+        break;
       }
     }
   }
   
-  msIO_fprintf(stream, "%s    </sequence>\n", tab);
-  msIO_fprintf(stream, "%s  </complexType>\n", tab);
-  msIO_fprintf(stream, "%s</element>\n", tab);
+  msIO_fprintf(stream, "%s  </sequence>\n", tab);
+  msIO_fprintf(stream, "%s</complexType>\n", tab);
 
   return;
 }
@@ -665,7 +721,9 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
   char *encoded_name = NULL, *encoded;
   
   int outputformat = OWS_DEFAULT_SCHEMA; /* default output is GML 2.1 compliant schema*/
-  
+
+  gmlNamespaceListObj *namespaceList=NULL; /* for external application schema support */
+
   if(paramsObj->pszTypeName && numlayers == 0) {
     /* Parse comma-delimited list of type names (layers) */
     /*  */
@@ -676,16 +734,16 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
       /* strip namespace if there is one :ex TYPENAME=cdf:Other */
       tokens = split(layers[0], ':', &n);
       if (tokens && n==2 && msGetLayerIndex(map, layers[0]) < 0) {
-	msFreeCharArray(tokens, n);
-	for (i=0; i<numlayers; i++) {
-	  tokens = split(layers[i], ':', &n);
-	  if (tokens && n==2) {
-	    free(layers[i]);
-	    layers[i] = strdup(tokens[1]);
-	  }
-	  if (tokens)
-	    msFreeCharArray(tokens, n);
-	}
+        msFreeCharArray(tokens, n);
+        for (i=0; i<numlayers; i++) {
+            tokens = split(layers[i], ':', &n);
+            if (tokens && n==2) {
+                free(layers[i]);
+                layers[i] = strdup(tokens[1]);
+            }
+            if (tokens)
+                msFreeCharArray(tokens, n);
+        }
       }
     }
   } 
@@ -696,9 +754,7 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
     else if(strcasecmp(paramsObj->pszOutputFormat, "SFE_XMLSCHEMA") == 0)
       outputformat = OWS_SFE_SCHEMA;
     else {
-      msSetError(MS_WFSERR, 
-		 "Unsupported DescribeFeatureType outputFormat (%s).", 
-		 "msWFSDescribeFeatureType()", paramsObj->pszOutputFormat);
+      msSetError(MS_WFSERR, "Unsupported DescribeFeatureType outputFormat (%s).", "msWFSDescribeFeatureType()", paramsObj->pszOutputFormat);
       return msWFSException(map, paramsObj->pszVersion);
     }
   }
@@ -707,14 +763,17 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
   if (numlayers > 0) {
     for (i=0; i<numlayers; i++) {
       if (msGetLayerIndex(map, layers[i]) < 0) {
-	msSetError(MS_WFSERR, 
-		   "Invalid typename (%s).", 
-		   "msWFSDescribeFeatureType()", layers[i]);/* paramsObj->pszTypeName); */
-	return msWFSException(map, paramsObj->pszVersion);
+	      msSetError(MS_WFSERR, "Invalid typename (%s).", "msWFSDescribeFeatureType()", layers[i]);/* paramsObj->pszTypeName); */
+        return msWFSException(map, paramsObj->pszVersion);
       }
     }
   }
-            
+
+  /*
+  ** retrieve any necessary external namespace/schema configuration information
+  */
+  namespaceList = msGMLGetNamespaces(&(map->web), "OFG");
+
   /*
   ** DescribeFeatureType response
   */
@@ -739,25 +798,50 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
 		"   xmlns:ogc=\"http://www.opengis.net/ogc\"\n"
 		"   xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"\n"
 		"   xmlns=\"http://www.w3.org/2001/XMLSchema\"\n"
-		"   xmlns:gml=\"http://www.opengis.net/gml\"\n"
-		"   elementFormDefault=\"qualified\" version=\"0.1\" >\n", 
+		"   xmlns:gml=\"http://www.opengis.net/gml\"\n",  
 		user_namespace_uri_encoded, user_namespace_prefix,  user_namespace_uri_encoded);
 
+  /* any additional namespaces */
+  for(i=0; i<namespaceList->numnamespaces; i++) {
+    if(namespaceList->namespaces[i].uri) {
+      char *uri_encoded=NULL;
+
+      uri_encoded = msEncodeHTMLEntities(namespaceList->namespaces[i].uri);
+      msIO_printf("   xmlns:%s=\"%s\" \n", namespaceList->namespaces[i].prefix, uri_encoded);
+      msFree(uri_encoded);
+    }
+  }
+
+  msIO_printf("   elementFormDefault=\"qualified\" version=\"0.1\" >\n");
+
   encoded = msEncodeHTMLEntities( msOWSGetSchemasLocation(map) );
-  if(outputformat == OWS_SFE_SCHEMA) /* reference GML 3.1.0 schema */
+  if(outputformat == OWS_SFE_SCHEMA) /* reference GML 3.1.1 schema */
     msIO_printf("\n  <import namespace=\"http://www.opengis.net/gml\"\n"
-		"          schemaLocation=\"%s/gml/3.1.0/base/feature.xsd\" />\n", encoded);
+		"          schemaLocation=\"%s/gml/3.1.1/base/feature.xsd\" />\n", encoded);
   else /* default GML 2.1.x schema */
     msIO_printf("\n  <import namespace=\"http://www.opengis.net/gml\"\n"
 	      "          schemaLocation=\"%s/gml/2.1.2/feature.xsd\" />\n", encoded);
   msFree(encoded);
 
+  /* any additional namespace includes */
+    for(i=0; i<namespaceList->numnamespaces; i++) {
+    if(namespaceList->namespaces[i].uri && namespaceList->namespaces[i].schemalocation) {
+      char *schema_location_encoded=NULL, *uri_encoded=NULL;
+
+      uri_encoded = msEncodeHTMLEntities(namespaceList->namespaces[i].uri);
+      schema_location_encoded = msEncodeHTMLEntities(namespaceList->namespaces[i].schemalocation);
+      msIO_printf("\n  <import namespace=\"%s\"\n schemaLocation=\"%s\" />\n", uri_encoded, schema_location_encoded);
+      msFree(uri_encoded);
+      msFree(schema_location_encoded);
+    }
+  }
+
   /* output definition for the default feature container, can't use wfs:FeatureCollection with GML3 */
   if(outputformat == OWS_SFE_SCHEMA) {
     value = msOWSLookupMetadata(&(map->web.metadata), "FO", "feature_collection");
     if(value) collection_name = value;
     
-    msIO_printf("  <element name=\"%s\" type=\"%s:%sType\" substitutionGroup=\"gml:FeatureCollection\"/>\n", collection_name, user_namespace_prefix, collection_name);
+    msIO_printf("  <element name=\"%s\" type=\"%s:%sType\" substitutionGroup=\"gml:_FeatureCollection\"/>\n", collection_name, user_namespace_prefix, collection_name);
     msIO_printf("  <complexType name=\"%sType\">\n", collection_name);
     msIO_printf("    <complexContent>\n");
     msIO_printf("      <extension base=\"gml:AbstractFeatureCollectionType\">\n");
@@ -778,84 +862,104 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
 
     for (j=0; j<numlayers && !bFound; j++) {
       if ( lp->name && strcasecmp(lp->name, layers[j]) == 0)
-	bFound = 1;
+      bFound = 1;
     }
 
     if ((numlayers == 0 || bFound) && msWFSIsLayerSupported(lp)) {
 
       /*
-      ** OK, describe this layer
+      ** OK, describe this layer IF you can open it and retrieve items
       */
-      /* value = msOWSLookupMetadata(&(lp->metadata), "OFG", "layername"); */      
-      encoded_name = msEncodeHTMLEntities( lp->name );
-      if (user_namespace_prefix)
-	msIO_printf("\n"
-		    "  <element name=\"%s\" \n"
-		    "           type=\"%s:%sType\" \n"
-		    "           substitutionGroup=\"gml:_Feature\" />\n\n",
-		    encoded_name, user_namespace_prefix, encoded_name);
-      else
-	msIO_printf("\n"
-		    "  <element name=\"%s\" \n"
-		    "           type=\"ms:%sType\" \n"
-		    "           substitutionGroup=\"gml:_Feature\" />\n\n",
-		    encoded_name, encoded_name);
-      
-      msIO_printf("  <complexType name=\"%sType\">\n", encoded_name);
-      msIO_printf("    <complexContent>\n");
-      msIO_printf("      <extension base=\"gml:AbstractFeatureType\">\n");
-      msIO_printf("        <sequence>\n");
-      
       if (msLayerOpen(lp) == MS_SUCCESS) {
-	if (msLayerGetItems(lp) == MS_SUCCESS) {	  
-	  int k;
-	  gmlGroupListObj *groupList=NULL;
-	  gmlItemListObj *itemList=NULL;
+        if (msLayerGetItems(lp) == MS_SUCCESS) {
+          int k;
+          gmlGroupListObj *groupList=NULL;
+          gmlItemListObj *itemList=NULL;
           gmlConstantListObj *constantList=NULL;
-	  gmlGeometryListObj *geometryList=NULL;
-	  gmlItemObj *item=NULL;
-	  gmlConstantObj *constant=NULL;
+          gmlGeometryListObj *geometryList=NULL;
+          gmlItemObj *item=NULL;
+          gmlConstantObj *constant=NULL;
 
-	  itemList = msGMLGetItems(lp); /* GML-related metadata */
-	  constantList = msGMLGetConstants(lp);
-	  groupList = msGMLGetGroups(lp);
-	  geometryList = msGMLGetGeometries(lp);
+          const char *layer_namespace_prefix;
+          char *encoded_type=NULL;
 
-	  /* write the geometry schema element(s) */
-	  msWFSWriteGeometryElement(stdout, geometryList, outputformat, "          ");
+          itemList = msGMLGetItems(lp, "OFG"); /* GML-related metadata */
+          constantList = msGMLGetConstants(lp, "OFG");
+          groupList = msGMLGetGroups(lp, "OFG");
+          geometryList = msGMLGetGeometries(lp, "OFG");
 
-	  /* write the constant-based schema elements */
-	  for(k=0; k<constantList->numconstants; k++) {
-	    constant = &(constantList->constants[k]);  
-	    if(msItemInGroups(constant->name, groupList) == MS_FALSE) 
-	      msWFSWriteConstantElement(stdout, constant, "          ");
-	  }
+          value = msOWSLookupMetadata(&(lp->metadata), "OFG", "namespace_prefix");
+          if(value) 
+            layer_namespace_prefix = value;
+          else
+            layer_namespace_prefix = user_namespace_prefix;
 
-	  /* write the item-based schema elements */
-	  for(k=0; k<itemList->numitems; k++) {
-	    item = &(itemList->items[k]);  
-	    if(msItemInGroups(item->name, groupList) == MS_FALSE) 
-	      msWFSWriteItemElement(stdout, item, "          ");
-	  }
+          /* value = msOWSLookupMetadata(&(lp->metadata), "OFG", "layername"); */      
+          encoded_name = msEncodeHTMLEntities( lp->name );          
+          value = msOWSLookupMetadata(&(lp->metadata), "OFG", "layer_type");
+          if(value) {
+            encoded_type = msEncodeHTMLEntities(value);
+            msIO_printf("\n"
+                      "  <element name=\"%s\" \n"
+                      "           type=\"%s:%s\" \n"
+                      "           substitutionGroup=\"gml:_Feature\" />\n\n",
+                      encoded_name, layer_namespace_prefix, encoded_type);
+            msFree(encoded_type);
+          } else
+            msIO_printf("\n"
+                      "  <element name=\"%s\" \n"
+                      "           type=\"%s:%sType\" \n"
+                      "           substitutionGroup=\"gml:_Feature\" />\n\n",
+                      encoded_name, layer_namespace_prefix, encoded_name);
 
-	  for(k=0; k<groupList->numgroups; k++)
-	    msWFSWriteGroupElement(stdout, &(groupList->groups[k]), itemList, constantList, "          ");
+          if(strcmp(layer_namespace_prefix, user_namespace_prefix) != 0)
+						continue; /* the rest is defined in an external schema */
 
-	  msGMLFreeItems(itemList);
-	  msGMLFreeConstants(constantList);
-	  msGMLFreeGroups(groupList);
-	  msGMLFreeGeometries(geometryList);
-	}
+          msIO_printf("  <complexType name=\"%sType\">\n", encoded_name);
+          msIO_printf("    <complexContent>\n");
+          msIO_printf("      <extension base=\"gml:AbstractFeatureType\">\n");
+          msIO_printf("        <sequence>\n");
 
-	msLayerClose(lp);
+          /* write the geometry schema element(s) */
+          msWFSWriteGeometryElement(stdout, geometryList, outputformat, "          ");
+
+          /* write the constant-based schema elements */
+          for(k=0; k<constantList->numconstants; k++) {
+            constant = &(constantList->constants[k]);  
+            if(msItemInGroups(constant->name, groupList) == MS_FALSE) 
+              msWFSWriteConstantElement(stdout, constant, "          ");
+          }
+
+          /* write the item-based schema elements */
+          for(k=0; k<itemList->numitems; k++) {
+            item = &(itemList->items[k]);  
+            if(msItemInGroups(item->name, groupList) == MS_FALSE) 
+              msWFSWriteItemElement(stdout, item, "          ");
+          }
+
+          for(k=0; k<groupList->numgroups; k++)
+            msWFSWriteGroupElement(stdout, &(groupList->groups[k]), "          ", user_namespace_prefix);
+
+          msIO_printf("        </sequence>\n");
+          msIO_printf("      </extension>\n");
+          msIO_printf("    </complexContent>\n");
+          msIO_printf("  </complexType>\n");
+
+          /* any group types */
+          for(k=0; k<groupList->numgroups; k++)
+            msWFSWriteGroupElementType(stdout, &(groupList->groups[k]), itemList, constantList, "  ");
+
+          msGMLFreeItems(itemList);
+          msGMLFreeConstants(constantList);
+          msGMLFreeGroups(groupList);
+          msGMLFreeGeometries(geometryList);
+        }
+
+        msLayerClose(lp);
       } else {
-	msIO_printf("\n\n<!-- ERROR: Failed openinig layer %s -->\n\n", encoded_name);
+        msIO_printf("\n\n<!-- ERROR: Failed opening layer %s -->\n\n", encoded_name);
       }
-      
-      msIO_printf("        </sequence>\n");
-      msIO_printf("      </extension>\n");
-      msIO_printf("    </complexContent>\n");
-      msIO_printf("  </complexType>\n");      
+          
     }
   }
   
@@ -867,10 +971,9 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
   msFree(encoded_name);
   msFree(user_namespace_uri_encoded);
   
-  if (layers)
+  if(layers)
     msFreeCharArray(layers, numlayers);
-  
-  
+    
   return MS_SUCCESS;
 }
 
@@ -904,7 +1007,9 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 
   const char *output_schema_format = "XMLSCHEMA";  
   int outputformat = OWS_GML2; /* default output is GML 2.1 */
-  
+    
+  gmlNamespaceListObj *namespaceList=NULL; /* for external application schema support */
+
   /* Default filter is map extents */
   bbox = map->extent;
   
@@ -964,6 +1069,8 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 	  msFreeCharArray(tokens, n);
       }
     }
+    else
+      msFreeCharArray(tokens, n);
 
     pszMapSRS = msOWSGetEPSGProj(&(map->projection), &(map->web.metadata), "FO", MS_TRUE);
 
@@ -1173,9 +1280,14 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 		     "msWFSGetFeature()", pszFilter);
 	  return msWFSException(map, paramsObj->pszVersion);
 	}
-	FLTApplyFilterToLayer(psNode, map, iLayerIndex, MS_FALSE);
-	
+
+	if( FLTApplyFilterToLayer(psNode, map, iLayerIndex, MS_FALSE) != MS_SUCCESS )
+	  return msWFSException(map, paramsObj->pszVersion);
+
+        FLTFreeFilterEncodingNode( psNode );
+        psNode = NULL;
       }
+
       if (paszFilter)
 	free(paszFilter);
     }/* end if filter set */
@@ -1224,6 +1336,10 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
       return msWFSException(map, paramsObj->pszVersion);
     }
 
+    /*
+    ** retrieve any necessary external namespace/schema configuration information
+    */
+    namespaceList = msGMLGetNamespaces(&(map->web), "OFG");
 
     msIO_printf("Content-type: text/xml%c%c",10,10);
 
@@ -1250,40 +1366,67 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
     encoded = msEncodeHTMLEntities( paramsObj->pszVersion );
     encoded_typename = msEncodeHTMLEntities( typename );
     encoded_schema = msEncodeHTMLEntities( msOWSGetSchemasLocation(map) );
-    if(outputformat == OWS_GML2) /* use a wfs:FeatureCollection */
+    if(outputformat == OWS_GML2) { /* use a wfs:FeatureCollection */
       msIO_printf("<wfs:FeatureCollection\n"
 		  "   xmlns:%s=\"%s\"\n"  
 		  "   xmlns:wfs=\"http://www.opengis.net/wfs\"\n"
 		  "   xmlns:gml=\"http://www.opengis.net/gml\"\n"
 		  "   xmlns:ogc=\"http://www.opengis.net/ogc\"\n"
-		  "   xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
-		  "   xsi:schemaLocation=\"http://www.opengis.net/wfs %s/wfs/%s/WFS-basic.xsd \n"
+      "   xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n",
+      user_namespace_prefix, user_namespace_uri_encoded);
+
+      /* any additional namespaces */
+      for(i=0; i<namespaceList->numnamespaces; i++) {
+        if(namespaceList->namespaces[i].uri) {
+          char *uri_encoded=NULL;
+
+          uri_encoded = msEncodeHTMLEntities(namespaceList->namespaces[i].uri);
+          msIO_printf("   xmlns:%s=\"%s\" \n", namespaceList->namespaces[i].prefix, uri_encoded);
+          msFree(uri_encoded);
+        }
+      }
+
+
+		  msIO_printf("   xsi:schemaLocation=\"http://www.opengis.net/wfs %s/wfs/%s/WFS-basic.xsd \n"
 		  "                       %s %sSERVICE=WFS&amp;VERSION=%s&amp;REQUEST=DescribeFeatureType&amp;TYPENAME=%s&amp;OUTPUTFORMAT=%s\">\n",
-		  user_namespace_prefix, user_namespace_uri_encoded,
 		  encoded_schema, encoded, user_namespace_uri_encoded, 
 		  script_url_encoded, encoded, encoded_typename, output_schema_format);
-    else 
+    } else { 
       msIO_printf("<%s:%s\n"
+      "   version=\"1.0.0\"\n"
 		  "   xmlns:%s=\"%s\"\n"  
 		  "   xmlns:gml=\"http://www.opengis.net/gml\"\n"
 		  "   xmlns:ogc=\"http://www.opengis.net/ogc\"\n"
-		  "   xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
-		  "   xsi:schemaLocation=\"%s %sSERVICE=WFS&amp;VERSION=%s&amp;REQUEST=DescribeFeatureType&amp;TYPENAME=%s&amp;OUTPUTFORMAT=%s\">\n",
-		  user_namespace_prefix, collection_name,
-		  user_namespace_prefix, user_namespace_uri_encoded,
+      "   xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n",
+      user_namespace_prefix, collection_name, user_namespace_prefix, user_namespace_uri_encoded);
+
+      /* any additional namespaces */
+      for(i=0; i<namespaceList->numnamespaces; i++) {
+        if(namespaceList->namespaces[i].uri) {
+          char *uri_encoded=NULL;
+
+          uri_encoded = msEncodeHTMLEntities(namespaceList->namespaces[i].uri);
+          msIO_printf("   xmlns:%s=\"%s\" \n", namespaceList->namespaces[i].prefix, uri_encoded);
+          msFree(uri_encoded);
+        }
+      }
+
+		  msIO_printf("   xsi:schemaLocation=\"%s %sSERVICE=WFS&amp;VERSION=%s&amp;REQUEST=DescribeFeatureType&amp;TYPENAME=%s&amp;OUTPUTFORMAT=%s\">\n",
 		  user_namespace_uri_encoded, script_url_encoded, encoded, encoded_typename, output_schema_format);
+    }
 
     msFree(encoded);
     msFree(encoded_schema);
     msFree(encoded_typename);
 
+    /* handle case of maxfeatures = 0 */
     if(maxfeatures != 0)
       msGMLWriteWFSQuery(map, stdout, maxfeatures, pszNameSpace, outputformat);
     
     /* if no results where written (TODO: this needs to be GML2/3 specific I imagine */
     for(i=0; i<map->numlayers; i++) {
       if (map->layers[i].resultcache && map->layers[i].resultcache->numresults > 0)
-	break;
+        break;
     }
     if ((i==map->numlayers) || (maxfeatures == 0)) {
       msIO_printf("   <gml:boundedBy>\n"); 
@@ -1306,7 +1449,6 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
       free(pszNameSpace);
     msFree(user_namespace_uri_encoded);
 
-
     return MS_SUCCESS;
 }
 
