@@ -27,6 +27,9 @@
  **********************************************************************
  *
  * $Log: mapwfslayer.c,v $
+ * Revision 1.37  2006/05/19 20:53:27  dan
+ * Use lp->layerinfo for OGR connections (instead of ogrlayerinfo) (bug 331)
+ *
  * Revision 1.36  2005/10/28 01:09:42  jani
  * MS RFC 3: Layer vtable architecture (bug 1477)
  *
@@ -154,7 +157,7 @@
 #include <process.h>
 #endif
 
-MS_CVSID("$Id: mapwfslayer.c,v 1.36 2005/10/28 01:09:42 jani Exp $")
+MS_CVSID("$Id: mapwfslayer.c,v 1.37 2006/05/19 20:53:27 dan Exp $")
 
 #define WFS_V_0_0_14  14
 #define WFS_V_1_0_0  100
@@ -1175,7 +1178,7 @@ int msWFSLayerClose(layerObj *lp)
 /* ------------------------------------------------------------------
  * Cleanup OGR connection
  * ------------------------------------------------------------------ */
-    if (lp->ogrlayerinfo)
+    if (lp->layerinfo)
         msOGRLayerClose(lp);
 
 /* ------------------------------------------------------------------
