@@ -27,7 +27,13 @@
  ******************************************************************************
  *
  * $Log: shp2img.c,v $
- * Revision 1.17.4.1  2006/02/01 04:10:52  sdlime
+ * Revision 1.20  2006/05/12 20:59:07  frank
+ * Removed -t switch, it doesn't work.
+ *
+ * Revision 1.19  2006/02/10 05:51:45  sdlime
+ * Applied patch to resolve issue with layers with no names if -l option is specified. (bug 1655)
+ *
+ * Revision 1.18  2006/02/01 04:08:35  sdlime
  * Converted strncmp to strcmp in shp2img.c to avoid parameter confusion and for consistency. (bug 1635)
  *
  * Revision 1.17  2005/06/14 16:03:35  dan
@@ -53,7 +59,7 @@
 
 #include "map.h"
 
-MS_CVSID("$Id: shp2img.c,v 1.17.4.1 2006/02/01 04:10:52 sdlime Exp $")
+MS_CVSID("$Id: shp2img.c,v 1.20 2006/05/12 20:59:07 frank Exp $")
 
 int main(int argc, char *argv[])
 {
@@ -76,11 +82,10 @@ int main(int argc, char *argv[])
   if( argc < 3 ) {
     fprintf(stdout,
             "Syntax: shp2img -m [mapfile] -o [image] -e minx miny maxx maxy\n"
-            "                -t -l [layers] -i [format]\n");
+            "                -l [layers] -i [format]\n");
 
     fprintf(stdout,"  -m mapfile: Map file to operate on - required.\n" );
     fprintf(stdout,"  -i format: Override the IMAGETYPE value to pick output format.\n" );
-    fprintf(stdout,"  -t: enable transparency\n" );
     fprintf(stdout,"  -o image: output filename (stdout if not provided)\n");
     fprintf(stdout,"  -e minx miny maxx maxy: extents to render - optional\n");
     fprintf(stdout,"  -l layers: layers to enable - optional\n" );
@@ -158,9 +163,6 @@ int main(int argc, char *argv[])
       i+=2;
     }
 
-    if(strcmp(argv[i], "-t") == 0) /* transparency */
-      map->transparent = MS_ON;
-    
     if(strcmp(argv[i], "-all_debug") == 0) /* debug */
     {
         int debug_level = atoi(argv[++i]);
@@ -213,7 +215,7 @@ int main(int argc, char *argv[])
 	else {
 	  map->layers[j].status = MS_OFF;
 	  for(k=0; k<num_layers; k++) {
-	    if(strcmp(map->layers[j].name, layers[k]) == 0) {
+	    if(map->layers[j].name && strcmp(map->layers[j].name, layers[k]) == 0) {
 	      map->layers[j].status = MS_ON;
 	      break;
 	    }
