@@ -5,7 +5,7 @@
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,13 @@
  ******************************************************************************
  *
  * $Log: shptreevis.c,v $
+ * Revision 1.14  2005/06/14 16:03:35  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.13  2005/04/26 03:37:50  frank
+ * Fixed setting of this_rec from hDBF->nRecords, was screwing up dbf file.
+ * http://mapserver.gis.umn.edu/bugs/show_bug.cgi?id=1332
+ *
  * Revision 1.12  2004/10/21 04:30:55  frank
  * Added standardized headers.  Added MS_CVSID().
  *
@@ -40,7 +47,7 @@
 #endif
 #include <stdlib.h>
 
-MS_CVSID("$Id: shptreevis.c,v 1.12 2004/10/21 04:30:55 frank Exp $")
+MS_CVSID("$Id: shptreevis.c,v 1.14 2005/06/14 16:03:35 dan Exp $")
 
 #ifdef SHPT_POLYGON
    #undef MAPSERVER
@@ -183,14 +190,7 @@ int main( int argc, char ** argv )
       if (node )
       {
 
-#ifdef MAPSERVER
-        this_rec = hDBF->nRecords - 1;
-#else
-        this_rec = hDBF->nRecords;	
-/*      Shapelib currently works this way
- *      Map server is based on an old version of shapelib      
- */
-#endif
+        this_rec = hDBF->nRecords;
 
 #ifdef  MAPSERVER
         msDBFWriteIntegerAttribute( hDBF, this_rec, 0, node->numshapes);
