@@ -27,8 +27,8 @@
  ******************************************************************************
  *
  * $Log: sortshp.c,v $
- * Revision 1.5.2.1  2005/07/15 14:01:52  frank
- * bug 1418: fixed big memory leak in sortshp.c
+ * Revision 1.6  2005/07/15 13:57:50  frank
+ * bug 1418: fixed memory leak of shapes
  *
  * Revision 1.5  2005/06/14 16:03:35  dan
  * Updated copyright date to 2005
@@ -48,7 +48,7 @@
 
 #include "map.h"
 
-MS_CVSID("$Id: sortshp.c,v 1.5.2.1 2005/07/15 14:01:52 frank Exp $")
+MS_CVSID("$Id: sortshp.c,v 1.6 2005/07/15 13:57:50 frank Exp $")
 
 typedef struct {
   double number;
