@@ -5,7 +5,7 @@
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,12 @@
  ******************************************************************************
  *
  * $Log: sym2img.c,v $
+ * Revision 1.9  2005/06/14 16:03:35  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.8  2005/02/18 03:06:48  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
  * Revision 1.7  2004/10/21 04:30:55  frank
  * Added standardized headers.  Added MS_CVSID().
  *
@@ -37,13 +43,13 @@
 
 #include "map.h"
 
-MS_CVSID("$Id: sym2img.c,v 1.7 2004/10/21 04:30:55 frank Exp $")
+MS_CVSID("$Id: sym2img.c,v 1.9 2005/06/14 16:03:35 dan Exp $")
 
 #define MAXCOLORS 256
 #define CELLSIZE 100
 #define NCOLS 5
 #define RATIO .5
-#define LBUF 5 // space to reserve around line symbols
+#define LBUF 5 /* space to reserve around line symbols */
 
 int main(int argc, char *argv[])
 {
