@@ -1,5 +1,5 @@
 /******************************************************************************
-* $Id: tile4ms.c,v 1.9 2005/02/18 03:06:48 dan Exp $
+* $Id: tile4ms.c,v 1.10 2006/02/09 05:06:25 frank Exp $
 *
 * tile4ms.mc  
 *
@@ -12,11 +12,18 @@
 * For use with Mapserv tiling capability.
 * Issues: resulting shape files do not display in ArcView.
 *
-* This code is based on shapelib API by Frank Warmerdam.
-* Released without restrictions except the disclaimer below
-* into custodianship of mapserv committers.
+* Copyright (c) 1996-2005 Regents of the University of Minnesota.
+*
+* Permission is hereby granted, free of charge, to any person obtaining a
+* copy of this software and associated documentation files (the "Software"),
+* to deal in the Software without restriction, including without limitation
+* the rights to use, copy, modify, merge, publish, distribute, sublicense,
+* and/or sell copies of the Software, and to permit persons to whom the
+* Software is furnished to do so, subject to the following conditions:
+*
+* The above copyright notice and this permission notice shall be included in 
+* all copies of this Software or works derived from this Software.
 *
-* Disclaimer:
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
@@ -25,16 +32,16 @@
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 ******************************************************************************
- *
- * This is a modified version of Herbie's program that works with MapServer's
- * shapelib and should be using the MapServer makefile.
- *
- */
+*
+* This is a modified version of Herbie's program that works with MapServer's
+* shapelib and should be using the MapServer makefile.
+*
+*/
 
 #include "map.h"
 #include <string.h>
 
-MS_CVSID("$Id: tile4ms.c,v 1.9 2005/02/18 03:06:48 dan Exp $")
+MS_CVSID("$Id: tile4ms.c,v 1.10 2006/02/09 05:06:25 frank Exp $")
 
 /***********************************************************************/
 int process_shapefiles(char *metaFileNameP, char *tileFileNameP, 
